import com.javageeks.classloader.FileSystemClassLoader;
import com.javageeks.gjas.ConfigurationInfo;
import com.javageeks.gjas.ServerManager;
import com.javageeks.gjas.Service;
import com.javageeks.gjas.services.ThreadedServer;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.Vector;

class ConsoleThread extends Thread
{
    public ConsoleThread()
    {
        setDaemon(true);
    }

    public void run()
    {
        try
        {
            // Set up
            BufferedReader in =
                new BufferedReader(
                    new InputStreamReader(System.in));

            System.out.print("ServerManager >");
            for (String line = in.readLine();
                !line.equals("quit");
                line = in.readLine())
            {
                ServerManager.instance().log(this.toString() +
                    ": '" + line + "'");
                if (line.trim().equals("shutdown"))
                {
                    ServerManager.instance().shutdown();
                    //return;
                }
                else if (line.trim().startsWith("start "))
                {
                    // Extract classname
                    String currentLine = line.substring(6, line.length());
                    
                    if (currentLine.indexOf(" ") < 0)
                    {
                        // Classname appeared by itself, so there are no
                        // additional args to parse
                        String classname = currentLine;

                        ServerManager.instance().log(
                            "ConsoleControlService.run(): " +
                            "Calling ServerManager.addService(" +
                            classname + ", null)");

                        // Create a new FileSystemClassLoader to
                        // load the Service
                        try
                        {
                            ClassLoader cl =
                                new FileSystemClassLoader(
                                    System.getProperty("user.dir"));
                            Service svc = 
                                (Service)cl.loadClass(classname).newInstance();
                            ServerManager.instance().addService(svc, null);
                        }
                        catch (Exception ex)
                        {
                            ServerManager.log(ex);
                        }
                    }
                    else
                    {
                        ServerManager.instance().log(
                            "ConsoleControlService.run: " +
                            "Any service started by the this service " +
                            "cannot have args; sorry.");
                    }
                }
                else if (line.trim().startsWith("list"))
                {
                    String[] svcs = ServerManager.instance().getServices();
                    
                    System.out.println("Services: {");
                    for (int i=0; i<svcs.length; i++)
                        System.out.println("    " + svcs[i]);
                    System.out.println("}");
                }
                else if (line.trim().startsWith("remove "))
                {
                    // Parse argument, confirm removal,
                    // call ServerManager.removeService()
                }
                else if (line.trim().startsWith("threads"))
                {
                    // List all threads running in the JVM
                    //
                    
                    // Find the ultimate ThreadGroup parent
                    ThreadGroup ancestor = Thread.currentThread().getThreadGroup();
                    while (ancestor.getParent() != null)
                        ancestor = ancestor.getParent();
                        
                    // List all threads
                    int ct = ancestor.activeCount();
                    ct += ct/2;
                    Thread[] array = new Thread[ct];
                    ancestor.enumerate(array, true);
                    
                    for (int i=0; i<array.length; i++)
                    {
                        if (array[i] != null)
                        {
                            String msg = array[i].toString();
                            msg += ": [";
                            if (array[i].isAlive())
                                msg += " ALIVE";
                            if (array[i].isDaemon())
                                msg += " DAEMON";
                            msg += " ]";
                            System.out.println(msg);
                        }
                    }
                }
                else
                {
                    System.out.println("Unrecognized command: " + line);
                }
                
                System.out.print("ServerManager >");
            }
        }
        catch (java.io.IOException IOEx)
        { }
    }
}    

/**
 * ConsoleControlService:
 */
public class ConsoleControlService extends ThreadedServer
    implements Runnable
{
    public void start(String[] args)
        throws Exception
    {
        setRunnable(this);
        
        super.start(args);
    }


    public void run()
    {
        ConsoleThread t = new ConsoleThread();
        t.start();
        
        try
        {
            t.join();
        }
        catch (InterruptedException intEx)
        {
            // Do nothing but return
            return;
        }
    }

    public ConfigurationInfo getConfigInfo()
    {
        return null;
    }
    public void setConfigInfo(ConfigurationInfo configInfo)
        throws Exception
    {
    }
}    