import java.io.*;

public class ExecService
    implements com.javageeks.gjas.Service
{
    // Internal data
    //
    private String m_state = STOPPED;
    private String m_cmdLine = null;

    
    public ExecService(String commandLine)
    {
        m_cmdLine = commandLine;
    }

    /**
     *
     */    
    public void start()
        throws Exception
    {
        // We're starting
        //
        m_state = STARTING;

        // We expect at least one argument--the text of the command-line
        // to fire off
        //        
        if (m_cmdLine == null)
        {
            m_state = STOPPED;
            throw new IllegalArgumentException();
        }
    
        // Start the Process, and capture its output
        //
        Process p = Runtime.getRuntime().exec(m_cmdLine);
        InputStream procOut = p.getInputStream();
        InputStream procErr = p.getErrorStream();

        while (true)
        {
            try
            {
                int exitVal = p.exitValue();
                
                // If we didn't throw an exception on that call, then
                // the Process has terminated. Capture what remaining
                // output might be in stdout or stderr, display it,
                // and return
                //

                // Capture and display stderr output
                //
                int errAvail = procErr.available();
                byte[] errBytes = new byte[errAvail];
                int bytesRead = procErr.read(errBytes);
                if (bytesRead > 0)
                {
                    String sb = new String(errBytes);
                    System.out.print(sb);

                    //err.print(sb);
                    //err.flush();
                }


                // Capture and display stderr output
                //
                int outAvail = procOut.available();
                byte[] outBytes = new byte[outAvail];
                bytesRead = procOut.read(outBytes);
                if (bytesRead > 0)
                {
                    String sb = new String(outBytes);
                    System.out.print(sb);
                    
                    //err.print(sb);
                    //err.flush();
                }
                
                break;
            }
            catch (IllegalThreadStateException ex)
            {
                // Not terminated yet, so display output
                //

                // Capture and display stderr output
                //
                int errAvail = procErr.available();
                byte[] errBytes = new byte[errAvail];
                int bytesRead = procErr.read(errBytes);
                if (bytesRead > 0)
                {
                    String sb = new String(errBytes);
                    System.out.print(sb);

                    //err.print(sb);
                    //err.flush();
                }


                // Capture and display stderr output
                //
                int outAvail = procOut.available();
                byte[] outBytes = new byte[outAvail];
                bytesRead = procOut.read(outBytes);
                if (bytesRead > 0)
                {
                    String sb = new String(outBytes);
                    System.out.print(sb);
                    
                    //err.print(sb);
                    //err.flush();
                }
            }
        }
        
        
        // We're running
        //
        m_state = RUNNING;
    }
    /**
     *
     */    
    public void stop()
        throws Exception
    {
        // We're stopping
        //
        m_state = STOPPING;
        
        
        // We've stopped
        //
        m_state = STOPPED;
    }
    /**
     *
     */    
    public void pause()
        throws Exception
    {
        // We're pausing
        //
        m_state = PAUSING;
        
        
        // We've paused
        //
        m_state = PAUSED;
    }
    /**
     *
     */    
    public void resume()
        throws Exception
    {
        // We're resuming
        //
        m_state = RESUMING;
        
        
        // We've started up again
        //
        m_state = RUNNING;
    }
        

    /**
     *
     */    
    public String getState()
    {
        return m_state;
    }


    /**
     *
     */    
    public String getInstanceID()
        throws Exception
    {
        return getClass() + ":1.0:" + System.currentTimeMillis();
    }
    
    
    /**
     * Return the ConfigurationInfo instance for this Service
     */
    public com.javageeks.gjas.ConfigProperties getConfigInfo()
    {
        return new com.javageeks.gjas.ConfigProperties();
    }
    /**
     * Set the ConfigurationInfo instance for this Service
     */
    public void setConfigInfo(com.javageeks.gjas.ConfigProperties configInfo)
    {
        // Do nothing
    }
}    