package Employee;

import java.util.*;


/**
 *
 */
class HashtablePerson
    implements IPerson, java.io.Serializable
{
    HashtablePerson()
    { }

    HashtablePerson(String fName, String mName, String lName,
                    String ssn)
    {
        m_firstName = fName;
        m_middleName = mName;
        m_lastName = lName;
        m_ssn = ssn;
    }

    /**
     *
     */
    public String getFirstName()
        throws BusinessLayerException
    {
        return m_firstName;
    }
    /**
     *
     */
    public void setFirstName(String fName)
        throws BusinessLayerException
    {
        m_firstName = fName;
    }
    

    /**
     *
     */
    public String getMiddleName()
        throws BusinessLayerException
    {
        return m_middleName;
    }
    /**
     *
     */
    public void setMiddleName(String mName)
        throws BusinessLayerException
    {
        m_middleName = mName;
    }


    /**
     *
     */
    public String getLastName()
        throws BusinessLayerException
    {
        return m_lastName;
    }
    /**
     *
     */
    public void setLastName(String lName)
        throws BusinessLayerException
    {
        m_lastName = lName;
    }


    /**
     *
     */
    public String getSSN()
        throws BusinessLayerException
    {
        return m_ssn;
    }
    /**
     *
     */
    public void setSSN(String ssn)
        throws BusinessLayerException
    {
        m_ssn = ssn;
    }

    
    /**
     *
     */
    public IContactInfo[] getContactInfo()
        throws BusinessLayerException
    {
        IContactInfo[] retArray = 
            new IContactInfo[m_contactInfo.size()];
        for (int i=0; i<retArray.length; i++)
            retArray[i] = (IContactInfo)m_contactInfo.elementAt(i);
        
        return retArray;
    }
    /**
     *
     */
    public void addEMail(String desc, String email)
        throws BusinessLayerException
    {
        HashtableEMail e = new HashtableEMail();
        e.m_desc = desc;
        e.m_email = email;
        m_contactInfo.addElement(e);
    }
    /**
     *
     */
    public void addAddress(String desc, String street, String city,
        String state, String zip, String country)
        throws BusinessLayerException
    {
        HashtableAddress addr = new HashtableAddress();
        addr.m_desc = desc;
        addr.m_street = street;
        addr.m_city = city;
        addr.m_state = state;
        addr.m_zip = zip;
        addr.m_country = country;
        m_contactInfo.addElement(addr);
    }
    /**
     *
     */
    public void addPhone(String desc, String prefix, String number,
        String suffix)
        throws BusinessLayerException
    {
        HashtablePhone phone = new HashtablePhone();
        phone.m_desc = desc;
        phone.m_prefix = prefix;
        phone.m_number = number;
        phone.m_suffix = suffix;
        m_contactInfo.addElement(phone);
    }
    /**
     *
     */
    public void removeContactInfo(IContactInfo contactInfo)
        throws BusinessLayerException
    {
        m_contactInfo.remove(contactInfo);
    }
    
    
    // Internal members
    private String m_firstName;
    private String m_middleName;
    private String m_lastName;
    private String m_ssn;
    private Vector m_contactInfo = new Vector();
}



/**
 *
 */
class HashtableEmployee extends HashtablePerson
    implements IEmployee, java.io.Serializable
{
    HashtableEmployee()
    {
        super();
    }
    
    HashtableEmployee(String fName, String mName, String lName,
                      String ssn, IPosition pos, int salary)
    {
        super(fName, mName, lName, ssn);
        m_position = pos;
        m_salary = salary;
    }


    /**
     *
     */
    public IPosition getPosition()
        throws BusinessLayerException
    {
        return m_position;
    }
    /*
     *
     */
    public void setPosition(IPosition position)
        throws BusinessLayerException
    {
        m_position = position;
    }
    
    /**
     *
     */
    public int getSalary()
        throws BusinessLayerException
    {
        return m_salary;
    }
    /**
     *
     */
    public void setSalary(int salary)
        throws BusinessLayerException, IllegalSalaryException
    {
        if (salary > getPosition().getMaxSalary())
            throw new IllegalSalaryException("Max salary is " +
                getPosition().getMaxSalary());
    
        m_salary = salary;
    }

    
    // Internal members
    private IPosition m_position;
    private int m_salary;
}



/**
 *
 */
class HashtableManager extends HashtableEmployee
    implements IManager, java.io.Serializable
{
    HashtableManager()
    {
        super();
    }
    
    HashtableManager(String fName, String mName, String lName,
                     String ssn, IPosition pos, int salary)
    {
        super(fName, mName, lName, ssn, pos, salary);
    }

    /**
     *
     */
    public IEmployee[] getEmployees()
        throws BusinessLayerException
    {
        IEmployee[] retArray = new IEmployee[m_employees.size()];
        for (int i=0; i<retArray.length; i++)
            retArray[i] = (IEmployee)m_employees.elementAt(i);
        return retArray;
    }
    /**
     *
     */
    public void addEmployee(IEmployee employee)
        throws BusinessLayerException
    {
        m_employees.addElement(employee);
    }
    /**
     *
     */
    public void removeEmployee(IEmployee employee)
        throws BusinessLayerException
    {
        m_employees.remove(employee);
    }
    
    // Internal members
    private Vector m_employees = new Vector();
}



/**
 *
 */
class HashtableDepartment
    implements IDepartment, java.io.Serializable
{
    /**
     *
     */
    public String getTitle()
        throws BusinessLayerException
    {
        return m_title;
    }
        
    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_desc;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_desc = desc;
    }
    
    /**
     *
     */
    public IDepartment getParentDepartment()
        throws BusinessLayerException
    {
        return m_parentDept;
    }

    /**
     *
     */
    public IDepartment[] getContainedDepartments()
        throws BusinessLayerException
    {
        // Find all departments
        IDepartment[] allDepts = m_model.findAllDepartments();
        
        // Find those who have a parent of 'this'
        Vector tmp = new Vector();
        for (int i=0; i<allDepts.length; i++)
        {
            if (allDepts[i].getParentDepartment() == this)
                tmp.addElement(allDepts[i]);
        }
        
        // Return the set
        IDepartment[] retArray = null;
        if (tmp.size() > 0)
        {
            retArray = new IDepartment[tmp.size()];
            for (int i=0; i<retArray.length; i++)
                retArray[i] = (IDepartment)tmp.elementAt(i);
        }
        return retArray;
    }
    
    /**
     *
     */
    public IManager getManager()
        throws BusinessLayerException
    {
        return m_manager;
    }
    /**
     *
     */
    public void setManager(IManager mgr)
        throws BusinessLayerException
    {
        m_manager = mgr;
    }
    
    // Internal members
    String m_title;
    String m_desc;
    IDepartment m_parentDept;
    IManager m_manager;
    
    HashtableModel m_model;
}



/**
 *
 */
class HashtablePosition
    implements IPosition, java.io.Serializable
{
    /**
     *
     */
    public String getTitle()
        throws BusinessLayerException
    {
        return m_title;
    }
    
    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_desc;
    }
    
    /**
     *
     */
    public String getCategory()
        throws BusinessLayerException
    {
        return m_category;
    }
    
    /**
     *
     */
    public int getMinSalary()
        throws BusinessLayerException
    {
        return m_minSalary;
    }
    
    /**
     *
     */
    public int getMaxSalary()
        throws BusinessLayerException
    {
        return m_maxSalary;
    }
    
    
    // Internal members
    String m_title;
    String m_desc;
    String m_category;
    int m_minSalary;
    int m_maxSalary;
}



/**
 *
 */
class HashtableContactInfo
    implements IContactInfo, java.io.Serializable
{
    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_desc;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_desc = desc;
    }
    
    String m_desc;
}



/**
 *
 */
class HashtableAddress extends HashtableContactInfo
    implements IAddress, java.io.Serializable
{
    /**
     *
     */
    public String getStreet()
        throws BusinessLayerException
    {
        return m_street;
    }
    /**
     *
     */
    public void setStreet(String street)
        throws BusinessLayerException
    {
        m_street = street;
    }

    /**
     *
     */
    public String getCity()
        throws BusinessLayerException
    {
        return m_city;
    }
    /**
     *
     */
    public void setCity(String city)
        throws BusinessLayerException
    {
        m_city = city;
    }

    /**
     *
     */
    public String getState()
        throws BusinessLayerException
    {
        return m_state;
    }
    /**
     *
     */
    public void setState(String state)
        throws BusinessLayerException
    {
        m_state = state;
    }

    /**
     *
     */
    public String getZip()
        throws BusinessLayerException
    {
        return m_zip;
    }
    /**
     *
     */
    public void setZip(String zip)
        throws BusinessLayerException
    {
        m_zip = zip;
    }

    /**
     *
     */
    public String getCountry()
        throws BusinessLayerException
    {
        return m_country;
    }
    /**
     *
     */
    public void setCountry(String country)
        throws BusinessLayerException
    {
        m_country = country;
    }
    
    /**
     *
     */
    public String toString()
    {
        return m_street + " " + m_city + ", " + m_state +
            " " + m_zip + " " + m_country;
    }
    
    
    // Internal members
    String m_street;
    String m_city;
    String m_state;
    String m_zip;
    String m_country;
}



/**
 *
 */
class HashtableEMail extends HashtableContactInfo
    implements IEMail, java.io.Serializable
{
    /**
     *
     */
    public String getEmail()
        throws BusinessLayerException
    {
        return m_email;
    }
    /**
     *
     */
    public void setEmail(String email)
        throws BusinessLayerException
    {
        m_email = email;
    }

    /**
     *
     */
    public String toString()
    {
        return m_email;
    }
    
    
    // Internal members
    String m_email;
}



/**
 *
 */
class HashtablePhone extends HashtableContactInfo
    implements IPhone, java.io.Serializable
{
    /**
     *
     */
    public String getPrefix()
        throws BusinessLayerException
    {
        return m_prefix;
    }
    /**
     *
     */
    public void setPrefix(String prefix)
        throws BusinessLayerException
    {
         m_prefix = prefix;
    }

    /**
     *
     */
    public String getNumber()
        throws BusinessLayerException
    {
        return m_number;
    }
    /**
     *
     */
    public void setNumber(String number)
        throws BusinessLayerException
    {
        m_number = number;
    }

    /**
     *
     */
    public String getSuffix()
        throws BusinessLayerException
    {
        return m_suffix;
    }
    /**
     *
     */
    public void setSuffix(String suffix)
        throws BusinessLayerException
    {
        m_suffix = suffix;
    }
    
    /**
     *
     */
    public String toString()
    {
        return m_prefix + " " + m_number + " " + m_suffix;
    }
    
    
    // Internal members
    String m_prefix;
    String m_number;
    String m_suffix;
}



/**
 * $Id: HashtableModel.java,v 1.2 1999/07/07 16:51:27 tneward Exp $
 *
 * This class represents our business object layer "model".
 * In this case, the data-access layer is a wrapper around a
 * collection of Java java.util.Hashtable objects, and not any
 * persistent store. Note, though, that this model *could*, in
 * fact, be persistent if it could be told when to save itself off
 * to disk (via Serialization), and where to reload when restarting.
 *
 * $Log: HashtableModel.java,v $
 * Revision 1.2  1999/07/07 16:51:27  tneward
 * Duh; you need to be Serializable if you're going to be Serialized, silly....
 *
 */
public class HashtableModel
    implements IEmployeeModel, java.io.Serializable
{
    /**
     * Create IPerson object
     */
    public IPerson createPerson(
        String firstName, String middleName, String lastName,
        String ssn)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_persons.get(ssn) == null)
        {
            HashtablePerson person = 
                new HashtablePerson(firstName, middleName,
                    lastName, ssn);
            m_persons.put(ssn, person);
            return person;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    
    /**
     * Create IEmployee object
     */
    public IEmployee createEmployee(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_employees.get(ssn) == null &&
            m_persons.get(ssn) == null)
        {
            HashtableEmployee employee =
                new HashtableEmployee(firstName, lastName,
                    middleName, ssn, position, salary);
            
            m_persons.put(ssn, employee);
            m_employees.put(ssn, employee);
            
            return employee;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    
    /**
     * Create IEmployee from IPerson
     */
    public IEmployee createEmployee(IPerson person, 
        IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_employees.get(person.getSSN()) == null)
        {
            HashtableEmployee employee =
                new HashtableEmployee(person.getFirstName(),
                    person.getLastName(),
                    person.getMiddleName(),
                    person.getSSN(), position, salary);
            
            m_persons.remove(employee.getSSN());
            m_persons.put(employee.getSSN(), employee);
            m_employees.put(employee.getSSN(), employee);
            
            return employee;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    
    /**
     * Create IManager object
     */
    public IManager createManager(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_managers.get(ssn) == null &&
            m_employees.get(ssn) == null && 
            m_persons.get(ssn) == null)
        {
            HashtableManager mgr =
                new HashtableManager(firstName, middleName,
                    lastName, ssn, position, salary);
            
            m_persons.put(ssn, mgr);
            m_employees.put(ssn, mgr);
            m_managers.put(ssn, mgr);
            
            return mgr;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }

    /**
     * Create IManager from IEmployee
     */
    public IManager createManager(IEmployee employee)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_managers.get(employee.getSSN()) == null)
        {
            HashtableManager mgr =
                new HashtableManager(employee.getFirstName(),
                    employee.getMiddleName(),
                    employee.getLastName(),
                    employee.getSSN(),
                    employee.getPosition(),
                    employee.getSalary());
            
            m_persons.remove(employee.getSSN());
            m_employees.remove(employee.getSSN());
            
            m_persons.put(mgr.getSSN(), mgr);
            m_employees.put(mgr.getSSN(), mgr);
            m_managers.put(mgr.getSSN(), mgr);
            
            return mgr;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    
    /**
     * Create IDepartment object
     */
    public IDepartment createDepartment(
        String name, IDepartment parent)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_departments.get(name) == null)
        {
            HashtableDepartment dept = new HashtableDepartment();
            dept.m_title = name;
            dept.m_parentDept = parent;
            dept.m_model = this;
            
            m_departments.put(name, dept);
            return dept;
        }
        else
            throw new DuplicateObjectException();
    }
    
    /**
     * Create IPosition object
     */
    public IPosition createPosition(
        String title, String desc, int minSalary, int maxSalary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_positions.get(title) == null)
        {
            HashtablePosition pos = new HashtablePosition();
            pos.m_title = title;
            pos.m_desc = desc;
            pos.m_minSalary = minSalary;
            pos.m_maxSalary = maxSalary;
            
            m_positions.put(title, pos);
            return pos;
        }
        else
            throw new DuplicateObjectException();
    }



    /**
     * Query for all Persons
     */
    public IPerson[] findAllPersons()
        throws BusinessLayerException
    {
        IPerson[] retArray = new IPerson[m_persons.size()];
        int i = 0;
        for (Enumeration e = m_persons.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IPerson)e.nextElement();
        }
        
        return retArray;
    }
    
    /**
     * Query for all Persons by last name
     */
    public IPerson[] findPersonsByLastName(String lastName)
        throws BusinessLayerException
    {
        // Filter out only those Persons with the given last name
        Vector tmp = new Vector();
        for (Enumeration e = m_persons.elements(); 
             e.hasMoreElements(); )
        {
            IPerson person = (IPerson)e.nextElement();
            if (person.getLastName().equals(lastName))
                tmp.addElement(person);
        }
        
        // Return the collection as an array of IPerson
        IPerson[] retArray = null;
        if (tmp.size() > 0)
        {
            retArray = new IPerson[tmp.size()];
            
            for (int i=0; i<retArray.length; i++)
                retArray[i] = (IPerson)tmp.elementAt(i);

            return retArray;
        }
        else
            return null;
    }
    
    /**
     * Query for all Persons by SSN
     */
    public IPerson[] findPersonsBySSN(String ssn)
        throws BusinessLayerException
    {
        // Filter out only those Persons with the given SSN
        Vector tmp = new Vector();
        for (Enumeration e = m_persons.elements(); 
             e.hasMoreElements(); )
        {
            IPerson person = (IPerson)e.nextElement();
            if (person.getSSN().equals(ssn))
                tmp.addElement(person);
        }
        
        // Return the collection as an array of IPerson
        IPerson[] retArray = null;
        if (tmp.size() > 0)
        {
            retArray = new IPerson[tmp.size()];
            
            for (int i=0; i<retArray.length; i++)
                retArray[i] = (IPerson)tmp.elementAt(i);
        }

        return retArray;
    }

    /**
     * Query for all Employees
     */
    public IEmployee[] findAllEmployees()
        throws BusinessLayerException
    {
        IEmployee[] retArray = new IEmployee[m_employees.size()];
        int i = 0;
        for (Enumeration e = m_employees.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IEmployee)e.nextElement();
        }
        
        return retArray;
    }
    
    /**
     * Query for all Managers
     */
    public IManager[] findAllManagers()
        throws BusinessLayerException
    {
        IManager[] retArray = new IManager[m_managers.size()];
        int i = 0;
        for (Enumeration e = m_managers.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IManager)e.nextElement();
        }
        
        return retArray;
    }
    
    /**
     * Query for all Positions
     */
    public IPosition[] findAllPositions()
        throws BusinessLayerException
    {
        IPosition[] retArray = new IPosition[m_positions.size()];
        int i = 0;
        for (Enumeration e = m_positions.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IPosition)e.nextElement();
        }
        
        return retArray;
    }
    
    /**
     * Query for all Departments
     */
    public IDepartment[] findAllDepartments()
        throws BusinessLayerException
    {
        IDepartment[] retArray = 
            new IDepartment[m_departments.size()];
        int i = 0;
        for (Enumeration e = m_departments.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IDepartment)e.nextElement();
        }
        
        return retArray;
    }
    
    /**
     * Query for "root" Department
     */
    public IDepartment findRootDepartment()
        throws BusinessLayerException
    {
        IDepartment[] allDepts = findAllDepartments();
        
        for (int i=0; i<allDepts.length; i++)
            if (allDepts[i].getParentDepartment() == null)
                return allDepts[i];
                
        throw new BusinessLayerException(
            "No root Department found");
    }
        
        
        
    /**
     * Remove a Person from the system
     */
    public void removePerson(IPerson person)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_persons.remove(person.getSSN());
    }

    /**
     * Remove an Employee from the system
     */
    public void removeEmployee(IEmployee employee)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_employees.remove(employee.getSSN());
        m_persons.remove(employee.getSSN());
    }

    /**
     * Remove a Manager from the system
     */
    public void removeManager(IManager manager)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_managers.remove(manager.getSSN());
        m_employees.remove(manager.getSSN());
        m_persons.remove(manager.getSSN());
    }

    /**
     * Remove a Department from the system
     */
    public void removeDepartment(IDepartment dept)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_departments.remove(dept.getTitle());
    }

    /**
     * Remove a Position from the system
     */
    public void removePosition(IPosition position)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_positions.remove(position);
    }
    
    
    // Internal members
    private Hashtable m_persons = new Hashtable();
    private Hashtable m_employees = new Hashtable();
    private Hashtable m_managers = new Hashtable();
    private Hashtable m_positions = new Hashtable();
    private Hashtable m_departments = new Hashtable();
}    