package Employee;

/**
 * This class represents our business object layer
 * "model".
 */
public interface IEmployeeModel
{
    /**
     * Create IPerson object
     */
    public IPerson createPerson(
        String firstName, String middleName, String lastName,
        String ssn)
        throws BusinessLayerException, DuplicateObjectException;
    
    /**
     * Create IEmployee object
     */
    public IEmployee createEmployee(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException;
    
    /**
     * Create IEmployee from IPerson
     */
    public IEmployee createEmployee(IPerson person, 
        IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException;
    
    /**
     * Create IManager object
     */
    public IManager createManager(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException;

    /**
     * Create IManager from IEmployee
     */
    public IManager createManager(IEmployee employee)
        throws BusinessLayerException, DuplicateObjectException;
    
    /**
     * Create IDepartment object
     */
    public IDepartment createDepartment(
        String name, IDepartment parent)
        throws BusinessLayerException, DuplicateObjectException;
    
    /**
     * Create IPosition object
     */
    public IPosition createPosition(
        String title, String desc, int minSalary, int maxSalary)
        throws BusinessLayerException, DuplicateObjectException;



    /**
     * Query for all Persons
     */
    public IPerson[] findAllPersons()
        throws BusinessLayerException;
    
    /**
     * Query for all Persons by last name
     */
    public IPerson[] findPersonsByLastName(String lastName)
        throws BusinessLayerException;
    
    /**
     * Query for all Persons by SSN
     */
    public IPerson[] findPersonsBySSN(String ssn)
        throws BusinessLayerException;

    /**
     * Query for all Employees
     */
    public IEmployee[] findAllEmployees()
        throws BusinessLayerException;
    
    /**
     * Query for all Managers
     */
    public IManager[] findAllManagers()
        throws BusinessLayerException;
    
    /**
     * Query for all Positions
     */
    public IPosition[] findAllPositions()
        throws BusinessLayerException;
    
    /**
     * Query for all Departments
     */
    public IDepartment[] findAllDepartments()
        throws BusinessLayerException;
    
    /**
     * Query for "root" Department
     */
    public IDepartment findRootDepartment()
        throws BusinessLayerException;
        
        
        
    /**
     * Remove a Person from the system
     */
    public void removePerson(IPerson person)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException;

    /**
     * Remove an Employee from the system
     */
    public void removeEmployee(IEmployee employee)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException;

    /**
     * Remove a Person from the system
     */
    public void removeManager(IManager manager)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException;

    /**
     * Remove a Department from the system
     */
    public void removeDepartment(IDepartment dept)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException;

    /**
     * Remove a Position from the system
     */
    public void removePosition(IPosition position)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException;
}    