package Employee;

import com.sun.media.jsdt.*;
import com.sun.media.jsdt.event.*;
import java.io.*;
import java.util.*;


/**
 *
 */
class JSDTPerson
    implements IPerson, java.io.Serializable
{
    // Internal members
    //
    private String m_firstName;
    private String m_middleName;
    private String m_lastName;
    private String m_ssn;
    private Vector m_contactInfo = new Vector();
    
    JSDTModel m_model;

    JSDTPerson()
    { }

    JSDTPerson(String fName, String mName, String lName,
                    String ssn)
    {
        m_firstName = fName;
        m_middleName = mName;
        m_lastName = lName;
        m_ssn = ssn;
    }

    /**
     *
     */
    public String getFirstName()
        throws BusinessLayerException
    {
        return m_firstName;
    }
    /**
     *
     */
    public void setFirstName(String fName)
        throws BusinessLayerException
    {
        m_firstName = fName;
        m_model.submit();
    }
    

    /**
     *
     */
    public String getMiddleName()
        throws BusinessLayerException
    {
        return m_middleName;
    }
    /**
     *
     */
    public void setMiddleName(String mName)
        throws BusinessLayerException
    {
        m_middleName = mName;
        m_model.submit();
    }


    /**
     *
     */
    public String getLastName()
        throws BusinessLayerException
    {
        return m_lastName;
    }
    /**
     *
     */
    public void setLastName(String lName)
        throws BusinessLayerException
    {
        m_lastName = lName;
        m_model.submit();
    }


    /**
     *
     */
    public String getSSN()
        throws BusinessLayerException
    {
        return m_ssn;
    }
    /**
     *
     */
    public void setSSN(String ssn)
        throws BusinessLayerException
    {
        m_ssn = ssn;
        m_model.submit();
    }

    
    /**
     *
     */
    public IContactInfo[] getContactInfo()
        throws BusinessLayerException
    {
        IContactInfo[] retArray = 
            new IContactInfo[m_contactInfo.size()];
        for (int i=0; i<retArray.length; i++)
            retArray[i] = (IContactInfo)m_contactInfo.elementAt(i);
        
        return retArray;
    }
    /**
     *
     */
    public void addEMail(String desc, String email)
        throws BusinessLayerException
    {
        JSDTEMail e = new JSDTEMail();
        e.m_desc = desc;
        e.m_email = email;
        m_contactInfo.addElement(e);

        m_model.submit();
    }
    /**
     *
     */
    public void addAddress(String desc, String street, String city,
        String state, String zip, String country)
        throws BusinessLayerException
    {
        JSDTAddress addr = new JSDTAddress();
        addr.m_desc = desc;
        addr.m_street = street;
        addr.m_city = city;
        addr.m_state = state;
        addr.m_zip = zip;
        addr.m_country = country;
        m_contactInfo.addElement(addr);
        
        m_model.submit();
    }
    /**
     *
     */
    public void addPhone(String desc, String prefix, String number,
        String suffix)
        throws BusinessLayerException
    {
        JSDTPhone phone = new JSDTPhone();
        phone.m_desc = desc;
        phone.m_prefix = prefix;
        phone.m_number = number;
        phone.m_suffix = suffix;
        m_contactInfo.addElement(phone);
        
        m_model.submit();
    }
    /**
     *
     */
    public void removeContactInfo(IContactInfo contactInfo)
        throws BusinessLayerException
    {
        m_contactInfo.remove(contactInfo);
    
        m_model.submit();
    }
    
    
}



/**
 *
 */
class JSDTEmployee extends JSDTPerson
    implements IEmployee, java.io.Serializable
{
    // Internal members
    private IPosition m_position;
    private int m_salary;
    
    
    JSDTEmployee()
    {
        super();
    }
    JSDTEmployee(String fName, String mName, String lName,
                 String ssn, IPosition pos, int salary)
    {
        super(fName, mName, lName, ssn);
        m_position = pos;
        m_salary = salary;
    }


    /**
     *
     */
    public IPosition getPosition()
        throws BusinessLayerException
    {
        return m_position;
    }
    /*
     *
     */
    public void setPosition(IPosition position)
        throws BusinessLayerException
    {
        m_position = position;
    }
    
    /**
     *
     */
    public int getSalary()
        throws BusinessLayerException
    {
        return m_salary;
    }
    /**
     *
     */
    public void setSalary(int salary)
        throws BusinessLayerException, IllegalSalaryException
    {
        if (salary > getPosition().getMaxSalary())
            throw new IllegalSalaryException("Max salary is " +
                getPosition().getMaxSalary());
    
        m_salary = salary;
    }
}



/**
 *
 */
class JSDTManager extends JSDTEmployee
    implements IManager, java.io.Serializable
{
    // Internal members
    private Vector m_employees = new Vector();
    
    
    JSDTManager()
    {
        super();
    }
    JSDTManager(String fName, String mName, String lName,
                String ssn, IPosition pos, int salary)
    {
        super(fName, mName, lName, ssn, pos, salary);
    }

    /**
     *
     */
    public IEmployee[] getEmployees()
        throws BusinessLayerException
    {
        IEmployee[] retArray = new IEmployee[m_employees.size()];
        for (int i=0; i<retArray.length; i++)
            retArray[i] = (IEmployee)m_employees.elementAt(i);
        return retArray;
    }
    /**
     *
     */
    public void addEmployee(IEmployee employee)
        throws BusinessLayerException
    {
        m_employees.addElement(employee);
    }
    /**
     *
     */
    public void removeEmployee(IEmployee employee)
        throws BusinessLayerException
    {
        m_employees.remove(employee);
    }
}



/**
 *
 */
class JSDTDepartment
    implements IDepartment, java.io.Serializable
{
    // Internal members
    //
    String m_title;
    String m_desc;
    IDepartment m_parentDept;
    IManager m_manager;
    
    JSDTModel m_model;
    
    
    /**
     *
     */
    public String getTitle()
        throws BusinessLayerException
    {
        return m_title;
    }
        
    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_desc;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_desc = desc;
        m_model.submit();
    }
    
    /**
     *
     */
    public IDepartment getParentDepartment()
        throws BusinessLayerException
    {
        return m_parentDept;
    }

    /**
     *
     */
    public IDepartment[] getContainedDepartments()
        throws BusinessLayerException
    {
        // Find all departments
        IDepartment[] allDepts = m_model.findAllDepartments();
        
        // Find those who have a parent of 'this'
        Vector tmp = new Vector();
        for (int i=0; i<allDepts.length; i++)
        {
            if (allDepts[i].getParentDepartment() == this)
                tmp.addElement(allDepts[i]);
        }
        
        // Return the set
        IDepartment[] retArray = null;
        if (tmp.size() > 0)
        {
            retArray = new IDepartment[tmp.size()];
            for (int i=0; i<retArray.length; i++)
                retArray[i] = (IDepartment)tmp.elementAt(i);
        }
        return retArray;
    }
    
    /**
     *
     */
    public IManager getManager()
        throws BusinessLayerException
    {
        return m_manager;
    }
    /**
     *
     */
    public void setManager(IManager mgr)
        throws BusinessLayerException
    {
        m_manager = mgr;
        m_model.submit();
    }
}



/**
 *
 */
class JSDTPosition
    implements IPosition, java.io.Serializable
{
    // Internal members
    //
    String m_title;
    String m_desc;
    String m_category;
    int m_minSalary;
    int m_maxSalary;

    JSDTModel m_model;
    
    
    /**
     *
     */
    public String getTitle()
        throws BusinessLayerException
    {
        return m_title;
    }
    
    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_desc;
    }
    
    /**
     *
     */
    public String getCategory()
        throws BusinessLayerException
    {
        return m_category;
    }
    
    /**
     *
     */
    public int getMinSalary()
        throws BusinessLayerException
    {
        return m_minSalary;
    }
    
    /**
     *
     */
    public int getMaxSalary()
        throws BusinessLayerException
    {
        return m_maxSalary;
    }
}



/**
 *
 */
class JSDTContactInfo
    implements IContactInfo, java.io.Serializable
{
    // Internal members
    //
    String m_desc;
    JSDTModel m_model;

    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_desc;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_desc = desc;
        m_model.submit();
    }
}



/**
 *
 */
class JSDTAddress extends JSDTContactInfo
    implements IAddress, java.io.Serializable
{
    // Internal members
    String m_street;
    String m_city;
    String m_state;
    String m_zip;
    String m_country;
    
    
    /**
     *
     */
    public String getStreet()
        throws BusinessLayerException
    {
        return m_street;
    }
    /**
     *
     */
    public void setStreet(String street)
        throws BusinessLayerException
    {
        m_street = street;
        m_model.submit();
    }

    /**
     *
     */
    public String getCity()
        throws BusinessLayerException
    {
        return m_city;
    }
    /**
     *
     */
    public void setCity(String city)
        throws BusinessLayerException
    {
        m_city = city;
        m_model.submit();
    }

    /**
     *
     */
    public String getState()
        throws BusinessLayerException
    {
        return m_state;
    }
    /**
     *
     */
    public void setState(String state)
        throws BusinessLayerException
    {
        m_state = state;
        m_model.submit();
    }

    /**
     *
     */
    public String getZip()
        throws BusinessLayerException
    {
        return m_zip;
    }
    /**
     *
     */
    public void setZip(String zip)
        throws BusinessLayerException
    {
        m_zip = zip;
        m_model.submit();
    }

    /**
     *
     */
    public String getCountry()
        throws BusinessLayerException
    {
        return m_country;
    }
    /**
     *
     */
    public void setCountry(String country)
        throws BusinessLayerException
    {
        m_country = country;
        m_model.submit();
    }
    
    /**
     *
     */
    public String toString()
    {
        return m_street + " " + m_city + ", " + m_state +
            " " + m_zip + " " + m_country;
    }
}



/**
 *
 */
class JSDTEMail extends JSDTContactInfo
    implements IEMail, java.io.Serializable
{
    // Internal members
    String m_email;
    
    
    /**
     *
     */
    public String getEmail()
        throws BusinessLayerException
    {
        return m_email;
    }
    /**
     *
     */
    public void setEmail(String email)
        throws BusinessLayerException
    {
        m_email = email;
        m_model.submit();
    }

    /**
     *
     */
    public String toString()
    {
        return m_email;
    }
}



/**
 *
 */
class JSDTPhone extends JSDTContactInfo
    implements IPhone, java.io.Serializable
{
    // Internal members
    //
    String m_prefix;
    String m_number;
    String m_suffix;
    
    
    /**
     *
     */
    public String getPrefix()
        throws BusinessLayerException
    {
        return m_prefix;
    }
    /**
     *
     */
    public void setPrefix(String prefix)
        throws BusinessLayerException
    {
         m_prefix = prefix;
         m_model.submit();
    }

    /**
     *
     */
    public String getNumber()
        throws BusinessLayerException
    {
        return m_number;
    }
    /**
     *
     */
    public void setNumber(String number)
        throws BusinessLayerException
    {
        m_number = number;
        m_model.submit();
    }

    /**
     *
     */
    public String getSuffix()
        throws BusinessLayerException
    {
        return m_suffix;
    }
    /**
     *
     */
    public void setSuffix(String suffix)
        throws BusinessLayerException
    {
        m_suffix = suffix;
        m_model.submit();
    }
    
    /**
     *
     */
    public String toString()
    {
        return m_prefix + " " + m_number + " " + m_suffix;
    }
}



/**
 * $Id: JSDTModel.java,v 1.9 1999/07/27 06:15:10 tneward Exp $
 *
 * JSDTModel
 *
 * $Log: JSDTModel.java,v $
 * Revision 1.9  1999/07/27 06:15:10  tneward
 * Periodic update
 *
 * Revision 1.8  1999/07/10 05:24:51  tneward
 * Periodic update
 *
 * Revision 1.7  1999/07/09 11:16:26  tneward
 * Periodic update
 *
 * Revision 1.6  1999/07/07 10:48:54  tneward
 * Revamped JSDTModel; seems to have a problem with retaining references, though.
 *
 */
public class JSDTModel
    implements IEmployeeModel, java.io.Serializable,
        Client, SessionManager, SessionListener
{
    // Package-friendly data
    //
    Hashtable m_persons = new Hashtable();
    Hashtable m_employees = new Hashtable();
    Hashtable m_managers = new Hashtable();
    Hashtable m_positions = new Hashtable();
    Hashtable m_departments = new Hashtable();
    
    // Internal data
    //
    transient private Session m_jsdtSession = null;
    transient private String m_clientName = "";
    transient private URLString m_jsdtURL = null;
    transient private ByteArray m_jsdtModel = null;
    transient private Token m_jsdtModelToken = null;


    /**
     *
     */
    public JSDTModel(String name, String host)
    {
        this(name, host, 4567, "socket", "JSDTModelSession");
    }
    /**
     *
     */
    public JSDTModel(String name, String host, int port, 
                     String type, String session)
    {
        m_clientName = name;

        URLString url =
            URLString.createSessionURL(host, port, type, session);
        
        boolean created = false;
        try
        {
            // Create the Registry if it doesn't exist yet; this
            // should probably be in a separate process
            if (RegistryFactory.registryExists(type) == false)
            {
                RegistryFactory.startRegistry(type);
            }

            // Need to check to see if the Session exists; if not,
            // create it with us as a SessionManager; if so, just
            // do a normal create() --FUTURE

            // Create the Session if it doesn't exist yet
            m_jsdtSession =
                SessionFactory.createSession(this, url, true);

            // Put up or get the HashtableModel
            if (m_jsdtSession.byteArrayExists("JSDTModel"))
            {
                // Retrieve it
                m_jsdtModel = m_jsdtSession.createByteArray(this,
                    "HashtableModel", true);

                // Retrieve the Token
                m_jsdtModelToken = m_jsdtSession.createToken(this,
                    "HashtableModelToken", true);

                retrieve();
            }
            else
            {
                // Put it up
                m_jsdtModel = m_jsdtSession.createByteArray(this,
                    "HashtableModel", true);

                // Create the Token
                m_jsdtModelToken = m_jsdtSession.createToken(this,
                    "HashtableModelToken", true);

                submit();
            }

            // Add a listener to track changes to the shared object
            m_jsdtModel.addByteArrayListener(new ByteArrayAdaptor()
            {
                public void byteArrayValueChanged(ByteArrayEvent e)
                {
                    System.out.println(e.toString());
                    retrieve();
                }
            });
        }
        catch (JSDTException jsdtEx)
        {
            jsdtEx.printStackTrace();
        }
    }


    void retrieve()
    {
        try
        {
            // Is somebody in the process of modifying the data?
            //
            while (m_jsdtModelToken.test() != Token.NOT_IN_USE)
            {
                // While somebody's doing something to it, go ahead
                // and wait for them to finish; not the world's most
                // efficient algorithm, but it works
                try
                { Thread.sleep(1000); }
                catch (Exception ex)
                { ex.printStackTrace(); }
            }

            // Obtain the data
            //
            byte[] bytes = m_jsdtModel.getValueAsBytes();
            ByteArrayInputStream bais =
                new ByteArrayInputStream(bytes);
            ObjectInputStream ois =
                new ObjectInputStream(bais);
    
            m_persons = (Hashtable)ois.readObject();
            m_employees = (Hashtable)ois.readObject();
            m_managers = (Hashtable)ois.readObject();
            m_positions = (Hashtable)ois.readObject();
            m_departments = (Hashtable)ois.readObject();
        }
        catch (ClassNotFoundException cnfEx)
        {
            cnfEx.printStackTrace();
        }
        catch (java.io.IOException ioEx)
        {
            ioEx.printStackTrace();
        }
        catch (JSDTException jsdtEx)
        {
            jsdtEx.printStackTrace();
        }
    }
    void submit()
    {
        try
        {
            // Grab the Token (exclusively)
            //
            m_jsdtModelToken.grab(this, true);

            // Do the update
            //
            ByteArrayOutputStream baos =
                new ByteArrayOutputStream();
            ObjectOutputStream oos =
                new ObjectOutputStream(baos);
            
            oos.writeObject(m_persons);
            oos.writeObject(m_employees);
            oos.writeObject(m_managers);
            oos.writeObject(m_positions);
            oos.writeObject(m_departments);
            
            byte[] bytes = baos.toByteArray();

            m_jsdtModel.setValue(this, bytes);

            // Release the Token
            //
            m_jsdtModelToken.release(this);
        }
        catch (java.io.IOException ioEx)
        {
            ioEx.printStackTrace();
        }
        catch (JSDTException jsdtEx)
        {
            jsdtEx.printStackTrace();
        }
    }
    

    //=====================================================
    // Client methods
    /**
     *
     */
    public Object authenticate(AuthenticationInfo info)
    {
        return null;
    }
    /**
     *
     */
    public String getName()
    {
        return m_clientName;
    }


    //=====================================================
    // SessionManager methods
    /**
     * Authenticate clients requesting access; do nothing, for now.
     */
    public boolean sessionRequest(Session session,
                                  AuthenticationInfo info,
                                  Client client)
    {
        String challenge = "<challenge>";
        String expectedResponse = "<response>";
        String reply = null;

        info.setChallenge(challenge);
        reply = (String)client.authenticate(info);
        return (reply == null);
            // For the moment, they *all* return null (no security)
    }


    //=====================================================
    // SessionListener methods
    /**
     *
     */
    public void byteArrayCreated(SessionEvent evt)
    {
        System.out.println("byteArrayCreated: " + evt.toString());
    }
    /**
     *
     */
    public void byteArrayDestroyed(SessionEvent evt)
    {
    }
    /**
     *
     */
    public void channelCreated(SessionEvent evt)
    {
    }
    /**
     *
     */
    public void channelDestroyed(SessionEvent evt)
    {
    }
    /**
     *
     */
    public void sessionDestroyed(SessionEvent evt)
    {
    }
    /**
     *
     */
    public void sessionJoined(SessionEvent evt)
    {
        System.out.println("SessionJoined: " + evt.toString());
    }
    /**
     *
     */
    public void sessionLeft(SessionEvent evt)
    {
        System.out.println("SessionLeft: " + evt.toString());
    }
    /**
     *
     */
    public void sessionInvited(SessionEvent evt)
    { 
    }
    /**
     *
     */
    public void sessionExpelled(SessionEvent evt)
    {
    }
    /**
     *
     */
    public void tokenCreated(SessionEvent evt)
    {
        System.out.println("tokenCreated:" + evt.toString());
    }
    /**
     *
     */
    public void tokenDestroyed(SessionEvent evt)
    {
        System.out.println("tokenDestroyed:" + evt.toString());
    }


    //=====================================================
    // IEmployeeModel methods
    /**
     * Create IPerson object
     */
    public IPerson createPerson(
        String firstName, String middleName, String lastName,
        String ssn)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_persons.get(ssn) == null)
        {
            JSDTPerson person = 
                new JSDTPerson(firstName, middleName,
                    lastName, ssn);
            
            // Hook the JSDTPerson to the JSDTModel
            person.m_model = this;

            m_persons.put(ssn, person);

            submit();
            return person;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    /**
     * Create IEmployee object
     */
    public IEmployee createEmployee(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_employees.get(ssn) == null &&
            m_persons.get(ssn) == null)
        {
            JSDTEmployee employee =
                new JSDTEmployee(firstName, lastName,
                    middleName, ssn, position, salary);
            
            // Hook the JSDTEmployee to the JSDTModel
            employee.m_model = this;

            m_persons.put(ssn, employee);
            m_employees.put(ssn, employee);
            
            submit();
            
            return employee;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    /**
     * Create IEmployee from IPerson
     */
    public IEmployee createEmployee(IPerson person, 
        IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_employees.get(person.getSSN()) == null)
        {
            JSDTEmployee employee =
                new JSDTEmployee(person.getFirstName(),
                    person.getLastName(),
                    person.getMiddleName(),
                    person.getSSN(), position, salary);
            
            // Hook the JSDTEmployee to the JSDTModel
            employee.m_model = this;

            m_persons.remove(employee.getSSN());
            m_persons.put(employee.getSSN(), employee);
            m_employees.put(employee.getSSN(), employee);
            
            submit();
            
            return employee;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    /**
     * Create IManager object
     */
    public IManager createManager(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_managers.get(ssn) == null &&
            m_employees.get(ssn) == null && 
            m_persons.get(ssn) == null)
        {
            JSDTManager mgr =
                new JSDTManager(firstName, middleName,
                    lastName, ssn, position, salary);
            
            // Hook the JSDTManager to the JSDTModel
            mgr.m_model = this;

            m_persons.put(ssn, mgr);
            m_employees.put(ssn, mgr);
            m_managers.put(ssn, mgr);
            
            submit();
            
            return mgr;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    /**
     * Create IManager from IEmployee
     */
    public IManager createManager(IEmployee employee)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_managers.get(employee.getSSN()) == null)
        {
            JSDTManager mgr =
                new JSDTManager(employee.getFirstName(),
                    employee.getMiddleName(),
                    employee.getLastName(),
                    employee.getSSN(),
                    employee.getPosition(),
                    employee.getSalary());
            
            // Hook the JSDTManager to the JSDTModel
            mgr.m_model = this;

            m_persons.remove(employee.getSSN());
            m_employees.remove(employee.getSSN());
            
            m_persons.put(mgr.getSSN(), mgr);
            m_employees.put(mgr.getSSN(), mgr);
            m_managers.put(mgr.getSSN(), mgr);
            
            submit();
            
            return mgr;
        }
        else
        {
            throw new DuplicateObjectException();
        }
    }
    /**
     * Create IDepartment object
     */
    public IDepartment createDepartment(
        String name, IDepartment parent)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_departments.get(name) == null)
        {
            JSDTDepartment dept = new JSDTDepartment();
            dept.m_title = name;
            dept.m_parentDept = parent;
            dept.m_model = this;
            
            // Hook the JSDTDepartment to the JSDTModel
            dept.m_model = this;

            m_departments.put(name, dept);
            
            submit();
            
            return dept;
        }
        else
            throw new DuplicateObjectException();
    }
    /**
     * Create IPosition object
     */
    public IPosition createPosition(
        String title, String desc, int minSalary, int maxSalary)
        throws BusinessLayerException, DuplicateObjectException
    {
        if (m_positions.get(title) == null)
        {
            JSDTPosition pos = new JSDTPosition();
            pos.m_title = title;
            pos.m_desc = desc;
            pos.m_minSalary = minSalary;
            pos.m_maxSalary = maxSalary;
            
            // Hook the JSDTPosition to the JSDTModel
            pos.m_model = this;

            m_positions.put(title, pos);
            
            submit();
            
            return pos;
        }
        else
            throw new DuplicateObjectException();
    }


    /**
     * Query for all Persons
     */
    public IPerson[] findAllPersons()
        throws BusinessLayerException
    {
        IPerson[] retArray = new IPerson[m_persons.size()];
        int i = 0;
        for (Enumeration e = m_persons.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IPerson)e.nextElement();
        }
        
        return retArray;
    }
    /**
     * Query for all Persons by last name
     */
    public IPerson[] findPersonsByLastName(String lastName)
        throws BusinessLayerException
    {
        // Filter out only those Persons with the given last name
        Vector tmp = new Vector();
        for (Enumeration e = m_persons.elements(); 
             e.hasMoreElements(); )
        {
            IPerson person = (IPerson)e.nextElement();
            if (person.getLastName().equals(lastName))
                tmp.addElement(person);
        }
        
        // Return the collection as an array of IPerson
        IPerson[] retArray = null;
        if (tmp.size() > 0)
        {
            retArray = new IPerson[tmp.size()];
            
            for (int i=0; i<retArray.length; i++)
                retArray[i] = (IPerson)tmp.elementAt(i);

            return retArray;
        }
        else
            return null;
    }
    /**
     * Query for all Persons by SSN
     */
    public IPerson[] findPersonsBySSN(String ssn)
        throws BusinessLayerException
    {
        // Filter out only those Persons with the given SSN
        Vector tmp = new Vector();
        for (Enumeration e = m_persons.elements(); 
             e.hasMoreElements(); )
        {
            IPerson person = (IPerson)e.nextElement();
            if (person.getSSN().equals(ssn))
                tmp.addElement(person);
        }
        
        // Return the collection as an array of IPerson
        IPerson[] retArray = null;
        if (tmp.size() > 0)
        {
            retArray = new IPerson[tmp.size()];
            
            for (int i=0; i<retArray.length; i++)
                retArray[i] = (IPerson)tmp.elementAt(i);
        }

        return retArray;
    }
    /**
     * Query for all Employees
     */
    public IEmployee[] findAllEmployees()
        throws BusinessLayerException
    {
        IEmployee[] retArray = new IEmployee[m_employees.size()];
        int i = 0;
        for (Enumeration e = m_employees.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IEmployee)e.nextElement();
        }
        
        return retArray;
    }
    /**
     * Query for all Managers
     */
    public IManager[] findAllManagers()
        throws BusinessLayerException
    {
        IManager[] retArray = new IManager[m_managers.size()];
        int i = 0;
        for (Enumeration e = m_managers.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IManager)e.nextElement();
        }
        
        return retArray;
    }
    /**
     * Query for all Positions
     */
    public IPosition[] findAllPositions()
        throws BusinessLayerException
    {
        IPosition[] retArray = new IPosition[m_positions.size()];
        int i = 0;
        for (Enumeration e = m_positions.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IPosition)e.nextElement();
        }
        
        return retArray;
    }
    /**
     * Query for all Departments
     */
    public IDepartment[] findAllDepartments()
        throws BusinessLayerException
    {
        IDepartment[] retArray = 
            new IDepartment[m_departments.size()];
        int i = 0;
        for (Enumeration e = m_departments.elements(); 
             e.hasMoreElements(); )
        {
            retArray[i++] = (IDepartment)e.nextElement();
        }
        
        return retArray;
    }
    /**
     * Query for "root" Department
     */
    public IDepartment findRootDepartment()
        throws BusinessLayerException
    {
        IDepartment[] allDepts = findAllDepartments();
        
        for (int i=0; i<allDepts.length; i++)
            if (allDepts[i].getParentDepartment() == null)
                return allDepts[i];
                
        throw new BusinessLayerException(
            "No root Department found");
    }
        
        
    /**
     * Remove a Person from the system
     */
    public void removePerson(IPerson person)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_persons.remove(person.getSSN());
        submit();
    }
    /**
     * Remove an Employee from the system
     */
    public void removeEmployee(IEmployee employee)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_employees.remove(employee.getSSN());
        m_persons.remove(employee.getSSN());
        submit();
    }
    /**
     * Remove a Manager from the system
     */
    public void removeManager(IManager manager)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_managers.remove(manager.getSSN());
        m_employees.remove(manager.getSSN());
        m_persons.remove(manager.getSSN());
        submit();
    }
    /**
     * Remove a Department from the system
     */
    public void removeDepartment(IDepartment dept)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_departments.remove(dept.getTitle());
        submit();
    }
    /**
     * Remove a Position from the system
     */
    public void removePosition(IPosition position)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        m_positions.remove(position);
        submit();
    }


    /**
     * Test driver
     */
    public static void main(String[] args)
        throws Exception
    {
        JSDTModel m = new JSDTModel("Host", "localhost");

        System.out.println("JSDTModel bound");

        /*
        IPerson[] ret = m.findPersonsBySSN("123-45-6789");
        if (ret == null || ret.length == 0)
        {
            System.out.println("Creating John Doe");
            IPerson p = m.createPerson("John", "", "Doe", "123-45-6789");
        }
        else
        {
            System.out.println("Found John Doe: " + ret[0].toString());
        }
        */
    }
}
