package Employee;

import java.sql.*;
import java.util.*;


/**
 * RDBMSPerson is the RDBMS Model implementation of IPerson
 */
class RDBMSPerson
    implements IPerson
{
    /**
     *
     */
    RDBMSPerson(int personID)
        throws BusinessLayerException
    {
        try
        {
            // Retrieve Person info by ID
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID, f_name, m_name, l_name, ssn " +
                "FROM person WHERE person.id = " + personID);
                
            if (rs.next())
            {
                m_id = rs.getInt("ID");
                m_firstName = rs.getString("f_name");
                m_middleName = rs.getString("m_name");
                m_lastName = rs.getString("l_name");
                m_SSN = rs.getString("ssn");
            }
            else
                throw new BusinessLayerException("Person ID " +
                    personID + " not found");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }

    /**
     *
     */
    public String getFirstName()
        throws BusinessLayerException
    {
        return m_firstName;
    }
    /**
     *
     */
    public void setFirstName(String fName)
        throws BusinessLayerException
    {
        m_firstName = fName;
        commit();
    }
    

    /**
     *
     */
    public String getLastName()
        throws BusinessLayerException
    {
        return m_lastName;
    }
    /**
     *
     */
    public void setLastName(String lName)
        throws BusinessLayerException
    {
        m_lastName = lName;
        commit();
    }


    /**
     *
     */
    public String getMiddleName()
        throws BusinessLayerException
    {
        return m_middleName;
    }
    /**
     *
     */
    public void setMiddleName(String mName)
        throws BusinessLayerException
    {
        m_middleName = mName;
        commit();
    }


    /**
     *
     */
    public String getSSN()
        throws BusinessLayerException
    {
        return RDBMSModel.SSNFromDB(m_SSN);
    }
    /**
     *
     */
    public void setSSN(String ssn)
        throws BusinessLayerException
    {
        // Reduce ssn to all-numeric format
                
        m_SSN = RDBMSModel.SSNToDB(ssn);
        commit();
    }

    
    /**
     *
     */
    public IContactInfo[] getContactInfo()
        throws BusinessLayerException
    {
        Vector tempVector = new Vector();
    
        try
        {
            // First grab all Email instances
            //
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID " +
                "FROM email WHERE person_id_fk = " + m_id);
                
            while (rs.next())
            {
                int email_id = rs.getInt("ID");
                IEMail e = new RDBMSEMail(email_id);
                
                tempVector.addElement(e);
            }
            rs.close();
            
            // Next grab all Phone instances
            //
            rs = stmt.executeQuery(
                "SELECT ID " +
                "FROM phone WHERE person_id_fk = " + m_id);
                
            while (rs.next())
            {
                int phone_id = rs.getInt("ID");
                IPhone p = new RDBMSPhone(phone_id);
                
                tempVector.addElement(p);
            }
            rs.close();
            
            // Next grab all Address instances
            //
            rs = stmt.executeQuery(
                "SELECT ID " +
                "FROM address WHERE person_id_fk = " + m_id);
                
            while (rs.next())
            {
                int addr_id = rs.getInt("ID");
                IAddress a = new RDBMSAddress(addr_id);
                
                tempVector.addElement(a);
            }
            rs.close();
            
            // Return the whole batch
            IContactInfo[] retArray = new IContactInfo[tempVector.size()];
            for (int i=0; i<retArray.length; i++)
                retArray[i] = (IContactInfo)tempVector.elementAt(i);
                
            return retArray;
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    /**
     *
     */
    public void addEMail(String desc, String email)
        throws BusinessLayerException
    {
        // Create a new row in the email table
        try
        {
            int emailID = 
                RDBMSModel.nextDatabaseID("email");

            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO email " +
                    "(ID, person_id_fk, description, email) " +
                "VALUES " +
                    "(" + emailID + ", " +
                    m_id + ", " +
                    "'" + desc + "', " +
                    "'" + email + "')");
            if (result < 1)
                throw new BusinessLayerException(
                    "INSERT into database failed");
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    /**
     *
     */
    public void addAddress(String desc, String street, String city, 
        String state, String zip, String country)
        throws BusinessLayerException
    {
        // Create a new row in the address table
        try
        {
            int addrID = 
                RDBMSModel.nextDatabaseID("address");

            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO address " +
                    "(ID, person_id_fk, description, " +
                    "street, city, state, postal_code, country) " +
                "VALUES " +
                    "(" + addrID + ", " +
                    m_id + ", " +
                    "'" + desc + "', " +
                    "'" + street + "', " +
                    "'" + city + "', " +
                    "'" + state + "', " +
                    "'" + zip + "', " +
                    "'" + country + "')");
            if (result < 1)
                throw new BusinessLayerException(
                    "INSERT into database failed");
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    /**
     *
     */
    public void addPhone(String desc, String prefix, 
        String number, String suffix)
        throws BusinessLayerException
    {
        // Create a new row in the email table
        try
        {
            int phoneID = 
                RDBMSModel.nextDatabaseID("phone");
                
            String toDBNumber = RDBMSModel.PhoneToDB(number);

            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO phone " +
                    "(ID, person_id_fk, description, " +
                    "prefix, number, suffix) " +
                "VALUES " +
                    "(" + phoneID + ", " +
                    m_id + ", " +
                    "'" + desc + "', " +
                    "'" + prefix + "', " +
                    "'" + toDBNumber + "', " +
                    "'" + suffix + "')");
            if (result < 1)
                throw new BusinessLayerException(
                    "INSERT into database failed");
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    /**
     *
     */
    public void removeContactInfo(IContactInfo contactInfo)
        throws BusinessLayerException
    {
        try
        {
            // Need to determine which ContactInfo instance this is;
            // only way to do that is to cast (or getClass())
            String tableName = null;
            int infoID = 0;
            if (contactInfo instanceof RDBMSPhone)
            {
                RDBMSPhone phone = (RDBMSPhone)contactInfo;
                
                tableName = "phone";
                infoID = phone.m_id;
            }
            else if (contactInfo instanceof RDBMSEMail)
            {
                RDBMSEMail email = (RDBMSEMail)contactInfo;
                
                tableName = "email";
                infoID = email.m_id;
            }
            else if (contactInfo instanceof RDBMSAddress)
            {
                RDBMSAddress addr = (RDBMSAddress)contactInfo;
                
                tableName = "address";
                infoID = addr.m_id;
            }
            else
            {
                // We tried to remove an IContactInfo that doesn't
                // come from the RDBMSModel; throw an exception
                throw new BusinessLayerException("Not an RDBMSModel" +
                    " ContactInfo instance: " + contactInfo.toString());
            }
        
            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            int result = stmt.executeUpdate(
                "DELETE FROM " + tableName + " " +
                "WHERE ID = " + infoID);
            if (result < 1)
                throw new BusinessLayerException(
                    "DELETE from database failed");
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    // Internal methods
    //
    private void commit()
        throws BusinessLayerException
    {
        try
        {
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE person SET f_name='" + m_firstName + "', " +
                    "m_name='" + m_middleName + "', " +
                    "l_name='" + m_lastName + "', " +
                    "ssn='" + m_SSN + "' " +
                    "WHERE id = " + m_id);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    
    // Internal members
    int m_id;
    private String m_firstName;
    private String m_middleName;
    private String m_lastName;
    private String m_SSN;
} // RDBMSPerson


/**
 * RDBMSEmployee is the RDBMS Model implementation of IEmployee
 */
class RDBMSEmployee extends RDBMSPerson
    implements IEmployee
{
    /**
     *
     */
    RDBMSEmployee(int empID)
        throws BusinessLayerException
    {
        super(RDBMSModel.findPersonIDFromEmployeeID(empID));
    
        try
        {
            // Retrieve Person info by ID
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID, salary, position_id_fk " +
                "FROM employee WHERE employee.id = " + empID);
                
            if (rs.next())
            {
                m_id = rs.getInt("ID");
                m_salary = rs.getInt("salary");
                int positionID = rs.getInt("position_id_fk");
                m_position = new RDBMSPosition(positionID);
            }
            else
                throw new BusinessLayerException("Employee ID " +
                    empID + " not found");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    
    /**
     *
     */
    public int getSalary()
        throws BusinessLayerException
    {
        return m_salary;
    }
    /**
     *
     */
    public void setSalary(int salary)
        throws BusinessLayerException, IllegalSalaryException
    {
        if (salary > getPosition().getMaxSalary())
            throw new IllegalSalaryException("Max salary is " +
                getPosition().getMaxSalary());
        
        m_salary = salary;
        commit();
    }

    /**
     *
     */
    public IPosition getPosition()
        throws BusinessLayerException
    {
        return m_position;
    }
    /**
     *
     */
    public void setPosition(IPosition position)
        throws BusinessLayerException
    {
        try
        {
            RDBMSPosition pos = (RDBMSPosition)position;
        
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE employee SET position_id_fk=" + pos.m_id +
                "WHERE id = " + m_id);

            if (result > 0)
                m_position = position;
            else
                throw new BusinessLayerException(
                    "Update failed for an unknown reason");
        }
        catch (ClassCastException classCastEx)
        {
            // Handle this differently--maybe add the IPosition
            // to the database if no position by that title already
            // exists?
            
            throw new BusinessLayerException(classCastEx);
        }
        catch (Exception ex)
        {
            throw new BusinessLayerException(ex);
        }
    }
    
    
    // Internal methods
    private void commit()
        throws BusinessLayerException
    {
        try
        {
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE employee SET salary=" + m_salary +
                "WHERE id = " + m_id);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    
    
    // Internal members
    int m_id;
    private int m_salary;
    private IPosition m_position;
} // RDBMSEmployee


/**
 * RDBMSManager is the RDBMS Model implementation of IManager
 */
class RDBMSManager extends RDBMSEmployee
    implements IManager
{
    /**
     *
     */
    RDBMSManager(int mgrID)
        throws BusinessLayerException
    {
        super(RDBMSModel.findEmployeeIDFromManagerID(mgrID));
        
        m_id = mgrID;
            // Because we don't actually store any additional
            // fields in the database manager table, there's
            // no need to query the database; just store off
            // the manager ID for later use
    }

    /**
     *
     */
    public IEmployee[] getEmployees()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT manager_employees.employee_id_fk " + 
                "FROM manager_employees " +
                "WHERE manager_employees.manager_id_fk = " + m_id);
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int employeeID =
                    rs.getInt("manager_employees.employee_id_fk");

                IEmployee e = new RDBMSEmployee(employeeID);
                tempVector.addElement(e);
            }
            
            // Copy over from the Vector to the array to return
            IEmployee retArray[] = new RDBMSEmployee[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IEmployee)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }

    /**
     *
     */
    public void addEmployee(IEmployee employee)
        throws BusinessLayerException
    {
        try
        {
            RDBMSEmployee emp = (RDBMSEmployee)employee;
        
            // Check first to see if employee is already in the
            // manager_employees table under this manager
            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT manager_employees.employee_id_fk " + 
                "FROM manager_employees " +
                "WHERE manager_employees.manager_id_fk = " + m_id);
                
            while (rs.next())
            {
                int employeeID =
                    rs.getInt("manager_employees.employee_id_fk");

                if (employeeID == emp.m_id)
                {
                    // The employee is already under this manager;
                    // no point in adding them again
                    return;
                }
            }
        
            // Add a row to the manager_employees table
            stmt = RDBMSModel.getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO manager_employees " +
                    "(manager_id_fk, employee_id_fk) " +
                "VALUES " + "(" + m_id + ", " + emp.m_id + ")");
            
            if (result < 1)
                throw new BusinessLayerException(
                    "INSERT into database failed");
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    /**
     *
     */
    public void removeEmployee(IEmployee employee)
        throws BusinessLayerException
    {
        try
        {
            RDBMSEmployee emp = (RDBMSEmployee)employee;
        
            // Remove a row to the manager_employees table; if the
            // row isn't there, just ignore it
            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            int result = stmt.executeUpdate(
                "DELETE FROM manager_employees " +
                "WHERE manager_id_fk = " + m_id + " AND " +
                    "employee_id_fk = " + emp.m_id + ")");
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    
    // Internal members
    int m_id;
        // A concession; we make m_id package-friendly so that
        // RDBMSDepartment.setManager() can access the ID of the
        // RDBMSManager class. We could encapsulate this member
        // properly via a getID() method that was package-friendly,
        // but what would be the point? If we ever move away from
        // an ID-based scheme, then we need to rewrite a good
        // chunk of the whole codebase anyway.
} // RDBMSManager


/**
 * RDBMSEmail is the RDBMS Model implementation of IEMail
 */
class RDBMSEMail
    implements IContactInfo, IEMail
{
    /**
     *
     */
    RDBMSEMail(int emailID)
        throws BusinessLayerException
    {
        try
        {
            // Retrieve Email info by ID
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID, description, email " +
                "FROM email " +
                "WHERE email.id = " + emailID);
                
            if (rs.next())
            {
                m_id = rs.getInt("ID");
                m_description = rs.getString("description");
                m_email = rs.getString("email");
            }
            else
                throw new BusinessLayerException("EMail ID " +
                    emailID + " not found");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }

    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_description;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_description = desc;
        commit();
    }


    /**
     *
     */
    public String getEmail()
        throws BusinessLayerException
    {
        return m_email;
    }
    /**
     *
     */
    public void setEmail(String email)
        throws BusinessLayerException
    {
        m_email = email;
        commit();
    }
    
    
    /**
     *
     */    
    public String toString()
    {
        return m_email;
    }
    
    
    // Internal methods
    private void commit()
        throws BusinessLayerException
    {
        try
        {
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE email SET description='" + 
                    m_description + "', " +
                    "email='" + m_email + "' " +
                "WHERE id = " + m_id);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    
    
    // Internal members
    int m_id;
    private String m_description;
    private String m_email;
} // RDBMSEMail


/**
 * RDBMSPhone is the RDBMS Model implementation of IPhone
 */
class RDBMSPhone
    implements IContactInfo, IPhone
{
    /**
     *
     */
    RDBMSPhone(int phoneID)
        throws BusinessLayerException
    {
        try
        {
            // Retrieve Phone info by ID
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID, description, prefix, number, suffix " +
                "FROM phone " +
                "WHERE phone.id = " + phoneID);
                
            if (rs.next())
            {
                m_id = rs.getInt("ID");
                m_description = rs.getString("description");
                m_prefix = rs.getString("prefix");
                m_number = rs.getString("number");
                m_suffix = rs.getString("suffix");
            }
            else
                throw new BusinessLayerException("Phone ID " +
                    phoneID + " not found");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }

    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_description;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_description = desc;
        commit();
    }

    /**
     *
     */
    public String getPrefix()
        throws BusinessLayerException
    {
        return m_prefix;
    }
    /**
     *
     */
    public void setPrefix(String prefix)
        throws BusinessLayerException
    {
        m_prefix = prefix;
        commit();
    }

    /**
     *
     */
    public String getNumber()
        throws BusinessLayerException
    {
        return RDBMSModel.PhoneFromDB(m_number);
    }
    /**
     *
     */
    public void setNumber(String number)
        throws BusinessLayerException
    {
        m_number = RDBMSModel.PhoneToDB(number);
        commit();
    }

    /**
     *
     */
    public String getSuffix()
        throws BusinessLayerException
    {
        return m_suffix;
    }
    /**
     *
     */
    public void setSuffix(String suffix)
        throws BusinessLayerException
    {
        m_suffix = suffix;
        commit();
    }


    /**
     *
     */    
    public String toString()
    {
        return m_prefix + " " + m_number + " " + m_suffix;
    }


    // Internal methods
    private void commit()
        throws BusinessLayerException
    {
        try
        {
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE phone SET description='" + 
                    m_description + "', " +
                    "prefix='" + m_prefix + "', " +
                    "number='" + m_number + "', " +
                    "suffix='" + m_suffix + "' " +
                "WHERE id = " + m_id);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    
    
    // Internal members
    int m_id;
    private String m_description;
    private String m_prefix;
    private String m_number;
    private String m_suffix;
} // RDBMSPhone


/**
 * RDBMSAddress is the RDBMS Model implementation of IAddress
 */
class RDBMSAddress
    implements IContactInfo, IAddress
{
    /**
     *
     */
    RDBMSAddress(int addrID)
        throws BusinessLayerException
    {
        try
        {
            // Retrieve Address info by ID
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID, description, street, city, state, " +
                    "zip, country " +
                "FROM address " +
                "WHERE address.id = " + addrID);
                
            if (rs.next())
            {
                m_id = rs.getInt("ID");
                m_description = rs.getString("description");
                m_street = rs.getString("street");
                m_city = rs.getString("city");
                m_state = rs.getString("state");
                m_zip = rs.getString("zip");
                m_country = rs.getString("country");
            }
            else
                throw new BusinessLayerException("Address ID " +
                    addrID + " not found");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }

    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_description;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_description = desc;
        commit();
    }


    /**
     *
     */
    public String getStreet()
        throws BusinessLayerException
    {
        return m_street;
    }
    /**
     *
     */
    public void setStreet(String street)
        throws BusinessLayerException
    {
        m_street = street;
        commit();
    }

    /**
     *
     */
    public String getCity()
        throws BusinessLayerException
    {
        return m_city;
    }
    /**
     *
     */
    public void setCity(String city)
        throws BusinessLayerException
    {
        m_city = city;
        commit();
    }

    /**
     *
     */
    public String getState()
        throws BusinessLayerException
    {
        return m_state;
    }
    /**
     *
     */
    public void setState(String state)
        throws BusinessLayerException
    {
        m_state = state;
        commit();
    }

    /**
     *
     */
    public String getZip()
        throws BusinessLayerException
    {
        return m_zip;
    }
    /**
     *
     */
    public void setZip(String zip)
        throws BusinessLayerException
    {
        m_zip = zip;
        commit();
    }

    /**
     *
     */
    public String getCountry()
        throws BusinessLayerException
    {
        return m_country;
    }
    /**
     *
     */
    public void setCountry(String country)
        throws BusinessLayerException
    {
        m_country = country;
        commit();
    }
    

    /**
     *
     */    
    public String toString()
    {
        return m_street + " " + m_city + ", " + m_state +
            " " + m_zip + " " + m_country;
    }


    // Internal methods
    private void commit()
        throws BusinessLayerException
    {
        try
        {
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE address SET description='" + 
                    m_description + "', " +
                    "street='" + m_street + "', " +
                    "city='" + m_city + "', " +
                    "state='" + m_state + "', " +
                    "zip='" + m_zip + "', " +
                    "country='" + m_country + "' " +
                "WHERE id = " + m_id);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    
    
    // Internal members
    int m_id;
    private String m_description;
    private String m_street;
    private String m_city;
    private String m_state;
    private String m_zip;
    private String m_country;
} // RDBMSAddress


/**
 *
 */
class RDBMSPosition
    implements IPosition
{
    /**
     *
     */
    RDBMSPosition(int posID)
        throws BusinessLayerException
    {
        try
        {
            // Retrieve Position info by ID
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID, title, description, min_salary, " +
                    "max_salary, category " +
                "FROM position " +
                "WHERE position.id = " + posID);
                
            if (rs.next())
            {
                m_id = rs.getInt("ID");
                m_title = rs.getString("title");
                m_description = rs.getString("description");
                m_minSalary = rs.getInt("min_salary");
                m_maxSalary = rs.getInt("max_salary");
                m_category = rs.getString("category");
            }
            else
                throw new BusinessLayerException("Position ID " +
                    posID + " not found");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    
    /**
     *
     */
    public String getTitle()
        throws BusinessLayerException
    {
        return m_title;
    }
    
    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_description;
    }
    
    /**
     *
     */
    public String getCategory()
        throws BusinessLayerException
    {
        return m_category;
    }
    
    /**
     *
     */
    public int getMinSalary()
        throws BusinessLayerException
    {
        return m_minSalary;
    }
    
    /**
     *
     */
    public int getMaxSalary()
        throws BusinessLayerException
    {
        return m_maxSalary;
    }
    
    
    /**
     *
     */
    public String toString()
    {
        return m_title;
    }
    
    
    // Internal members
    int m_id;
    private String m_title;
    private String m_description;
    private String m_category;
    private int m_minSalary;
    private int m_maxSalary;
}    


/**
 *
 */
class RDBMSDepartment
    implements IDepartment
{
    /**
     *
     */
    RDBMSDepartment(int deptID)
        throws BusinessLayerException
    {
        try
        {
            // Retrieve Department info by ID
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT ID, name, description, manager_id_fk " +
                "FROM dept " +
                "WHERE dept.id = " + deptID);
                
            if (rs.next())
            {
                m_id = rs.getInt("ID");
                m_title = rs.getString("name");
                m_description = rs.getString("description");
                int mgrID = rs.getInt("manager_id_fk");
                if (mgrID != 0)
                    m_manager = new RDBMSManager(mgrID);
                else
                    m_manager = null;
            }
            else
                throw new BusinessLayerException("Dept ID " + 
                    deptID + " not found");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }
    

    /**
     *
     */
    public String getTitle()
        throws BusinessLayerException
    {
        return m_title;
    }
        
    /**
     *
     */
    public String getDescription()
        throws BusinessLayerException
    {
        return m_description;
    }
    /**
     *
     */
    public void setDescription(String desc)
        throws BusinessLayerException
    {
        m_description = desc;
        commit();
    }
    
    /**
     *
     */
    public IDepartment getParentDepartment()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT dept.parent_dept_id_fk FROM dept " +
                "WHERE dept.ID = " + m_id);
                
            if (rs.next())
            {
                int deptID = rs.getInt("dept.ID");
                
                if (deptID == 0 || deptID == m_id)
                    return null;
                else
                    return new RDBMSDepartment(deptID);
            }
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
        return null;
    }

    /**
     *
     */
    public IDepartment[] getContainedDepartments()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = 
                RDBMSModel.getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT dept.ID FROM dept " +
                "WHERE dept.parent_dept_id_fk = " + m_id);
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int deptID = rs.getInt("dept.ID");

                IDepartment e = new RDBMSDepartment(deptID);
                tempVector.addElement(e);
            }
            
            // Copy over from the Vector to the array to return
            IDepartment retArray[] = new RDBMSDepartment[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IDepartment)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (Exception x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    /**
     *
     */
    public IManager getManager()
        throws BusinessLayerException
    {
        return m_manager;
    }
    /**
     *
     */
    public void setManager(IManager mgr)
        throws BusinessLayerException
    {
        try
        {
            RDBMSManager rdbmsMgr = (RDBMSManager)mgr;
            
            // Update dept.manager_id_fk to be this Manager's ID
            int mgrID = rdbmsMgr.m_id;
            
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE dept SET manager_id_fk=" + mgrID + " " +
                "WHERE dept.id = " + m_id);

            m_manager = mgr;
        }
        catch (ClassCastException CCEx)
        {
            // Handle this differently--maybe add the IManager
            // to the database if no Manager, Employee or Person by
            // that name/SSN tuple 
            // exists?

            System.err.println("Must setManager() using Manager from same package!");
            CCEx.printStackTrace();
            throw new BusinessLayerException(CCEx);
        }
        catch (Exception Ex)
        {
            Ex.printStackTrace();
            throw new BusinessLayerException(Ex);
        }
    }
    
    
    /**
     *
     */
    public String toString()
    {
        return m_title;
    }
    
    
    // Internal methods
    private void commit()
        throws BusinessLayerException
    {
        // Update dept
        try
        {
            Connection c = RDBMSModel.getConnection();
            Statement stmt = c.createStatement();
            int result = stmt.executeUpdate(
                "UPDATE dept SET description='" +
                    m_description + "' " +
                "WHERE dept.id = " + m_id);
        }
        catch (Exception Ex)
        {
            Ex.printStackTrace();
            throw new BusinessLayerException(Ex);
        }
    }
    
    
    // Internal members
    int m_id;
        // See notes in other classes as to why this member is a
        // package-friendly member instead of private
    private String m_title;
    private String m_description;
    private IManager m_manager;
} // RDBMSDepartment




/**
 *
 */
public class RDBMSModel
    implements IEmployeeModel
{
    /**
     *
     */    
    public RDBMSModel(Connection conn)
    {
        s_connection = conn;
    }
    
    /**
     *
     */    
    public static Connection getConnection()
    {
        return s_connection;
    }
    private static Connection s_connection = null;

    
    /**
     * Create IPerson object
     */
    public IPerson createPerson(
        String firstName, String middleName, String lastName,
        String ssn)
        throws BusinessLayerException, DuplicateObjectException
    {
        try
        {
            int personID = nextDatabaseID("person");
            
            // Must format SSN appropriately (all numbers)
            String newSSN = new String();
            for (int i=0; i<ssn.length(); i++)
                if (Character.isDigit(ssn.charAt(i)))
                    newSSN += ssn.charAt(i);
            ssn = newSSN;
            
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO person " +
                "VALUES (" +
                    personID + ", " +
                    "'" + firstName + "', " +
                    "'" + middleName + "', " +
                    "'" + lastName + "', " +
                    "'" + ssn + "')");
            if (result > 0)
                return new RDBMSPerson(personID);
            else
                throw new BusinessLayerException(
                    "Invalid database state; can't insert");
        }
        catch (SQLException ex)
        {
            throw new BusinessLayerException(ex);
        }
    }    
    
    /**
     * Create IEmployee object
     */
    public IEmployee createEmployee(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        IPerson person = createPerson(firstName, lastName,
            middleName, ssn);
        if (person == null)
            return null;
            
        return createEmployee(person, position, salary);
    }    

    /**
     * Create IEmployee object
     */
    public IEmployee createEmployee(IPerson person, 
        IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        try
        {
            // First cast off person into an RDBMSPerson; in theory,
            // if person is from a different model, we could at this
            // point insert the IPerson into the RDBMS, but for this
            // example, we just call it an exception and leave it
            // at that.
            RDBMSPerson dbPerson = (RDBMSPerson)person;
            RDBMSPosition dbPosition = (RDBMSPosition)position;
            
            //
            int empID = nextDatabaseID("employee");
            
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO employee " +
                "VALUES (" +
                    empID + ", " +
                    dbPerson.m_id + ", " +
                    dbPosition.m_id + ", " +
                    salary + ")");
            
            if (result > 0)
                return new RDBMSEmployee(empID);
            else
                throw new BusinessLayerException(
                    "Invalid database state; can't insert");
        }
        catch (SQLException ex)
        {
            throw new BusinessLayerException(ex);
        }
    }    
    
    /**
     * Create IManager object
     */
    public IManager createManager(
        String firstName, String lastName, String middleName,
        String ssn, IPosition position, int salary)
        throws BusinessLayerException, DuplicateObjectException
    {
        IEmployee employee = createEmployee(firstName, lastName,
            middleName, ssn, position, salary);
        if (employee == null)
            return null;
        
        return createManager(employee);
    }    

    /**
     * Create IManager from IEmployee
     */
    public IManager createManager(IEmployee employee)
        throws BusinessLayerException, DuplicateObjectException
    {
        try
        {
            // First cast off person into an RDBMSEmployee; in
            // theory, if employee is from a different model, we
            // could at this point insert the IPerson into the
            // RDBMS, but for this example, we just call it an 
            // exception and leave it at that.
            RDBMSEmployee dbEmp = (RDBMSEmployee)employee;
            
            //
            int mgrID = nextDatabaseID("manager");
            
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO manager " +
                "VALUES (" +
                    mgrID + ", " +
                    dbEmp.m_id + ")");
            
            if (result > 0)
                return new RDBMSManager(mgrID);
            else
                throw new BusinessLayerException(
                    "Invalid database state; can't insert");
        }
        catch (SQLException ex)
        {
            throw new BusinessLayerException(ex);
        }
    }    
    
    /**
     * Create IDepartment object
     */
    public IDepartment createDepartment(
        String name, IDepartment parent)
        throws BusinessLayerException, DuplicateObjectException
    {
        try
        {
            //
            int deptID = nextDatabaseID("dept");
            int parentDeptId = 0;
            if (parent != null)
            {
                // Again, if parent is from another model, we need
                // to either insert it into the database or else
                // throw an exception; to be consistent with the
                // other places in this example, I throw an
                // exception.
                RDBMSDepartment dbParentDept =
                    (RDBMSDepartment)parent;
                parentDeptId = dbParentDept.m_id;
            }
            
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO dept " +
                    "(ID, name, parent_dept_id_fk) " +
                "VALUES (" +
                    deptID + ", " +
                    "'" + name + "', " +
                    parentDeptId + ")");
            
            if (result > 0)
                return new RDBMSDepartment(deptID);
            else
                throw new BusinessLayerException(
                    "Invalid database state; can't insert");
        }
        catch (SQLException ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
    }    
    
    /**
     * Create IPosition object
     */
    public IPosition createPosition(
        String title, String desc, int minSalary, int maxSalary)
        throws BusinessLayerException, DuplicateObjectException
    {
        try
        {
            int posID = nextDatabaseID("position");
        
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "INSERT INTO position " +
                "VALUES (" +
                    posID + ", " +
                    "'" + title + "', " +
                    "'" + desc + "', " +
                    minSalary + ", " +
                    maxSalary + ", " +
                    "'')");
            
            if (result > 0)
                return new RDBMSPosition(posID);
            else
                throw new BusinessLayerException(
                    "Invalid database state; can't insert");
        }
        catch (SQLException ex)
        {
            throw new BusinessLayerException(ex);
        }
    }    


    /**
     * Query for all Persons
     */
    public IPerson[] findAllPersons()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT person.ID FROM person");
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int personID = rs.getInt("person.ID");

                IPerson p = new RDBMSPerson(personID);
                tempVector.addElement(p);
            }
            
            // Copy over from the Vector to the array to return
            IPerson retArray[] = new RDBMSPerson[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IPerson)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    /**
     * Query for all Persons by last name
     */
    public IPerson[] findPersonsByLastName(String lastName)
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT person.ID FROM person " +
                "WHERE person.l_name = '" + lastName + "'");
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int personID = rs.getInt("person.ID");

                IPerson p = new RDBMSPerson(personID);
                tempVector.addElement(p);
            }
            
            // Copy over from the Vector to the array to return
            IPerson retArray[] = new RDBMSPerson[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IPerson)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    /**
     * Query for all Persons by SSN
     */
    public IPerson[] findPersonsBySSN(String ssn)
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT person.ID FROM person " +
                "WHERE person.ssn = '" + ssn + "'");
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int personID = rs.getInt("person.ID");

                IPerson p = new RDBMSPerson(personID);
                tempVector.addElement(p);
            }
            
            // Copy over from the Vector to the array to return
            IPerson retArray[] = new RDBMSPerson[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IPerson)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }

    /**
     * Query for all Employees
     */
    public IEmployee[] findAllEmployees()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT employee.ID FROM employee");
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int employeeID = rs.getInt("employee.ID");

                IEmployee e = new RDBMSEmployee(employeeID);
                tempVector.addElement(e);
            }
            
            // Copy over from the Vector to the array to return
            IEmployee retArray[] = new RDBMSEmployee[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IEmployee)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    /**
     * Query for all Managers
     */
    public IManager[] findAllManagers()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT manager.ID FROM manager");
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int mgrID = rs.getInt("manager.ID");

                IManager e = new RDBMSManager(mgrID);
                tempVector.addElement(e);
            }
            
            // Copy over from the Vector to the array to return
            IManager retArray[] = new RDBMSManager[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IManager)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    /**
     * Query for all Positions
     */
    public IPosition[] findAllPositions()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT position.ID FROM position");
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int posID = rs.getInt("position.ID");

                IPosition e = new RDBMSPosition(posID);
                tempVector.addElement(e);
            }
            
            // Copy over from the Vector to the array to return
            IPosition retArray[] = new RDBMSPosition[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IPosition)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    /**
     * Query for all Departments
     */
    public IDepartment[] findAllDepartments()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT dept.ID FROM dept");
                
            Vector tempVector = new Vector();
            
            while (rs.next())
            {
                int deptID = rs.getInt("dept.ID");

                IDepartment e = new RDBMSDepartment(deptID);
                tempVector.addElement(e);
            }
            
            // Copy over from the Vector to the array to return
            IDepartment retArray[] = new RDBMSDepartment[tempVector.size()];
            for (int i=0; i<tempVector.size(); i++)
                retArray[i] = (IDepartment)tempVector.elementAt(i);
            
            return retArray;
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }
    
    /**
     * Query for "root" Department
     */
    public IDepartment findRootDepartment()
        throws BusinessLayerException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT dept.ID FROM dept " +
                "WHERE dept.parent_dept_id_fk = 0");
                
            if (rs != null && rs.next())
            {
                int deptID = rs.getInt("dept.ID");

                return new RDBMSDepartment(deptID);
            }
            else
                throw new BusinessLayerException("Root Department "+
                    "not found"); 
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }

        
    /**
     * Remove a Person from the system
     */
    public void removePerson(IPerson person)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        try
        {
            // First remove all contact information associated with
            // this Person; if we don't delete these first,
            // referential integrity will prevent us from removing
            // their corresponding 'person' row
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "DELETE FROM email " +
                "WHERE email.person_id_fk = " +
                ((RDBMSPerson)person).m_id);
            stmt.close();
            
            stmt = getConnection().createStatement();
            result = stmt.executeUpdate(
                "DELETE FROM phone " +
                "WHERE phone.person_id_fk = " + 
                ((RDBMSPerson)person).m_id);
            stmt.close();
            
            stmt = getConnection().createStatement();
            result = stmt.executeUpdate(
                "DELETE FROM address " +
                "WHERE address.person_id_fk = " + 
                ((RDBMSPerson)person).m_id);

            // Now we can remove the Person row itself
            stmt = getConnection().createStatement();
            result = stmt.executeUpdate(
                "DELETE FROM person " +
                "WHERE person.ID = " + 
                ((RDBMSPerson)person).m_id);

            if (result < 1)
                throw new UnknownObjectException();
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }    

    /**
     * Remove an Employee from the system
     */
    public void removeEmployee(IEmployee employee)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "DELETE FROM employee " +
                "WHERE employee.ID = " + 
                ((RDBMSEmployee)employee).m_id);

            if (result < 1)
                throw new UnknownObjectException();
            
            removePerson(employee);
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }    

    /**
     * Remove a Manager from the system
     */
    public void removeManager(IManager manager)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "DELETE FROM manager " +
                "WHERE manager.ID = " + 
                ((RDBMSManager)manager).m_id);

            if (result < 1)
                throw new UnknownObjectException();
                
            removeEmployee(manager);
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }    

    /**
     * Remove a Department from the system
     */
    public void removeDepartment(IDepartment dept)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "DELETE FROM dept " +
                "WHERE dept.ID = " + 
                ((RDBMSDepartment)dept).m_id);

            if (result < 1)
                throw new UnknownObjectException();
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }

    /**
     * Remove a Position from the system
     */
    public void removePosition(IPosition position)
        throws BusinessLayerException, IntegrityConstraintException,
            UnknownObjectException
    {
        try
        {
            Statement stmt = getConnection().createStatement();
            int result = stmt.executeUpdate(
                "DELETE FROM position " +
                "WHERE position.ID = " + 
                ((RDBMSPosition)position).m_id);

            if (result < 1)
                throw new UnknownObjectException();
        }
        catch (SQLException x)
        {
            x.printStackTrace();
            throw new BusinessLayerException(x);
        }
    }


    // Package-friendly helper methods
    //
    /**
     *
     */
    static int findPersonIDFromEmployeeID(int empID)
        throws BusinessLayerException
    {
        // Retrieve the Person.ID from the given empID
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT person.ID FROM person, employee " +
                "WHERE employee.person_id_fk = person.ID " +
                    "AND employee.ID = " + empID);
                
            if (rs.next())
            {
                int personID = rs.getInt("person.ID");
                return personID;
            }
        }
        catch (SQLException ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
        return -1;
    }
    /**
     *
     */
    static int findEmployeeIDFromManagerID(int mgrID)
        throws BusinessLayerException
    {
        // Retrieve the Employee.ID from the given mgrID
        try
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT employee.ID FROM employee, manager " +
                "WHERE manager.employee_id_fk = employee.ID " +
                    "AND manager.ID = " + mgrID);
                
            if (rs.next())
            {
                int empID = rs.getInt("employee.ID");
                return empID;
            }
        }
        catch (SQLException ex)
        {
            ex.printStackTrace();
            throw new BusinessLayerException(ex);
        }
        return -1;
    }
    
    /**
     *
     */
    static int nextDatabaseID(String tableName)
        throws BusinessLayerException
    {
        try
        {
            // Retrieve the ID
            int dbID = 0;
            
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(
                "SELECT MAX(id) FROM " + tableName);
            if (rs.next())
                dbID = rs.getInt("MAX(id)");
            else
                return 1;
            
            // Increment it and return it
            return ++dbID;
        }
        catch(SQLException ex)
        {
            throw new BusinessLayerException(ex);
        }
    }

    /**
     *
     */    
    static String SSNToDB(String ssn)
    {
        // Transform the readable form of 'ssn' into an
        // all-numeric form for storage
        String newSSN = new String();
        
        for (int i=0; i<ssn.length(); i++)
            if (Character.isDigit(ssn.charAt(i)))
                newSSN += ssn.charAt(i);
                
        return newSSN;
    }
    /**
     *
     */    
    static String SSNFromDB(String ssn)
    {
        // Present SSN in a readable state
        String ret = ssn.substring(0, 3);
        ret += "/";
        ret += ssn.substring(3, 5);
        ret += "/";
        ret += ssn.substring(5);
        
        return ret;
    }
    
    /**
     *
     */    
    static String PhoneToDB(String phone)
    {
        // Transform the readable form of 'phone' into an
        // all-alphanumeric form for storage
        String newPhone = new String();
        
        for (int i=0; i<phone.length(); i++)
            if (Character.isLetterOrDigit(phone.charAt(i)))
                newPhone += phone.charAt(i);
                
        return newPhone;
    }
    /**
     *
     */    
    static String PhoneFromDB(String phone)
    {
        String ret = phone.substring(0, 3);
        ret += "-";
        ret += phone.substring(3);
        
        return ret;
    }
}    