-- This script creates an IDB database for the Employee example used by
-- this chapter (5) as well as chapters 6 and 7. Other scripts contain
-- the necessary DDL for other databases (EmployeeOraLiteSchema.sql for
-- Oracle Lite, EmployeeAccessSchema.sql for Access via ODBC, and so on).

--
-- Generic SQL for creating the Employee Schema
-- (will probably not work "off the shelf" with your RDBMS; almost no
-- RDBMS evaluates SQL in precisely the same fashion)
--

--
-- Drop all tables
--
DROP TABLE manager;
DROP TABLE employee;
DROP TABLE person;
DROP TABLE manager_employees;
DROP TABLE dept;
DROP TABLE position;
DROP TABLE phone;
DROP TABLE email;
DROP TABLE address;

--
-- Create the schema
--

-- Table: person
CREATE TABLE person
(
	ID INTEGER PRIMARY KEY,
	f_name VARCHAR(80),
	m_name VARCHAR(255),
	l_name VARCHAR(80),
	ssn CHAR(9)
);

-- Table: employee
CREATE TABLE employee
(
	ID INTEGER PRIMARY KEY,
	person_id_fk INTEGER FOREIGN KEY,
	position_id_fk INTEGER FOREIGN KEY,
	salary INTEGER
);
-- FOREIGN KEY (person_id_fk) REFERENCES person
-- FOREIGN KEY (position_id_fk) REFERENCES position


-- Table: manager
CREATE TABLE manager
(
	ID INTEGER PRIMARY KEY,
	employee_id_fk INTEGER FOREIGN KEY
);
-- FOREIGN KEY (employee_id_fk)	REFERENCES employee

-- Table: manager_employees
CREATE TABLE manager_employees
(
	manager_id_fk INTEGER FOREIGN KEY,
	employee_id_fk INTEGER FOREIGN KEY
);
-- FOREIGN KEY (manager_id_fk) REFERENCES manager
-- FOREIGN KEY (employee_id_fk) REFERENCES employee

-- Table: dept
CREATE TABLE dept
(
	ID INTEGER PRIMARY KEY,
	name VARCHAR(80),
	manager_id_fk INTEGER FOREIGN KEY,
	description VARCHAR(255),
	parent_dept_id_fk INTEGER
)
-- FOREIGN KEY (manager_id_fk) REFERENCES manager

-- Table: position
CREATE TABLE position
(
	ID INTEGER PRIMARY KEY,
	title VARCHAR(80),
	description VARCHAR(255),
	min_salary INTEGER,
	max_salary INTEGER,
	category VARCHAR(20)
);

-- Table: address
CREATE TABLE address
(
	ID INTEGER PRIMARY KEY,
	person_id_fk INTEGER FOREIGN KEY,
	description VARCHAR(255),
	street VARCHAR(255),
	city VARCHAR(255),
	state VARCHAR(80),
	postal_code VARCHAR(20),
	country VARCHAR(40)
);
-- FOREIGN KEY (person_id_fk) REFERENCES person

-- Table: email
CREATE TABLE email
(
	ID INTEGER PRIMARY KEY,
	person_id_fk INTEGER FOREIGN KEY,
	description VARCHAR(255),
	email VARCHAR(80)
);
-- FOREIGN KEY (person_id_fk) REFERENCES person;

-- Table: phone
CREATE TABLE phone
(
	ID INTEGER PRIMARY KEY,
	person_id_fk INTEGER FOREIGN KEY,
	description VARCHAR(255),
	prefix VARCHAR(40),
	number VARCHAR(7),
	suffix VARCHAR(20)
);
-- FOREIGN KEY (person_id_fk) REFERENCES person
