import java.net.Socket;

/**
 * JDBCTermConnection
 */
public class JDBCTermConnection
{
    public static void main(String[] args)
        throws Exception
    {
        JDBCTermConnection jtc =
            new JDBCTermConnection();
            
        jtc.setInput(System.in);
        jtc.setPrompt(System.out);
        
        jtc.prompt().println("JDBCTerm--local version 1.0");
        
        // Enter infinite loop
        while (jtc.m_shouldQuit == false)
            jtc.executeCommand(jtc.getUserInput());
    }

    /**
     * Offer service on the given socket. Remain open until told to quit
     * by the user (via the @QUIT command).
     */
    public void serve(Socket socket)
        throws Exception
    {
        // Create local stream objects
        setInput(socket.getInputStream());
        setPrompt(socket.getOutputStream());
        
        // Print JDBCTerm prompt
        prompt().println("JDBCTerm version 1.0");
        
        // Enter infinite loop
        while (m_shouldQuit == false)
            executeCommand(getUserInput());
    }
    
    /**
     * Quit the connection (that is, close it down and return)
     */
	public void quit()
	{ m_shouldQuit = true; }
    boolean m_shouldQuit = false;
    
	// Input stream functionality
	//
	public java.io.BufferedReader input()
	{ return m_input; }
	public void setInput(java.io.InputStream in)
	{
		m_inputStream = in; 
		m_input = new java.io.BufferedReader(new java.io.InputStreamReader(m_inputStream));
	}
	public java.io.InputStream getInputStream()
	{ return m_inputStream; }
	private java.io.BufferedReader m_input = null;
	private java.io.InputStream m_inputStream = null;

	// Output stream functionality
	//
	public java.io.PrintWriter prompt()
	{ return m_prompt; }
	public java.io.PrintWriter output()
	{ return m_output; }
	public void setOutput(java.io.OutputStream out)
	{ m_outputStream = out; m_output = new java.io.PrintWriter(m_outputStream, true); }
	public void setPrompt(java.io.OutputStream prompt)
	{
		m_promptStream = prompt;
		m_prompt = new java.io.PrintWriter(m_promptStream, true);
		
		// If m_output hasn't been set yet, set it to point here
		if (output() == null)
			setOutput(prompt);
	}
	public java.io.OutputStream getOutputStream()
	{ return m_outputStream; }
	public java.io.OutputStream getPromptStream()
	{ return m_promptStream; }
	private java.io.OutputStream m_outputStream = null;
	private java.io.PrintWriter m_output = null;
	private java.io.OutputStream m_promptStream = null;
	private java.io.PrintWriter m_prompt = null;

    /**
     * Obtain a line of input from the user. Will not return until a semicolon
     * is entered.
     */
	public String getUserInput()
	{
		String tmp = new String("");
		String cmd = new String("");
		
		// Keep reading from input until the line is terminated with a semicolon
        // KNOWN BUG: If the line ends with something OTHER than a semicolon,
        // we keep right on going. Whitespace after the semicolon is OK, but any
        // non-trim()-able characters will cause us to continue waiting.
		int lineCt = 1;
		while (true)
		{
			if (lineCt == 1)
				prompt().print(" >");
			else
				prompt().print(lineCt + ">");
			prompt().flush();

			// Get the next line of input
			try
			{
				tmp = input().readLine();
			}
			catch (java.io.IOException ex)
			{
				ex.printStackTrace(output());
				System.exit(1);
			}

			// We can't handle null Strings from here on down
			if (tmp == null)
				return(tmp);

			// Strip comments from input line
			tmp = stripComments(tmp);

			// Strip whitespace on either end
			tmp.trim();

			// If we're terminated with a semicolon, then return
			if (tmp.endsWith(";"))
			{
				// Add tmp to cmd
				cmd += tmp;
				// Send it back, with semicolon
				return cmd;
			}
			else
			{
				// Add tmp to cmd
				cmd += tmp + " ";
				// Increment the line count
				lineCt++;
			}
		}
	}
	private String stripComments(String in)
	{
		// If there's nothing there, then just return it
		if (in == null)
			return(in);
			
		// If we're in the middle of a slash-star comment,
		// run through to the end of the line or the closing
		// slash-star
		if (slashStarCommentsOn)
		{
			if (in.length() > 0 && in.indexOf("*/") > -1)
			{
				// Comment closes somewhere in here; strip everything
				// up until it
				in = in.substring(in.indexOf("*/") + 2, in.length());
				slashStarCommentsOn = false;
			}
			else
			{
				in = "";
			}
		}

		// slash-slash comments run to end of line
		if (in.length() > 0 && in.indexOf("//") > -1)
		{
			in = in.substring(0, in.indexOf("//"));
		}

		// dash-dash comments run to end of line
		if (in.length() > 0 && in.indexOf("--") > -1)
		{
			in = in.substring(0, in.indexOf("--"));
		}

		// slash-star comments run to star-slash
		while (in.length() > 0 && in.indexOf("/*") > -1)
		{
			if (in.indexOf("*/") > -1)
			{
				// Comment terminates within the same line; just strip
				// out everything in between the /* and the */
				in = in.substring(0, in.indexOf("/*")) + 
					in.substring(in.indexOf("*/") + 2, in.length());
			}
			else
			{
				// Comment runs pas the end of the line; set the slash-star
				// flag to true, and strip everything after the slash-star
				in = in.substring(0, in.indexOf("/*"));
				slashStarCommentsOn = true;
				break;
			}
		}
		
		return in;
	}
	private boolean slashStarCommentsOn = false;


	/**
     * User-input execution facilities
	 */
	public void executeCommand(String c)
	{
		// Gracefully return if c is null
		if (c == null || c.length() == 0)
			return;

		// Handle the possibility that, after comment-stripping,
		// there's no real content left
		c = c.trim();
		if (c.equals(""))
			return;

		// Handle the possibility that there could be more than
		// one command on a line
		java.util.StringTokenizer st = new java.util.StringTokenizer(c, ";");
		while (st.hasMoreTokens())
		{
			String cmd = st.nextToken();
			cmd.trim();

			if (cmd == null || cmd.length() == 0)
				break;

			// Walk through the Command array, and see if anybody
			// wants to handle this command
			boolean foundIt = false;
			for (int i=0; i<m_commandArray.length; i++)
			{
				cmd = cmd.trim();
				if (m_commandArray[i].acceptCmd(cmd))
				{
					try
					{
						m_commandArray[i].execute(cmd, this);
						foundIt = true;
						break;
					}
					catch (CommandSyntaxException ex)
					{
						output().println("Improper syntax: " + ex);
						m_commandArray[i].displayHelp(output());
						foundIt = true;
						break;
					}
				}
			}

			if (foundIt == false)
				output().println("Unrecognized command: " + cmd);
		}
		
		return;
	}
	public Command[] getCommandArray()
	{ return m_commandArray; }
	private Command m_commandArray[] =
	{
        new LogCommand(),
    
		// Database operations
		new OpenCommand(),
		new CloseCommand(),
		new CommitCommand(),
		new RollbackCommand(),
		
		// JDBC operations
		//new PrepareCommand(),
		//new CallCommand(),
		//new ReleaseCommand(),
		new RegisterCommand(),
		
		// JDBCTerm operations
		new ExecCommand(),
		new HelpCommand(),
		new QuitCommand(),
		//new MetadataCommand.instance(),
		//new DescCommand.instance(),
		
		// SQL operations
		new SQLSelectCommand(),
		new SQLInsertCommand(),
		new SQLUpdateCommand(),
		new SQLDeleteCommand(),
		new SQLPassthroughCommand()
	};
	
	
	// Database connections
	//
	public java.sql.Connection getConnection()
	{ return m_connection; }
	public void setConnection(java.sql.Connection conn)
	{ m_connection = conn; }
	private java.sql.Connection m_connection = null;
}



//-------------------------------------------------------------------
// Command classes
//-------------------------------------------------------------------

// Base class for JDBCTerm command keywords
//
abstract class Command
{
	Command(String cmdString)
	{ m_cmdString = cmdString; }
	
	public boolean acceptCmd(String cmd)
	{
		if (cmd.toUpperCase().startsWith(m_cmdString.toUpperCase()))
			return true;
		else
			return false;
	}
	
	public abstract void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException;
	public abstract void displayHelp(java.io.PrintWriter ps);
	
	private String m_cmdString;
}

// Exception class for improper Command-syntax
class CommandSyntaxException extends java.lang.Exception
{
	public CommandSyntaxException()
	    { super(); }
	
	public CommandSyntaxException(String s)
        { super(s); }
}


// Command to shut down the JDBCTerm instance
// Valid syntax:
//		@QUIT
//
class QuitCommand extends Command
{
	public QuitCommand()
	{ super("@QUIT"); }
	
	public static QuitCommand instance()
	{ return s_instance; }
	private static QuitCommand s_instance = new QuitCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		// Commit any pending transactions
		new CommitCommand().execute("@COMMIT", jdbcTerm);
	
		// Close any open connections
		new CloseCommand().execute("@CLOSE", jdbcTerm);
		
		jdbcTerm.output().println("Exiting JDBCTerm");
		
		// Quit JDBCTerm
		jdbcTerm.quit();
	}	

	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Quit JDBCTerm: @QUIT;");
	}
}

// Command to call commit() on connection
// Valid syntax:
//		@COMMIT
//
class CommitCommand extends Command
{
	public CommitCommand()
	{ super("@COMMIT"); }

	public static CommitCommand instance()
	{ return s_instance; }
	private static CommitCommand s_instance = new CommitCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		try
		{
			if (jdbcTerm.getConnection() != null)
				jdbcTerm.getConnection().commit();
		}
		catch (java.lang.NullPointerException ex)
		{
			jdbcTerm.output().println("No open connection; commit unsuccessful.");
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception on commit:" + ex.getMessage());
			ex.printStackTrace(jdbcTerm.output());
		}
	}	

	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Commit data on connection: @COMMIT;");
	}
}

// Command to call rollback() on connection
// Valid syntax:
//		@ROLLBACK
//
class RollbackCommand extends Command
{
	public RollbackCommand()
	{ super("@ROLLBACK"); }
	
	public static RollbackCommand instance()
	{ return s_instance; }
	private static RollbackCommand s_instance = new RollbackCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		try
		{
			if (jdbcTerm.getConnection() != null)
				jdbcTerm.getConnection().rollback();
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception on rollback:");
			ex.printStackTrace(jdbcTerm.output());
		}
	}	

	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Rollback data on connection: @ROLLBACK;");
	}
}

// Command to open a connection to a URL
// Valid syntax:
//		@OPEN url [uid pwd]
//
class OpenCommand extends Command
{
	public OpenCommand()
	{ super("@OPEN"); }
	
	public static OpenCommand instance()
	{ return s_instance; }
	private static OpenCommand s_instance = new OpenCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		// Tokenize cmd
		java.util.StringTokenizer st = new java.util.StringTokenizer(cmd);
		try
		{
			String c = st.nextToken();
			String url = st.nextToken();
            
			String uid = "";
			String pwd = "";

            try
            {
			    uid = st.nextToken();
			    pwd = st.nextToken();
            }
    		catch (java.util.NoSuchElementException ex)
            {
                // uid and pwd can remain defaulted as ""
            }
			
			try
			{
				// Close connections if we're already connected
				if (jdbcTerm.getConnection() != null)
					jdbcTerm.getConnection().close();
				
				// Attempt new connection
				java.sql.Connection conn =
					java.sql.DriverManager.getConnection(url, uid, pwd);
				
				// If we're still here, it was good--set it in JDBCTerm
				jdbcTerm.setConnection(conn);
				jdbcTerm.output().println("Connection successful.");
				
				// Display vendor's name and version & driver's name and version
				java.sql.DatabaseMetaData dbmd = conn.getMetaData();
				String db_name = dbmd.getDatabaseProductName();
				String db_ver = dbmd.getDatabaseProductVersion();
				String drv_name = dbmd.getDriverName();
				String drv_ver = dbmd.getDriverVersion();
				jdbcTerm.output().println("Database: " + db_name + " " + db_ver);
				jdbcTerm.output().println("Driver: " + drv_name + " " + drv_ver);
			}
			catch (java.sql.SQLException ex)
			{
				jdbcTerm.output().println("Connection failed:");
				ex.printStackTrace();
			}
		}
		catch (java.util.NoSuchElementException ex)
		{
			throw new CommandSyntaxException();
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Open new connection: @OPEN jdbc-url [user-id [user-password]];");
	}
}

// Command to close the open connection
// Valid syntax:
//		@CLOSE
//
class CloseCommand extends Command
{
	public CloseCommand()
	{ super("@CLOSE"); }
	
	public static CloseCommand instance()
	{ return s_instance; }
	private static CloseCommand s_instance = new CloseCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		try
		{
			// Close connections if we're already connected
			if (jdbcTerm.getConnection() != null)
			{
				jdbcTerm.getConnection().close();
				jdbcTerm.setConnection(null);
				jdbcTerm.output().println("Connection closed.");
			}
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception on close:" + ex.getMessage());
			ex.printStackTrace();
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Close open connection: @CLOSE;");
	}
}

// Command to open a file and start executing its contents as if it were being
// typed at the prompt() stream
// Valid syntax:
//	@EXEC filename
//
class ExecCommand extends Command
{
	public ExecCommand()
	{ super("@EXEC"); }
	
	public static ExecCommand instance()
	{ return s_instance; }
	private static ExecCommand s_instance = new ExecCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		// Point JDBCTerm's input stream to a file, instead of STDIN
		//
		java.util.StringTokenizer st = new java.util.StringTokenizer(cmd);
		String filename = new String("");
		try
		{
			String c = st.nextToken();
			filename = st.nextToken();

			// Open file
			java.io.FileInputStream fileInput =
				new java.io.FileInputStream(filename);

			// Save old input; insert new input
			java.io.InputStream oldInput = jdbcTerm.getInputStream();
			jdbcTerm.setInput(fileInput);

			// Save old prompt; we want to do prompting ourselves (?)
			TeeOutputStream emptyTee = new TeeOutputStream();
			java.io.OutputStream oldPrompt = jdbcTerm.getPromptStream();
			jdbcTerm.setPrompt(emptyTee);

			// Execute until we're done (out of file stuff)
			while (jdbcTerm.input().ready())
			{
				String input = jdbcTerm.getUserInput();
				input = input.trim();
				jdbcTerm.output().println(filename + ":> " + input);
				jdbcTerm.executeCommand(input);
			}
			jdbcTerm.output().println("End of input file");

			// Restore old input, prompt
			jdbcTerm.setInput(oldInput);
			jdbcTerm.setPrompt(oldPrompt);
            
            // Close file stream
            fileInput.close();
		}
		catch (java.io.FileNotFoundException ex)
		{
			jdbcTerm.output().println("File " + filename + " not found. Aborting.");
			return;
		}
		catch (java.io.IOException ex)
		{
			jdbcTerm.output().println("IOException caught.");
			return;
		}
	}
	
	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Execute script file: @EXEC filename;");
	}
}

// Command to register a new JDBC driver
// Valid syntax:
//		@REGISTER driver-class-name
//
class RegisterCommand extends Command
{
	public RegisterCommand()
	{ super("@REGISTER"); }
	
	public static RegisterCommand instance()
	{ return s_instance; }
	private static RegisterCommand s_instance = new RegisterCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		// Tokenize cmd
		java.util.StringTokenizer st = new java.util.StringTokenizer(cmd);
		try
		{
			String command = st.nextToken();
			String driver_class_name = st.nextToken();
			
			try
			{
				Class.forName(driver_class_name);
				jdbcTerm.output().println("Driver installed.");
			}
			catch (java.lang.ClassNotFoundException ex)
			{
				jdbcTerm.output().println("Registration unsuccessful -- ClassNotFoundException");
			}
			catch (java.lang.IllegalArgumentException ex)
			{
				jdbcTerm.output().println("Registration unsuccessful -- IllegalArgumentException");
			}
		}
		catch (java.util.NoSuchElementException ex)
		{
			throw new CommandSyntaxException("Expecting: driver-class-name");
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Register JDBC driver: @REGISTER driver-class-name;");
	}
}

// Command to turn on/off input/output logging
// Valid syntax:
//	@LOG FILE filename
//	@LOG ON|OFF
class LogCommand extends Command
{
	public LogCommand()
	{ super("@LOG"); }
	
	public static LogCommand instance()
	{ return s_instance; }
	private static LogCommand s_instance = new LogCommand();
	
	public boolean acceptCmd(String cmd)
	{
		// Are we currently logging?
		if (m_logFileStream != null)
		{
			// Write the input to m_logFileStream
			m_logStream.println(" > " + cmd);
		}
		
		if (cmd.toUpperCase().startsWith("@LOG"))
			return true;
		else
			return false;
	}
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		// Tokenize cmd
		java.util.StringTokenizer st = new java.util.StringTokenizer(cmd);
		try
		{
			String command = st.nextToken();
			String keyword = st.nextToken();

			if (keyword.toUpperCase().equals("ON"))
			{
				if (m_logFileStream == null)
				{
					// Capture original OutputStream
					m_originalOS = jdbcTerm.getOutputStream();
					
					// Build FileOutputStream
					try
					{
						m_logFileStream = new java.io.FileOutputStream(m_logFilename);
						m_logStream = new java.io.PrintWriter(m_logFileStream, true);
					}
					catch (java.io.IOException ex)
					{
						jdbcTerm.output().println("Unexpected exception caught; logging not started:");
						ex.printStackTrace(jdbcTerm.output());
						
						// Clear everything out
						m_originalOS = null;
						m_logFileStream = null;
						m_logStream = null;
						
						return;
					}
					
					// Build TeeOutputStream
					TeeOutputStream teeOS = new TeeOutputStream();
					teeOS.addOutputStream(m_originalOS);
					teeOS.addOutputStream(m_logFileStream);

					// Set TeeOutputStream to output()
					jdbcTerm.setOutput(teeOS);
					
					// Notify user that logging started
					jdbcTerm.output().println("Logging started.");
				}
			}
			else if (keyword.toUpperCase().equals("OFF"))
			{
				if (m_logFileStream != null)
				{
					// Notify user logging terminated
					jdbcTerm.output().println("Logging terminated.");
				
					// Reset output() to the original
					// OutputStream
					jdbcTerm.setOutput(m_originalOS);
					
					// Clear out current objects
					try
					{
						m_logFileStream.close();
					}
					catch (java.io.IOException ex)
					{
						// Not much to do, really; we're closing, after all
					}

					m_logStream = null;
					m_logFileStream = null;
					m_originalOS = null;
				}
			}
			else if (keyword.toUpperCase().equals("FILE"))
			{
				m_logFilename = st.nextToken();
				
				// If we're already logging, close the existing
				// log stream, then reopen it with the new
				// filename
				if (m_logFileStream != null)
				{
				}
			}
			else
			{
				throw new CommandSyntaxException("Expected: filename");
			}
		}
		catch (java.util.NoSuchElementException ex)
		{
			throw new CommandSyntaxException("Expected one of: ON, OFF, FILE");
		}
	}
	

	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Start logging: @LOG ON;");
		ps.println("Stop logging: @LOG OFF;");
		ps.println("Set logfile: @LOG FILE filename;");
	}

	private java.io.FileOutputStream m_logFileStream = null;
	private java.io.PrintWriter m_logStream = null;
	private String m_logFilename = "JDBCTERM.LOG";
	private java.io.OutputStream m_originalOS = null;
}

// Command to display help strings for all Commands
// Syntax:
//	@HELP
class HelpCommand extends Command
{
	public HelpCommand()
	{ super("@HELP"); }

	public static HelpCommand instance()
	{ return s_instance; }
	private static HelpCommand s_instance = new HelpCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
	{
		Command array[] = jdbcTerm.getCommandArray();
		for (int i=0; i<array.length; i++)
			array[i].displayHelp(jdbcTerm.output());
	}
	
	public void displayHelp(java.io.PrintWriter ps)
	{
		ps.println("Display help: @HELP;");
	}	
}


// Command to handle SELECT SQL statements
//
class SQLSelectCommand extends Command
{
	public SQLSelectCommand()
	{ super("SELECT"); }
	
	public static SQLSelectCommand instance()
	{ return s_instance; }
	private static SQLSelectCommand s_instance = new SQLSelectCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		if (jdbcTerm.getConnection() == null)
		{
			jdbcTerm.output().println("No open connection; aborting.");
			return;
		}
	
		// Build a Statement, execute() it, display the results
		java.sql.Statement stmt = null;
		
		try
		{
			stmt = jdbcTerm.getConnection().createStatement();
			
			// Execute the query and get the results
			java.sql.ResultSet rs = stmt.executeQuery(cmd);
            if (rs != null)
    			displayResults(jdbcTerm.output(), rs);
            else
                jdbcTerm.output().println("No ResultSet");
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception:" + ex.getMessage());
			ex.printStackTrace(jdbcTerm.output());
		}
		finally
		{
			if (stmt != null)
			{
				try { stmt.close(); }
				catch (java.sql.SQLException ex) { }
			}
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		// Maybe display some simple SQL SELECT help here?
	}
	
	public void displayResults(java.io.PrintWriter ps, java.sql.ResultSet results)
	{
		try
		{
			java.sql.ResultSetMetaData meta = results.getMetaData();
			int cols = meta.getColumnCount();
			
			// Display of columns returned by SQL SELECT statements can be
			// a tricky thing. Too many columns, written on one virtual line,
			// and the display gets messy; however, with only one or two
			// columns per row (and a lot of rows returned), and it makes
			// sense to display them in a semi-gridlike format. Thus I adopt
			// the following approach: if the number of columns returned can
			// be fit on a single line (assuming 80 text columns across),
			// then I'll display them in gridlike (a la SQL*Plus) fashion. If
			// not, then it's a single row-column per line.
			//
			int width = 0;
			for (int i=1; i<=cols; i++)
				width += meta.getColumnDisplaySize(i) + 1;
			int row_count = 0;	// This is built during the display

			if (width > 80)
			{
				// Generate the row&column per line output
				//
			
				// Save off column labels, to minimze metadata retrieval
				String colLabelArray[] = new String[cols];
				for (int i=1; i<=cols; i++)
					colLabelArray[i-1] = meta.getColumnLabel(i);
				
				// Display each column of each row on its own line
				while (results.next())
				{
					// Tally up the number of rows
					row_count++;
					
					// Iterate over all the columns
					for (int i=1; i<=cols; i++)
					{
						// Display "column name: data"
						ps.print(colLabelArray[i-1] + ": ");
						
						// So long as the column isn't wider than 156
						// characters - the length of the label, display
						// the entire data string; otherwise, truncate
						String data = results.getString(i);
						if (data == null)
							data = new String("(null)");
						else if (data.length() + colLabelArray[i-1].length() > 156)
							data = data.substring(0, 
								(156 - colLabelArray[i-1].length())) + "+";
						
						ps.println(data);
						ps.flush();
					}
				}
			}
			else
			{
				// Generate the SQL*Plus-lookalike output
				//
				
				// Print the column labels
				String buffer = new String();
				for (int i=1; i<=cols; i++)
				{
					String label = meta.getColumnLabel(i);
	
					int size = meta.getColumnDisplaySize(i);
			
					// If the label is long than the column is wide,
					// then we truncate the column label
					if (label.length() > size)
						label = label.substring(0, size);

					// If the label is shorter than the column, pad it with spaces
					else if (label.length() < size)
						for (int j=label.length(); j<=size; j++)
							label += " ";
	
					// Add the column header to the buffer
					buffer += label;
				}

				// create the bar that is as long as the total of all columns
				String bar = new String("");
				for (int i=1; i<=cols; i++)
				{
					width = meta.getColumnDisplaySize(i);
	
					while (width-- > 0)
						bar += "-";
					bar += " ";
				}
				
				ps.println(buffer);
				ps.println(bar);
	
				// Format each row in the result set and add it on
				while( results.next() )
				{
					row_count++;
			
					// Format each column of the row
					String buf = new String("");
					for (int i=1; i<=cols; i++)
					{
						String str = results.getString(i);
						int size = meta.getColumnDisplaySize(i);
			    
                        if (str != null)
                        {
    						if (str.length() > size)
    							str = str.substring(0, size);
    						else if (str.length() < size)
    							for (int j=str.length(); j<size; j++)
    								str += " ";
						
    						buf += str + " ";
                        }
                        else
                            buf += "(null) ";
					}
					ps.println(buf);
				}
			}
			
			// Display number of rows returned
			if (row_count == 0)
				ps.println("No rows selected.");
			else if (row_count == 1)
				ps.println("1 row selected.");
			else
				ps.println(row_count + " rows selected.");
		}
		catch (java.sql.SQLException ex)
		{
			ps.println("Unexpected exception: " + ex.getMessage());
			ex.printStackTrace(ps);
		}
		finally
		{
			try { results.close(); }
			catch (java.sql.SQLException ex) { }
		}
	}
}

// Command to handle INSERT SQL statements
//
class SQLInsertCommand extends Command
{
	public SQLInsertCommand()
	{ super("INSERT"); }
	
	public static SQLInsertCommand instance()
	{ return s_instance; }
	private static SQLInsertCommand s_instance = new SQLInsertCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		if (jdbcTerm.getConnection() == null)
		{
			jdbcTerm.output().println("No open connection; aborting.");
			return;
		}
	
		// Build a Statement, execute() it, display the results
		java.sql.Statement stmt = null;
		
		try
		{
			stmt = jdbcTerm.getConnection().createStatement();
			
			// Execute the insert and get the results
			int rowCount = stmt.executeUpdate(cmd);
			switch (rowCount)
			{
				case -1:
				case 0:
					jdbcTerm.output().println("No rows inserted");
					break;
				case 1:
					jdbcTerm.output().println("1 row inserted.");
					break;
				default:
					jdbcTerm.output().println(rowCount + " rows inserted.");
			}
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception:" + ex.getMessage());
			ex.printStackTrace(jdbcTerm.output());
		}
		finally
		{
			if (stmt != null)
			{
				try { stmt.close(); }
				catch (java.sql.SQLException ex) { }
			}
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		// Display nothing
	}
}

// Command to handle UPDATE SQL statements
//
class SQLUpdateCommand extends Command
{
	public SQLUpdateCommand()
	{ super("UPDATE"); }
	
	public static SQLUpdateCommand instance()
	{ return s_instance; }
	private static SQLUpdateCommand s_instance = new SQLUpdateCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		if (jdbcTerm.getConnection() == null)
		{
			jdbcTerm.output().println("No open connection; aborting.");
			return;
		}
	
		// Build a Statement, execute() it, display the results
		java.sql.Statement stmt = null;
		
		try
		{
			stmt = jdbcTerm.getConnection().createStatement();
			
			// Execute the insert and get the results
			int rowCount = stmt.executeUpdate(cmd);
			switch (rowCount)
			{
				case -1:
				case 0:
					jdbcTerm.output().println("No rows updated");
					break;
				case 1:
					jdbcTerm.output().println("1 row updated.");
					break;
				default:
					jdbcTerm.output().println(rowCount + " rows updated.");
			}
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception:" + ex.getMessage());
			ex.printStackTrace(jdbcTerm.output());
		}
		finally
		{
			if (stmt != null)
			{
				try { stmt.close(); }
				catch (java.sql.SQLException ex) { }
			}
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		// Display nothing
	}
}

// Command to handle DELETE SQL statements
//
class SQLDeleteCommand extends Command
{
	public SQLDeleteCommand()
	{ super("DELETE"); }
	
	public static SQLDeleteCommand instance()
	{ return s_instance; }
	private static SQLDeleteCommand s_instance = new SQLDeleteCommand();
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		if (jdbcTerm.getConnection() == null)
		{
			jdbcTerm.output().println("No open connection; aborting.");
			return;
		}
	
		// Build a Statement, execute() it, display the results
		java.sql.Statement stmt = null;
		
		try
		{
			stmt = jdbcTerm.getConnection().createStatement();
			
			// Execute the insert and get the results
			int rowCount = stmt.executeUpdate(cmd);
			switch (rowCount)
			{
				case -1:
				case 0:
					jdbcTerm.output().println("No rows deleted");
					break;
				case 1:
					jdbcTerm.output().println("1 row deleted.");
					break;
				default:
					jdbcTerm.output().println(rowCount + " rows deleted.");
			}
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception:" + ex.getMessage());
			ex.printStackTrace(jdbcTerm.output());
		}
		finally
		{
			if (stmt != null)
			{
				try { stmt.close(); }
				catch (java.sql.SQLException ex) { }
			}
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		// Display nothing
	}
}

// Command to pass the string directly through to the database, with no
// translation attempted by JDBCTerm; MUST be the last Command in the
// Command array, since it will match against anything!
//
class SQLPassthroughCommand extends Command
{
	public SQLPassthroughCommand()
	{ super(""); }
	
	public static SQLPassthroughCommand instance()
	{ return s_instance; }
	private static SQLPassthroughCommand s_instance = new SQLPassthroughCommand();
	
	public boolean acceptCmd(String cmd)
	{ return true; }
	
    public void execute(String cmd, JDBCTermConnection jdbcTerm)
		throws CommandSyntaxException
	{
		// Build a Statement, execute() it, display the results
		java.sql.Statement stmt = null;
		
		try
		{
			if (jdbcTerm.getConnection() == null)
			{
				jdbcTerm.output().println("No connection open; aborting.");
				return;
			}
			
			stmt = jdbcTerm.getConnection().createStatement();
			
			if (stmt.execute(cmd))
			{
				// Must've been a SELECT of some form
				jdbcTerm.output().println("Result set returned.");
			}
			else
			{
				// Display how many rows were affected
				int num;
				
				switch (num = stmt.getUpdateCount())
				{
					case -1:
						jdbcTerm.output().println("Statement successful.");
						break;
					case 0:
						jdbcTerm.output().println("No rows affected.");
						break;
					case 1:
						jdbcTerm.output().println("1 row affected.");
						break;
					default:
						jdbcTerm.output().println(num + " rows affected.");
						break;
				}
			}
		}
		catch (java.sql.SQLException ex)
		{
			jdbcTerm.output().println("Unexpected exception:" + ex.getMessage());
			ex.printStackTrace(jdbcTerm.output());
		}
		finally
		{
			if (stmt != null)
			{
				try { stmt.close(); }
				catch (java.sql.SQLException ex) { }
			}
		}
	}

	public void displayHelp(java.io.PrintWriter ps)
	{
		// Display nothing
	}
}


//-------------------------------------------------------------------
// Utility classes
//-------------------------------------------------------------------
/**
 * Class designed to "tee" output to multiple OutputStream objects
 *
 * @version 1.0
 * @author Ted Neward
 */
class TeeOutputStream extends java.io.OutputStream
{
	/**
	 * Builds a TeeOutputStream with no "captued" streams
	 */
	public TeeOutputStream()
	{ }
	
	/**
	 * Builds a TeeOutputStream with the streams passed already
	 * "captured" by the TeeOutputStream
	 */
	public TeeOutputStream(java.io.OutputStream streams[])
	{
		for (int i=0; i<streams.length; i++)
			addOutputStream(streams[i]);
	}
	
	/**
	 * Adds the OutputStream passed into the list of tee'ed streams
	 */
	public void addOutputStream(java.io.OutputStream os)
	{
		m_outputStreamList.addElement(os);
	}
	
	/**
	 * Writes a character to each of the OutputStreams "captured"
	 */
	public void write(int b) 
		throws java.io.IOException
	{
		for (int i=0; i<m_outputStreamList.size(); i++)
		{
			java.io.OutputStream os =
				(java.io.OutputStream)m_outputStreamList.elementAt(i);
			os.write(b);
		}
	}
	
	/**
	 * Calls flush() on each of the OutputStreams "captured"
	 */
	public void flush()
		throws java.io.IOException
	{
		for (int i=0; i<m_outputStreamList.size(); i++)
		{
			java.io.OutputStream os =
				(java.io.OutputStream)m_outputStreamList.elementAt(i);
			os.flush();
		}
	}

	/**
	 * Closes all of the OutputStreams "captured"
	 */
	public void close()
		throws java.io.IOException
	{
		for (int i=0; i<m_outputStreamList.size(); i++)
		{
			java.io.OutputStream os =
				(java.io.OutputStream)m_outputStreamList.elementAt(i);
			os.close();
		}
	}
	
	private java.util.Vector m_outputStreamList = new java.util.Vector(5,1);
}

