import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import Employee.*;


/**
 *
 */
class DepartmentTreeModel
    implements TreeModel
{
    private DepartmentTreeModel()
    { }
        // Hidden to prevent no-arg construction

    /**
     *
     */
    public DepartmentTreeModel(IEmployeeModel model)
    {
        m_model = model;
    }
    
    /**
     * TreeModel interface method.
     */
    public void addTreeModelListener(TreeModelListener l)
    {
        m_listeners.addElement(l);
    }
    /**
     * TreeModel interface method.
     */
    public Object getChild(Object parent, int index)
    {
        try
        {
            IDepartment dept = (IDepartment)parent;
            IDepartment[] children = dept.getContainedDepartments();
            if (children != null)
            {
                return children[index];
            }
            else
                return null;
        }
        catch (BusinessLayerException blEx)
        {
            blEx.printStackTrace();
            return null;
        }
    }
    /**
     * TreeModel interface method.
     */
    public int getChildCount(Object parent)
    {
        try
        {
            IDepartment dept = (IDepartment)parent;
            IDepartment[] children = dept.getContainedDepartments();
            if (children != null)
                return children.length;
            else
                return 0;
        }
        catch (BusinessLayerException blEx)
        {
            blEx.printStackTrace();
            return 0;
        }
    }
    /**
     * TreeModel interface method.
     */
    public int getIndexOfChild(Object parent, Object child)
    {
        return 0;
    }
    /**
     * TreeModel interface method.
     */
    public Object getRoot()
    {
        try
        {
            return m_model.findRootDepartment();
        }
        catch (BusinessLayerException blEx)
        {
            blEx.printStackTrace();
            return null;
        }
    }
    /**
     * TreeModel interface method.
     */
    public boolean isLeaf(Object node)
    {
        try
        {
            IDepartment dept = (IDepartment)node;
            if (dept.getContainedDepartments() != null)
                return false;
            else
                return true;
        }
        catch (BusinessLayerException blEx)
        {
            blEx.printStackTrace();
            return true;
        }
    }
    /**
     * TreeModel interface method.
     */
    public void removeTreeModelListener(TreeModelListener l)
    {
        m_listeners.remove(l);
    }
    /**
     * TreeModel interface method.
     */
    public void valueForPathChanged(TreePath path, Object newValue)
    {
        System.out.println("valueForPathChanged: " + path + " " + 
            newValue);
    }
    
    
    // Internal members
    IEmployeeModel m_model;
    Vector m_listeners = new Vector();
}    


/**
 *
 */
public class OrgTree
{
    public static void main(String[] args)
    {
        JFrame frame = new JFrame("Employee: OrgTree");
        JTree tree = new JTree(
            new DepartmentTreeModel(getEmployeeModel(args)));
        frame.getContentPane().add(tree);
        frame.pack();
        frame.show();
    }
    
    private static IEmployeeModel getEmployeeModel(String[] args)
    {
        IEmployeeModel model = null;
        if (args != null)
        {
            for (int i=0; i<args.length; i++)
            {
                if (args[i].equals("-model:RDBMS"))
                {
                    // Set up JDBC Connection
                    java.sql.Connection conn = null;
                    try
                    {
                        // Load the JDBC driver into the JVM
                        Class.forName("jdbc.idbDriver");
                        java.util.Properties p = 
                            new java.util.Properties();
                        conn = 
                            java.sql.DriverManager.getConnection(
                                "jdbc:idb:employee.prp", p);
                    }
                    catch (Exception ex)
                    {
                        ex.printStackTrace();
                        return null;
                    }
                    model = new RDBMSModel(conn);
                }
                if (args[i].equals("-model:Hashtable"))
                {
                    model = new HashtableModel();
                }
            }
        }
        
        return model;
    }
}    