import java.awt.*;
import java.awt.event.*;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.*;
import com.javageeks.classloader.PluginClassLoader;

/**
 *
 */
public class PluginApp
{
    // Private data
    //
    private JFrame frame;
    private static Vector plugins = new Vector();
    private PluginClassLoader pluginCL;
    private transient boolean canQuit;
        // State variable used in method exit(); should be modified
        // *only* within that context and not used elsewhere.

    /**
     * Plugins must implement this interface; the app will call
     * the plugin when appropriate.
     */
    public static interface Plugin
    {
        public void addToMenuBar(JMenuBar menu);
        public boolean canQuit();
    }

    
    /**
     * Plugins make themselves known to the App by calling this
     * method.
     */
    public static void registerPlugin(PluginApp.Plugin plugin)
    {
        // Just keep a reference to it for future use
        plugins.add(plugin);
    }

    
    /**
     * This is an interface to ease calling across all the plugins
     * in the system.
     */
    protected static interface PluginAction
    {
        public void action(Plugin plugin);
    }
    /**
     * General-purpose method for calling an action across all the
     * currently-registered plugins.
     */
    private void doPlugins(PluginAction pluginAction)
    {
        for (Iterator iter = plugins.iterator(); iter.hasNext(); )
        {
            System.out.println("calling addToMenuBar()");
            Plugin p = (Plugin)iter.next();
            pluginAction.action(p);
        }
    }
    
    
    /**
     *
     */
    public PluginApp()
    {
    }


    /**
     *
     */
    public PluginClassLoader.Listener getPluginListener()
    {
        return new PluginClassLoader.Listener()
        {
            public void pluginLoaded(String pluginName)
            {
                System.out.println(pluginName + " loaded.");
            }
            public void exception(Exception ex)
            {
                System.out.println("Exception:");
                ex.printStackTrace();
            }
        };
    }

    
    /**
     * Display the application
     */
    public void show()
    {
        frame = new JFrame("PluginApp Example");
        frame.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                exit();
            }
        });

        // Allow the plugins to register themselves with the app
        pluginCL = new PluginClassLoader(".", getPluginListener());

        JPanel contentPanel = new JPanel();
        contentPanel.add("North", createMenubar());
        
        frame.getContentPane().add(contentPanel);

        frame.pack();
        frame.show();
    }


    /**
     *
     */
    public void exit()
    {
        canQuit = true;
        doPlugins(new PluginAction()
        {
            public void action(Plugin plugin)
            {
                if (plugin.canQuit() == false)
                {
                    canQuit = false;
                }
            }
        });

        if (canQuit)
        {
            System.exit(0);
        }
    }

    
    /**
     * Build the application-shell's menubar; just "File" and "Help"
     */
    private JMenuBar createMenubar()
    {
        final JMenuBar mb = new JMenuBar();
        JMenu menu;
        JMenuItem mi;

        // "File"--"Reload"
        menu = new JMenu("File");
        mi = new JMenuItem("Reload");
        mi.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                pluginCL.reload(".",getPluginListener());
            }
        });
        menu.add(mi);
        
        // "File"--"Exit"
        mi = new JMenuItem("Exit");
        mi.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                exit();
            }
        });
        menu.add(mi);

        mb.add(menu);


        // "Help"--"About"
        menu = new JMenu("Help");
        mi = new JMenuItem("About");
        mi.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
            }
        });
        menu.add(mi);
        mb.add(menu);

        // Allow the Plugins to register themselves
        doPlugins(new PluginAction()
        {
            public void action(Plugin plugin)
            {
                plugin.addToMenuBar(mb);
            }
        });

        return mb;
    }

    
    /**
     *
     */
    public static void main (String args[])
    {
        // Create the basic app object
        PluginApp app = new PluginApp();

        // Display the app
        app.show();
    }
}
