// JPrimes.java: Java front-end to calculating prime numbers
//
import java.awt.*;

public class JPrimes extends Frame
    implements java.awt.event.ActionListener
{
        public static void main(String[] args)
        {
                // Build the GUI frame
                //
                JPrimes f = new JPrimes();
                f.show();
        }
        
        
        // GUI-related public interface
        //
        JPrimes()
        {
                /*
                 * Local initialization
                 */
                super("Java vs. C++ Primes calculation sample");
                
                // Turn off layout manager--we'll do it ourselves
                setLayout(null);
                reshape(100, 100, 310, 200);
                
                // Set height & width
                
                /*
                 * Insert controls
                 */
                // Close button
                m_btnClose = new Button("Close");
                m_btnClose.reshape(5, 25, 100, 25);
                m_btnClose.setActionCommand("Close");
                m_btnClose.addActionListener(this);
                this.add(m_btnClose);

                // "Java" button
                m_btnJava = new Button("Java");
                m_btnJava.reshape(105, 25, 100, 25);
                m_btnJava.setActionCommand("Java");
                m_btnJava.addActionListener(this);
                this.add(m_btnJava);

                // "Native" button
                m_btnNative = new Button("Native");
                m_btnNative.reshape(205, 25, 100, 25);
                m_btnNative.setActionCommand("Native");
                m_btnNative.addActionListener(this);
                this.add(m_btnNative);

                // Labels for calculation times
                Label l = new Label("Time started:");
                l.reshape(5, 55, 100, 20);
                this.add(l);
                l = new Label("Time ended:");
                l.reshape(5, 75, 100, 20);
                this.add(l);
                
                // Calculation time labels
                m_lblTimeStarted = new Label("");
                m_lblTimeStarted.reshape(105, 55, 100, 20);
                this.add(m_lblTimeStarted);
                
                m_lblTimeStopped = new Label("");
                m_lblTimeStopped.reshape(105, 75, 100, 20);
                this.add(m_lblTimeStopped);
                
                m_lblPrimes = new Label("");
                m_lblPrimes.reshape(5, 100, 300, 20);
                this.add(m_lblPrimes);
        }
        
        // ActionListener
        //
        public void actionPerformed(java.awt.event.ActionEvent e)
        {
                if (e.getActionCommand() == "Close")
                {
                        dispose();
                        System.exit(0);
                }
                else if (e.getActionCommand() == "Java")
                {
                        m_lblPrimes.setText("");
                        
                        // Note starting time
                        java.util.Date start = new java.util.Date();
                        
                        String result = calculatePrimes(50000);
                        
                        // Note ending time
                        java.util.Date end = new java.util.Date();
                        
                        m_lblTimeStarted.setText(start.getHours() + ":" + start.getMinutes() + ":" + start.getSeconds());
                        m_lblTimeStopped.setText(end.getHours() + ":" + end.getMinutes() + ":" + end.getSeconds());
                        m_lblPrimes.setText(result);
                }
                else if (e.getActionCommand() == "Native")
                {
                        m_lblPrimes.setText("");
                        
                        // Note starting time
                        java.util.Date start = new java.util.Date();
        
            String result = nativeCalculatePrimes(50000);
                        
                        // Note ending time
                        java.util.Date end = new java.util.Date();
                        
                        m_lblTimeStarted.setText(start.getHours() + ":" + start.getMinutes() + ":" + start.getSeconds());
                        m_lblTimeStopped.setText(end.getHours() + ":" + end.getMinutes() + ":" + end.getSeconds());
                        m_lblPrimes.setText(result);
                }
        }


        // Internal implementation
        //      
        public String calculatePrimes(int stop)
        {
                String primes = new String();
                
                // Algorithm cribbed from Sedgewick's "Algorithms in C++"
                int i;
                int j;
                int a[] = new int[stop + 1];
                
                for (a[1]=0, i=2; i<= stop; i++)
                        a[i] = 1;
                        
                for (i=2; i<stop/2; i++)
                        for (j=2; j<=stop/i; j++)
                                a[i*j] = 0;
                                
                for (i=1; i<=stop; i++)
                        if (a[i] != 0)
                                primes += i + " ";
                
                return primes;
        }
        public native String nativeCalculatePrimes(int stop);
        static
        {
                System.loadLibrary("JPrimes");
        }


        // Internal data
        //
        private Button m_btnClose;
        private Button m_btnJava;
        private Button m_btnNative;
        private Label m_lblTimeStarted;
        private Label m_lblTimeStopped;
        private Label m_lblPrimes;
}