#include "JVM.h"

// StdC++ headers
#include <iostream>
using namespace std;
// StdC headers
#include <stdlib.h>



//using namespace JavaGeeks::JNI;



//--------------------------------------------------- Impl interface
//
class Impl
{
    // Object lifetime mgmt
    //
public:
    static Impl* getImpl(bool debug);
        // Only means by which an implementation object can be 
        // obtained; this preserves portability


    // API
    //
public:
    virtual bool loadJVM(const char* JVMType) = 0;
    virtual jint createJavaVM(JavaVM**, void** env, void* args) = 0;
    virtual jint getDefaultJavaVMInitArgs(void* env) = 0;
};



//---------------------------------------- JVMOptions implementation
//
JVMOptions::JVMOptions()
    : m_options(0), m_optionsCt(0), m_optionsSz(0)
{ }
JVMOptions::~JVMOptions()
{
    if (m_optionsSz > 0)
        delete [] m_options;
}

void JVMOptions::add(const char* opt)
{
    resize();

    m_options[m_optionsCt].optionString = opt;
    m_options[m_optionsCt++].extraInfo = 0;
}
void JVMOptions::add(const char* opt, void* extra)
{
    resize();

    m_options[m_optionsCt].optionString = opt;
    m_options[m_optionsCt++].extraInfo = extra;
}
void JVMOptions::remove(const char* opt)
{
    // No-op for the moment
}

void JVMOptions::resize()
{
    /*
     * Expand options array if needed to accomodate at least one more
     * VM option.
     */
    if (m_optionsCt >= m_optionsSz)
    {
        if (m_optionsSz == 0)
        {
            m_optionsSz = 4;
            m_options = new JavaVMOption[m_optionsSz];
        }
        else
        {
            m_optionsSz *= 2;
            JavaVMOption* tmp = new JavaVMOption[m_optionsSz];
            memcpy(tmp, options, numOptions * sizeof(JavaVMOption));
            if (m_options)
                delete [] m_options;
            m_options = tmp;
        }
    }
}

JavaVMOption* JVMOptions::getOptions()
{
    return m_options;
}
int JVMOptions::getOptionsCt()
{
    return m_optionsCt;
}



//----------------------------------------------- JVM implementation
//
JVM::JVM()
    : m_debug(false), m_impl(0), m_javaVM(0), m_jniEnv(0), 
      m_jvmType(0), m_classpath(0)
{
    // Check environment variable for debug status
    //
    if (getenv("_JAVA_LAUNCHER_DEBUG") != 0)
        debug(true);
}
JVM::~JVM()
{
    if (m_javaVM && m_jniEnv)
    {
        m_javaVM->DestroyJavaVM();
        m_javaVM = 0;
        m_jniEnv = 0;
    }

    delete m_impl;
    if (m_jvmType)
        delete [] m_jvmType;
    if (m_classpath)
        delete [] m_classpath;
}


Impl* JVM::impl()
{
    if (m_impl == 0)
        m_impl = Impl::getImpl(debug());
    
    return m_impl;
}


const char* JVM::jvmType()
{
    return m_jvmType;
}
void JVM::jvmTye(const char* setting)
{
    if (setting == m_jvmType)
        return;

    if (m_jvmType != 0)
        delete [] m_jvmType;

    m_jvmType = new char[strlen(setting)+1];
    strcpy(m_jvmType, setting);
}

bool JVM::debug()
{
    return m_debug;
}
void JVM::debug(bool setting)
{
    m_debug = setting;
}

const char* JVM::classpath()
{
    return m_classpath;
}
void JVM::classpath(const char* cp)
{
    if (cp == m_classpath)
        return;

    if (m_classpath != 0)
        delete [] m_classpath;

    static const char* cpPrefix = "-Djava.class.path=";
    m_classpath = new char[strlen(cpPrefix)+strlen(cp)+1];
    strcpy(m_classpath, cpPrefix);
    strcat(m_classpath, cp);
}

/*
bool JVM::verboseGC()
{
}
void JVM::verboseGC(bool setting)
{
}
bool JVM::verboseClass()
{
}
void JVM::verboseClass(bool setting)
{
}
bool JVM::verboseJNI()
{
}
void JVM::verboseJNI(bool setting)
{
}

const char* JVM::property(const char* name)
{
}
void JVM::property(const char* name, const char* value)
{
}

const char* JVM::option(const char* name)
{
}
void JVM::option(const char* name, const char* value)
{
}
*/


JavaVM* JVM::getJavaVM()
{
    return m_javaVM;
}
JNIEnv* JVM::getJNIEnv()
{
    return m_jniEnv;
}


bool JVM::initializeJVM()
{
    impl()->loadJVM(m_jvmType);

    JavaVMInitArgs args;
    initializeJVMArgs(args);
    
    jint r = impl()->createJavaVM(&m_javaVM, (void **)&m_jniEnv, &args);
    return (r == JNI_OK);
}
void JVM::initializeJVMArgs(JavaVMInitArgs& args)
{
    int numOptions = 0;

    if (m_classpath != 0)
        numOptions++;

    JavaVMOption* options = (JavaVMOption*)NULL;

    if (numOptions > 0)
    {
        options = new JavaVMOption[numOptions];

        int n=0;

        if (m_classpath != 0)
            options[n++].optionString = m_classpath;
    }

    memset(&args, 0, sizeof(args));
    args.version  = JNI_VERSION_1_2;
    args.nOptions = numOptions;
    args.options  = options;
    args.ignoreUnrecognized = JNI_TRUE;

    if (m_debug)
    {
        int i = 0;
        cout << "JavaVM args:" << endl;
        cout << "    version " << hex << args.version << endl;
        cout << "    ignoreUnrecognized is "
             << (args.ignoreUnrecognized ? "JNI_TRUE" : "JNI_FALSE") 
             << endl;
        cout << "    nOptions is " << args.nOptions << endl;
        for (i = 0; i < args.nOptions; i++)
            cout << "    option[" << i << "] = '"
                 << args.options[i].optionString << "'" << endl;
    }
}
jclass JVM::loadClass(const char* name)
{
    char *buf = new char[strlen(name) + 1];
    char *s = buf;
    char *t = const_cast<char*>(name);
    char c;

    jclass cls;
    jlong start, end;

//    if (m_debug)
//        start = CounterGet();

    do
    {
        c = *t++;
        *s++ = (c == '.') ? '/' : c;
    } while (c != '\0');
    cls = m_jniEnv->FindClass(buf);
    delete [] buf;

    /*
    if (m_debug)
    {
        end = CounterGet();
        
        cout << (jint)Counter2Micros(end-start) 
             << " micro seconds to load class" << endl; 
        cout << "----_JAVA_LAUNCHER_DEBUG----" << endl;
    }
    */

    return cls;
}


/*
bool JVM::executeJAR(const char* jarFile)
{
}
*/
bool JVM::executeClass(const char* className, int argc, char* argv[])
{
    jclass mainClass = loadClass(className);
    if (mainClass == 0)
    {
        getJNIEnv()->ExceptionDescribe();
        return false;
    }

    jmethodID mainID = getClassMain(mainClass);
    if (mainID == NULL)
    {
        getJNIEnv()->ExceptionDescribe();
        return false;
    }

    jobjectArray mainArgs;
    mainArgs = javaStringArray(argc, argv);
    if (mainArgs == NULL)
    {
        getJNIEnv()->ExceptionDescribe();
        return false;
    }

    getJNIEnv()->CallStaticVoidMethod(mainClass, mainID, mainArgs);
    if (getJNIEnv()->ExceptionOccurred())
    {
        /* Formerly, we used to call the "uncaughtException" method of the
           main thread group, but this was later shown to be unnecessary
           since the default definition merely printed out the same exception
           stack trace as ExceptionDescribe and could never actually be
           overridden by application programs. */
        getJNIEnv()->ExceptionDescribe();
        return false;
    }

    return true;
}
/*
bool JVM::executeRaw(int argc, char* argv[])
{
}
*/


jmethodID JVM::getClassMain(jclass cls)
{
    jmethodID mainID = 
        getJNIEnv()->GetStaticMethodID(cls, "main", "([Ljava/lang/String;)V");
    return mainID;
}
jstring JVM::javaString(char *s)
{
    int len = strlen(s);
    jclass cls;
    jmethodID mid;
    jbyteArray ary;

    cls = getJNIEnv()->FindClass("java/lang/String");
    mid = getJNIEnv()->GetMethodID(cls, "<init>", "([B)V");
    ary = getJNIEnv()->NewByteArray(len);
    if (ary != 0)
    {
        jstring str = 0;
        getJNIEnv()->SetByteArrayRegion(ary, 0, len, (jbyte *)s);
        if (!getJNIEnv()->ExceptionOccurred())
        {
            str = (jstring)getJNIEnv()->NewObject(cls, mid, ary);
        }
        getJNIEnv()->DeleteLocalRef(ary);
        return str;
    }
    return 0;
}
jobjectArray JVM::javaStringArray(int strc, char** strv)
{
    jclass cls;
    jarray ary;
    int i;

    cls = getJNIEnv()->FindClass("java/lang/String");
    ary = getJNIEnv()->NewObjectArray(strc, cls, 0);
    for (i = 0; i < strc; i++)
    {
        jstring str = javaString(*strv++);
        getJNIEnv()->SetObjectArrayElement((jobjectArray)ary, i, str);
        getJNIEnv()->DeleteLocalRef(str);
    }
    return (jobjectArray)ary;
}



//---------------------------------------- Win32 Impl implementation
//
#ifdef WIN32

#include <string>
using namespace std;

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <atlbase.h>
#include <atlimpl.cpp>


class Win32Impl : public Impl
{
public:
    Win32Impl(bool debug);
    virtual ~Win32Impl();

public:
    virtual bool loadJVM(const char* JVMType);
    virtual jint createJavaVM(JavaVM**, void** env, void* args);
    virtual jint getDefaultJavaVMInitArgs(void* env);

private:
    string getAppHome();
    string findJREPath();

private:
    typedef jint (JNICALL *CreateJavaVMFnPtr)(JavaVM **pvm, void **env, void *args);
    CreateJavaVMFnPtr m_createJavaVMFnPtr;

    typedef jint (JNICALL *GetDefaultJavaVMInitArgsFnPtr)(void *args);
    GetDefaultJavaVMInitArgsFnPtr m_getDefaultJavaVMInitArgsFnPtr;

    bool m_debug;
};


#define DOTRELEASE  "1.2" /* Same for 1.2.1, 1.2.2 etc. */
#define JRE_KEY     "Software\\JavaSoft\\Java Runtime Environment"

#ifdef DEBUG
#define JVM_DLL "jvm_g.dll"
#define JAVA_DLL "java_g.dll"
#else
#define JVM_DLL "jvm.dll"
#define JAVA_DLL "java.dll"
#endif


Win32Impl::Win32Impl(bool debug)
    : m_debug(debug)
{ }
Win32Impl::~Win32Impl()
{ }


string Win32Impl::getAppHome()
{
    char buf[255];
    int bufsize = sizeof(buf);
    
    char *cp;
    ::GetModuleFileName(0, buf, bufsize);

    string name(buf);

    // remove "<filename>.exe" part
    name = name.substr(0, name.find_last_of('\\'));

    // check if this is a drive-root
    if (name.find_last_of('\\') == string::npos)
    {
        name += '\\';
    }
    else
    {
        // it's not, so check for a '\bin' directory
        string bindir = name.substr(name.find_last_of('\\'), name.length());
        bindir = strlwr(const_cast<char*>(bindir.c_str()));
            // ick--have to worry about case
        
        if (bindir == "\\bin")
            name = name.substr(0, name.find_last_of('\\'));
    }

    return name;
}
string Win32Impl::findJREPath()
{
    string appHome = getAppHome();

    // Is JRE co-located with application?
    //
    struct stat s;
    string javadll = appHome + "\\bin\\" JAVA_DLL;
    if (stat(javadll.c_str(), &s) == 0)
    {
        return javadll;
    }

    // Is JRE in a private directory? (Does this app ship a private
    // JRE in <apphome>\jre directory?)
    //
    javadll = appHome + "\\jre\\bin\\" JAVA_DLL;
    if (stat(javadll.c_str(), &s) == 0)
    {
        javadll += "\\jre";
        return javadll;
    }

    // Is JRE "public" (installed in Win32 Registry)?
    //
    CRegKey key;
    char version[255]; DWORD versionLen = sizeof(version);

    // Find the current version of the JRE
    if (key.Open(HKEY_LOCAL_MACHINE, JRE_KEY) != ERROR_SUCCESS || 
        key.QueryValue(version, "CurrentVersion", &versionLen) != ERROR_SUCCESS)
    {
        return "";
    }
    
    if (strcmp(version, DOTRELEASE) != 0)
    {
        return "";
    }

    // Find directory where the current version is installed.
    CRegKey subkey;
    char buf[255]; DWORD bufLen = sizeof(buf);
    if (subkey.Open(key, version) != ERROR_SUCCESS ||
        subkey.QueryValue(buf, "JavaHome", &bufLen) != ERROR_SUCCESS)
    {
        return "";
    }

    javadll = buf;

    if (m_debug)
    {
        char micro[255]; DWORD microLen = sizeof(micro);
        if (subkey.QueryValue("MicroVersion", micro, &microLen) != ERROR_SUCCESS)
            micro[0]='\0';
        cout << "Version major.minor.micro = " << version << "." << micro << endl;
    }

    return javadll;
}
bool Win32Impl::loadJVM(const char* JVMType)
{
    HINSTANCE handle;
    
    string jvmtype;
    string jvmdll;

    string jrePath = findJREPath();
    if (jrePath == "")
    {
        // JRE couldn't be found
        return false;
    }

    // Determine if Hotspot VM is installed.
    //
    if (JVMType == NULL)
    {
        struct stat s;
        jvmdll = jrePath + "\\bin\\hotspot\\" JVM_DLL;
        if (stat(const_cast<char*>(jvmdll.c_str()), &s) < 0)
            jvmtype = "classic";
        else
            jvmtype = "hotspot";
    }
    else
        jvmtype = JVMType;

    // We now know what jvmtype should be
    jvmdll = jrePath + "\\bin\\" + jvmtype + "\\" JVM_DLL;
    if (m_debug)
        cout << "Path to JVM is " << jvmdll << endl;

    // Load the Java VM DLL
    //
    if ((handle = LoadLibrary(const_cast<char*>(jvmdll.c_str()))) == 0)
    {
        //fprintf(stderr, "Error loading: %s\n", jvmdll);
        cout << "Error loading " << jvmdll << ":" << (DWORD)::GetLastError() << endl;
        return false;
    }

    // Now get the function addresses
    //
    m_createJavaVMFnPtr =
        (CreateJavaVMFnPtr)GetProcAddress(handle, "JNI_CreateJavaVM");
    m_getDefaultJavaVMInitArgsFnPtr =
        (GetDefaultJavaVMInitArgsFnPtr)GetProcAddress(handle, "JNI_GetDefaultJavaVMInitArgs");
    if (m_createJavaVMFnPtr == 0 ||
        m_getDefaultJavaVMInitArgsFnPtr == 0)
    {
        cout <<  "Can't find JNI interfaces in: " << jvmdll << endl;
        return false;
    }

    return true;
}
jint Win32Impl::createJavaVM(JavaVM** javaVM, void** env, void* args)
{
    return (*m_createJavaVMFnPtr)(javaVM, env, args);
}
jint Win32Impl::getDefaultJavaVMInitArgs(void* env)
{
    return (*m_getDefaultJavaVMInitArgsFnPtr)(env);
}


Impl* Impl::getImpl(bool debug)
{
    return new Win32Impl(debug);
}

#endif



//---------------------------------------- SunOS Impl implementation
//
#ifdef __SUNOS


class SunOSImpl : public Impl
{
public:
    SunOSImpl(bool debug);
    virtual ~SunOSImpl();

private:
    bool m_debug;
};


SunOSImpl::SunOSImpl(bool debug)
    : m_debug(debug)
{ }
SunOSImpl::~SunOSImpl()
{ }


Impl* Impl::getImpl(bool debug)
{
    return new SunOSImpl(debug);
}

#endif



//---------------------------------------- Linux Impl implementation
//
#ifdef __LINUX


class LinuxImpl : public Impl
{
public:
    LinuxImpl(bool debug);
    virtual ~LinuxImpl();

private:
    bool m_debug;
};


LinuxImpl::LinuxImpl(bool debug)
    : m_debug(debug)
{ }
LinuxImpl::~LinuxImpl()
{ }


Impl* Impl::getImpl(bool debug)
{
    return new LinuxImpl(debug);
}

#endif



//------------------------------------------------- Test driver code
//
int main(int argc, char* argv[])
{
    // Create JVM
    //
    JVM jvm;

    //jvm.debug(true);
    jvm.classpath(".");

    // Build JVM
    //
    if (jvm.initializeJVM() &&
        jvm.executeClass(argv[1], 0, (NULL)))
        return 0;
    else
        return 1;
}

