/*
 * 
 */
#ifndef __JAVAGEEKS_JVM_H
#define __JAVAGEEKS_JVM_H


// StdC++ headers
#include <deque>
// JNI headers
#include <jni.h>


// Wrap in a namespace
/*
namespace JavaGeeks
{


// Wrap in a sub-namespace
namespace JNI
{
*/


class Impl;
    

class JVMOptions
{
public:
    JVMOptions();
    ~JVMOptions();

public:
    void add(const char* opt);
    void add(const char* opt, void* extra);
    void remove(const char* opt);


public:
    JavaVMOption* getOptions();
    int getOptionsCt();

private:
    void resize(int newSz);

private:
    JavaVMOption* m_options;
    int m_optionsCt;
    int m_optionsSz;
};


/*
 * 
 */
class JVM
{
    // Object lifetime mgmt
    //
public:
    JVM();
    ~JVM();
        // Non-virtual--don't inherit from this!


    // Accessors/Mutators
    //
public:
    const char* jvmType();
    void jvmTye(const char*);
        // "classic" or "hotspot"

    bool debug();
    void debug(bool setting);
        // For turning on debug messages from within the JVM

    const char* classpath();
    void classpath(const char* cp);
        // For "-classpath $CLASSPATH:/usr/local/classes" settings

    bool verboseGC();
    void verboseGC(bool setting);
    bool verboseClass();
    void verboseClass(bool setting);
    bool verboseJNI();
    void verboseJNI(bool setting);
        // For "-verbose:{gc | class | jni}" settings

    const char* property(const char* name);
    void property(const char* name, const char* value);
        // For "-Dproperty=value" settings

    const char* option(const char* name);
    void option(const char* name, const char* value);
        // For "-X..." (nonstandard) settings


    // JNI API wrappers
    //
public:
    bool initializeJVM();
    jclass loadClass(const char* className);


    // JNI Data access
    //
public:
    JavaVM* getJavaVM();
    JNIEnv* getJNIEnv();
    

    // Convenience methods
    //
public:
    bool executeJAR(const char* jarFile, int argc, char* argv[]);
    bool executeClass(const char* className, int argc, char* argv[]);
    bool executeRaw(int argc, char* argv[]);


    // Internal methods
    //
private:
    Impl* impl();
    void initializeJVMArgs(JavaVMInitArgs& args);
    jmethodID getClassMain(jclass cls);
    jstring javaString(char *s);
    jobjectArray javaStringArray(int strc, char** strv);
    

    // Internal data
    //
private:
    Impl* m_impl;
    JavaVM* m_javaVM;
    JNIEnv* m_jniEnv;
    
    bool m_debug;
    char* m_jvmType;
    char* m_classpath;
    JVMOptions m_options;
};


//};  // end namespace "JavaGeeks::JNI"

//};  // end namespace "JavaGeeks"


#endif
