#include <windows.h>

#include <string>
#include <iostream>
using namespace std;

#include "NamedPipeInputStream.h"

class NamedPipe
{
    // Object lifetime
    //
public:
    NamedPipe();
    NamedPipe(const string& name);
    ~NamedPipe();

    // Read/write methods
    //
public:
    DWORD open(const string& name);
    void close();

    // Read methods
    //
public:
    int available();
    int read();
    DWORD read(char* buffer, int bufferLen, long offset, long length);
    DWORD skip(long numBytes);

    // Write methods
    //
public:
    DWORD write(LPVOID buffer, int bufferLen);

    // Internal data
    //
private:
    HANDLE hPipe;
    DWORD dwMode;
};

NamedPipe::NamedPipe()
    : hPipe(0)
{ }
NamedPipe::NamedPipe(const string& name)
{
    open(name);
}
NamedPipe::~NamedPipe()
{
    if (hPipe != (HANDLE)0)
        close();
}

DWORD NamedPipe::open(const string& name)
{
    // Close the connection if we're already connected
    if (hPipe != (HANDLE)0)
        close();

    // Attempt to open the pipe
    char pipeName[80];
    strcpy(pipeName, "\\\\.\\pipe\\");
    strcpy(pipeName, name.c_str());

    hPipe = ::CreateFile(
        pipeName,
        GENERIC_READ | GENERIC_WRITE,
        0,
        NULL,
        OPEN_EXISTING,
        0,
        NULL);
    if (hPipe == INVALID_HANDLE_VALUE)
        return GetLastError();
    else
    {
        dwMode = PIPE_READMODE_MESSAGE;
        BOOL fSuccess = ::SetNamedPipeHandleState(hPipe, &dwMode, NULL, NULL);
        if (!fSuccess)
            return GetLastError();
    }

    return 0;
}
void NamedPipe::close()
{
    if (hPipe != (HANDLE)0)
        ::CloseHandle(hPipe);
    hPipe = (HANDLE)0;
}

int NamedPipe::available()
{
    DWORD bytesAvail;
    DWORD bytesAvailThisMessage;
    BOOL fSuccess = ::PeekNamedPipe(hPipe, NULL, 0, NULL, &bytesAvail, NULL);
    if (fSuccess)
        return bytesAvail;
    else
        return -1;
}
int NamedPipe::read()
{
    if (hPipe == (HANDLE)0)
        return -1;

    CHAR chBuf[512];
    DWORD cbRead;
    BOOL fSuccess = ::ReadFile(hPipe, chBuf, 1, &cbRead, NULL);
    if (!fSuccess)
        return -1;
    else
        return chBuf[0];
}
DWORD NamedPipe::read(char* buffer, int bufferLen, long offset, long length)
{
    if (hPipe == (HANDLE)0)
        return -1;

    // Read offset bytes (and throw the results away)
    if (offset > 0)
        skip(offset);

    // Read length bytes
    DWORD cbRead;
    BOOL fSuccess = ::ReadFile(hPipe, buffer, bufferLen, &cbRead, NULL);
    if (!fSuccess)
        return -1;
    else
        return cbRead;
}    
DWORD NamedPipe::skip(long numBytes)
{
    // Read numBytes bytes and throw the results away
    CHAR* chBuf = new CHAR[numBytes + 1];
    DWORD cbRead;
    BOOL fSuccess = ::ReadFile(hPipe, chBuf, numBytes, &cbRead, NULL);
    delete [] chBuf;

    return cbRead;
}

DWORD NamedPipe::write(LPVOID buffer, int bufferLen)
{
    DWORD cbWritten;
    BOOL fSuccess = ::WriteFile(hPipe, buffer, bufferLen, &cbWritten, NULL);
    if (!fSuccess)
        return -1;
    else
        return cbWritten;
}    


// TEST TEST TEST
void main(int argc, char* argv[])
{
    cout << "Entering" << endl;

    NamedPipe client("mynamedpipe");
    string msg((argc > 1) ? argv[1] : "default message");
    client.write((void*)msg.c_str(), strlen(msg.c_str()));

    CHAR cbBuf[512];
    while (client.read(cbBuf, sizeof(cbBuf), 0, sizeof(cbBuf)) != -1)
    {
        cout << cbBuf;
    }

    cout << "Exiting" << endl;
}    


/*
 * Class:     NamedPipeInputStream
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_NamedPipeInputStream_initIDs
  (JNIEnv* env, jclass cls)
{
    // Get IDs for JNI fields, methods
}

/*
 * Class:     NamedPipeInputStream
 * Method:    available
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_NamedPipeInputStream_available
  (JNIEnv* env, jobject thisPtr)
{
    return 0;
}

/*
 * Class:     NamedPipeInputStream
 * Method:    close
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_NamedPipeInputStream_close
  (JNIEnv* env, jobject thisPtr)
{
    
}

/*
 * Class:     NamedPipeInputStream
 * Method:    open
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_NamedPipeInputStream_open
  (JNIEnv* env, jobject thisPtr, jstring name)
{
    
}

/*
 * Class:     NamedPipeInputStream
 * Method:    read
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_NamedPipeInputStream_read
  (JNIEnv* env, jobject thisPtr)
{
    return 0;
}

/*
 * Class:     NamedPipeInputStream
 * Method:    readBytes
 * Signature: ([BII)I
 */
JNIEXPORT jint JNICALL Java_NamedPipeInputStream_readBytes
  (JNIEnv* env, jobject thisPtr, jbyteArray bytes, jint off, jint len)
{
    return 0;
}

/*
 * Class:     NamedPipeInputStream
 * Method:    skip
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_NamedPipeInputStream_skip
  (JNIEnv* env, jobject thisPtr, jlong len)
{
    return 0;
}
