import java.io.*;

/**
 * NamedPipeInputStream encapsulates the client-side input stream
 * for a Win32 named pipe.
 */
public class NamedPipeInputStream extends InputStream
{
    /**
     * Create a NamedPipeInputStream unconnected
     */
    public NamedPipeInputStream()
    { }

    /**
     * Create and open(name);
     */
    public NamedPipeInputStream(String name)
        throws IOException
    {
        open(name);
    }
    
    /**
     * Connect to the Win32 named pipe called "\\.\pipe\<name>"
     */
    public native void open(String name)
        throws IOException;

    /**
     * Close the connection
     */    
    public native void close();
    
    /**
     * Reads a byte of data from this input stream. This method blocks
     * if no input is yet available.
     */
    public native int read() 
        throws IOException;


    /**
     * Reads a subarray as a sequence of bytes.
     */
    private native int readBytes(byte b[], int off, int len) 
        throws IOException;

    /**
     * Reads up to <code>b.length</code> bytes of data from this input
     * stream into an array of bytes. This method blocks until some input
     * is available.
     */
    public int read(byte b[]) 
        throws IOException 
    {
        return readBytes(b, 0, b.length);
    }

    /**
     * Reads up to <code>len</code> bytes of data from this input stream
     * into an array of bytes. This method blocks until some input is
     * available.
     */
    public int read(byte b[], int off, int len)
        throws IOException 
    {
        return readBytes(b, off, len);
    }

    /**
     * Skips over and discards <code>n</code> bytes of data from the
     * input stream. The <code>skip</code> method may, for a variety of
     * reasons, end up skipping over some smaller number of bytes,
     * possibly <code>0</code>. The actual number of bytes skipped is returned.
     */
    public native long skip(long n) 
        throws IOException;

    /**
     * Returns the number of bytes that can be read from this file input
     * stream without blocking.
     */
    public native int available() 
        throws IOException;

    protected void finalize()
    {
        if (m_nativePtr != 0)
            close();
    }
    

    // Internal (native) methods
    //
    private static native void initIDs();
    
    // Internal data
    //
    private int m_nativePtr;
    
    static
    {
        System.loadLibrary("NamedPipe");
        initIDs();
    }
}    