#include <jni.h>

// StdC++ headers
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

// Namespaces in use
using namespace std;

int main(int argc, char* argv[])
{
    vector<string> jvm_argv;
    string classname;
    string classpath;
    bool verbose = false;

    // We must have reached the classfile
    classname = argv[1];

    // The remainder of the arguments are assumed to be command-line
    // parameters to the Java class
    int j;
    for (j=2;j<argc;j++)
        jvm_argv.push_back(argv[j]);


    JavaVMInitArgs vm_args;
    JavaVMOption options[2]; int n=0;

    options[n++].optionString = "-Djava.compiler=NONE";
        /* disable JIT */
    options[n++].optionString = "-Djava.class.path=.";
        /* user classes */
    //options[n++].optionString = "-Djava.library.path=c:\mylibs";  
        /* set native library path */
    //options[n++].optionString = "-verbose:jni";                   
        /* print JNI-related messages */

    vm_args.version = JNI_VERSION_1_2;
    vm_args.options = options;
    vm_args.nOptions = n;
    vm_args.ignoreUnrecognized = true;

    /* Note that in JDK 1.2, there is no longer any need to call 
    * JNI_GetDefaultJavaVMInitArgs.  */
    JavaVM* vm;
    JNIEnv* env;
    jint res = JNI_CreateJavaVM(&vm, (void **)&env, &vm_args);

    // argv[1] is the Java class to load
    jclass cls = env->FindClass(argv[1]);
    if (!cls)
    {
        cout << argv[1] << ".class not found" << endl;
        return -1;
    }
    jmethodID cls_main = env->GetStaticMethodID(cls, "main", "([Ljava/lang/String;)V");
    if (!cls_main)
    {
        cout << "main(String[]) not found" << endl;
        return -1;
    }

    // Build args to main()
    jvalue call_args[1];                // One argument to main: "java.lang.String[] args"

    jclass cls_java_lang_String = env->FindClass("java/lang/String");
    jobject defaultValue = env->NewStringUTF("");
    jobjectArray args = env->NewObjectArray(argc-1, cls_java_lang_String, defaultValue);

    // Populate "args"
    int i=0;
    for (vector<string>::iterator iter = jvm_argv.begin(); 
        iter != jvm_argv.end();
        iter++)
    {
        jstring str = env->NewStringUTF((*iter).c_str());
        env->SetObjectArrayElement(args, i++, str);
    }
    call_args[0].l = args;

    //
    // Call "main"
    //
    env->CallStaticVoidMethodA(cls, cls_main, call_args);

    //
    // Report any uncaught Java exceptions thrown from within the JVM
    //
    jthrowable ex;
    if ((ex = env->ExceptionOccurred()) != NULL)
    {
        env->ExceptionDescribe();
    }

    //
    // Clean up
    //
    vm->DestroyJavaVM();

    return 0;
}    