#include <jvmdi.h>

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
using namespace std;


// Routine to indent method call output appropriately
//
static char scratchPad[1024];
char* makePadding(int padding)
{
   for (int i=0; i<padding; i++)
   {
     scratchPad[i] = ' ';
   }
   scratchPad[i]='\0';
   
   return scratchPad;
}    

// Global JVMDI_Interface pointer
//
JVMDI_Interface_1* jvmdi;
ofstream out("trace.log");

void EvtCallback(JNIEnv* env, JVMDI_Event* event)
{
   static int padding = 0;
   
   if (event->kind == JVMDI_EVENT_VM_INIT)
   {
      cout << "JVM's alive now" << endl;
      out << "JVM's alive now" << endl << flush;
   }
   else if (event->kind == JVMDI_EVENT_VM_DEATH)
   {
      cout << "JVM's dead now" << endl;
      out << "JVM's dead now" << endl << flush;
   }
   else if (event->kind == JVMDI_EVENT_METHOD_ENTRY)
   {
      jvmdiError err;

      //jclass classPtr;
      //err = jvmdi->GetMethodDeclaringClass(event->u.frame.clazz,
      //                                     event->u.frame.method,
      //                                     &classPtr);
      
      //char* classSig;
      //err = jvmdi->GetClassSignature(event->u.frame.clazz, &classSig);
      //jvmdi->Deallocate((jbyte*)classSig);

      //char* classSource;
      //err2 = jvmdi->GetSourceFileName(classPtr, &classSource);
      //jvmdi->Deallocate((jbyte*)classSource);
         
      char* methodName;
      char* signature;
      err = jvmdi->GetMethodName(event->u.frame.clazz, event->u.frame.method, 
         &methodName, &signature);

      padding += 2;
      if (methodName != 0) // && classSig != 0)
      {
         cout << makePadding(padding) << "Method '" <<
            methodName << "(" << signature << ")" <<
            "' entered" << endl;
         out << makePadding(padding) << "Method '" <<
            methodName << "(" << signature << ")" <<
            "' entered" << endl << flush;
      }
      else
      {
         cout << makePadding(padding) << "Method entered; err = " <<
            err << endl;
         out << makePadding(padding) << "Method entered; err = " <<
            err << endl << flush;
      }
      
      jvmdi->Deallocate((jbyte*)methodName);
      jvmdi->Deallocate((jbyte*)signature);
   }
   else if (event->kind == JVMDI_EVENT_METHOD_EXIT)
   {
      jvmdiError err;
      char* methodName;
      char* signature;
      err = jvmdi->GetMethodName(event->u.frame.clazz,
         event->u.frame.method, 
         &methodName,
         &signature);
      
      if (methodName != 0)
      {
         cout << makePadding(padding) << "Method '" << 
            methodName << "(" << signature << ")" <<
            "' exited" << endl;
         out << makePadding(padding) << "Method '" <<
            methodName << "(" << signature << ")" <<
            "' exited" << endl << flush;
      }
      else
      {
         cout << makePadding(padding) << "Method entered; err = " <<
            err << endl;
         out << makePadding(padding) << "Method entered; err = " <<
            err << endl << flush;
      }
      padding -= 2;
   }
}    

extern "C" JNIEXPORT jint JNICALL JVM_OnLoad(JavaVM *vm, char *options, void *reserved)
{
   vm->GetEnv((void**)&jvmdi, JVMDI_VERSION_1);
   
   jvmdi->SetEventHook(EvtCallback);
   jvmdi->SetEventNotificationMode(JVMDI_ENABLE, JVMDI_EVENT_METHOD_ENTRY, NULL);
   jvmdi->SetEventNotificationMode(JVMDI_ENABLE, JVMDI_EVENT_METHOD_EXIT, NULL);
   
   return JNI_VERSION_1_2;
}