#include "com_javageeks_net_NamedPipe.h"

#include <windows.h>

#include <iostream>
using namespace std;

/*
 * Class:     com_javageeks_net_NamedPipe
 * Method:    nativeOpen
 * Signature: (Ljava/lang/String;II)I
 */
JNIEXPORT jint JNICALL Java_com_javageeks_net_NamedPipe_nativeOpen
    (JNIEnv* env, jclass, jstring pipeName, jint mode, jint timeOut)
{
    int debug = (getenv("_JAVAGEEKS_DEBUG") != 0);

    // Convert from Java to UTF-8; we're in trouble if we ever use
    // Unicode as a pipeName, but I don't want to deal with
    // Unicode-to-ASCII conversions right now
    const char* c_pipeName = env->GetStringUTFChars(pipeName, NULL);
    if (debug)
    {
        cout << "Java_com_javageeks_net_NamedPipe_nativeConstruct: "
             << "c_pipeName = " << c_pipeName << endl;
    }

    DWORD fileMode;
    if (mode==com_javageeks_net_NamedPipe_DUPLEX)
    {
        fileMode = GENERIC_READ | GENERIC_WRITE;
        if (debug)
        {
            cout << "fileMode r/w" << endl;
        }
    }
    else if (mode==com_javageeks_net_NamedPipe_READ)
    {
        fileMode = GENERIC_READ;
        if (debug)
        {
            cout << "fileMode r" << endl;
        }
    }
    else if (mode==com_javageeks_net_NamedPipe_WRITE)
    {
        fileMode = GENERIC_WRITE;
        if (debug)
        {
            cout << "fileMode w" << endl;
        }
    }
    else
    {
        // Uh-oh; we didn't expect this
        if (debug)
        {
            cout << "What is mode " << mode << "???" << endl;
        }
    }

    // Make the call
    HANDLE hPipe = ::CreateFile(c_pipeName, fileMode,
                                0, NULL, OPEN_EXISTING, 0, NULL);
    // Check for busy
    if (hPipe == INVALID_HANDLE_VALUE)
    {
        if (::GetLastError() == ERROR_PIPE_BUSY)
        {
            // Wait up to the timeout parameter; after that, it's
            // a failed connect and return
            if (::WaitNamedPipe(c_pipeName, timeOut))
            {
                hPipe = ::CreateFile(c_pipeName, fileMode, 0, NULL,
                                     OPEN_EXISTING, 0, NULL);
            }
        }
    }

    if (hPipe == INVALID_HANDLE_VALUE)
    {
        DWORD error = ::GetLastError();

        if (debug)
        {
            cout << "hndl == INVALID_HANDLE_VALUE; "
                 << "ErrNo: " << ::GetLastError() << endl;
        }

        hPipe = (void*)-1;
    }

    if (debug)
    {
        cout << "hPipe = " << (void*)((HANDLE)hPipe) << endl;
    }

    // Release
    env->ReleaseStringUTFChars(pipeName, c_pipeName);

    return (jint)hPipe;
}

/*
 * Class:     com_javageeks_net_NamedPipe
 * Method:    nativeClose
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_com_javageeks_net_NamedPipe_nativeClose
    (JNIEnv* env, jclass cls, jint namedPipe)
{
    int debug = (getenv("_JAVAGEEKS_DEBUG") != 0);

    HANDLE hPipe = (HANDLE)namedPipe;

    if (::CloseHandle(hPipe))
    {
        return JNI_TRUE;
    }
    else
    {
        if (debug)
        {
            cout << "ConnectNamedPipe failed; error "
                 << ::GetLastError() << endl;
        }

        return JNI_FALSE;
    }
}

