#include "com_javageeks_net_NamedPipeOutputStream.h"

#include <windows.h>

#include <iostream>
using namespace std;

/*
 * Class:     com_javageeks_net_NamedPipeOutputStream
 * Method:    nativeFlush
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_com_javageeks_net_NamedPipeOutputStream_nativeFlush
    (JNIEnv* env, jclass, jint pipe)
{
    int debug = (getenv("_JAVAGEEKS_DEBUG") != 0);

    HANDLE hPipe = (HANDLE)pipe;

    if (debug)
    {
        cout << "Entered nativeFlush(); hPipe = " << (void*)hPipe << endl;
    }

    ::FlushFileBuffers(hPipe);
}

/*
 * Class:     com_javageeks_net_NamedPipeOutputStream
 * Method:    nativeWrite
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_com_javageeks_net_NamedPipeOutputStream_nativeWrite__II
    (JNIEnv* env, jclass, jint pipe, jint ch)
{
    int debug = (getenv("_JAVAGEEKS_DEBUG") != 0);

    HANDLE hPipe = (HANDLE)pipe;
    
    if (debug)
    {
        cout << "Entered nativeWrite__II(); hPipe = " << (void*)hPipe << endl;
    }

    // Horribly inefficient, but what can you do? Without implementing
    // some kind of buffering scheme, this is always going to be high
    // on overhead and low on throughput
    DWORD arrayLength = 1;
    CHAR* sendArray = new CHAR[arrayLength + 1];
    sendArray[0] = (CHAR)ch;
    sendArray[1] = '\0';
    
    if (debug)
    {
        cout << "Sending: '" << sendArray << "'" << endl;
    }

    DWORD cbWritten;

    // Do the Write
    BOOL success = ::WriteFile(hPipe, sendArray, 1,
                               &cbWritten, NULL);

    if (debug)
    {
        cout << cbWritten << " characters written" << endl;
    }
    
    if (!success)
    {
        if (debug)
        {
            cout << "ERROR: WriteFile failed: " << ::GetLastError()
                 << endl;
        }
    }
}

/*
 * Class:     com_javageeks_net_NamedPipeOutputStream
 * Method:    nativeWrite
 * Signature: (I[B)V
 */
JNIEXPORT void JNICALL Java_com_javageeks_net_NamedPipeOutputStream_nativeWrite__I_3B
    (JNIEnv* env, jclass, jint pipe, jbyteArray bytes)
{
    int debug = (getenv("_JAVAGEEKS_DEBUG") != 0);
    if (debug)
    {
        cout << "Entering NamedPipeOutputStream_nativeWrite__I_3B;" 
             << " hPipe = " << (void*)((HANDLE)pipe) << endl;
    }

    HANDLE hPipe = (HANDLE)pipe;

    // Convert jbyteArray to char*
    DWORD arrayLength = env->GetArrayLength(bytes);
    CHAR* sendArray = new CHAR[arrayLength];
    env->GetByteArrayRegion(bytes, 0, arrayLength, (jbyte*)sendArray);
    
    if (debug)
    {
        cout << "Sending: '" << sendArray << "'" << endl;
    }

    DWORD cbWritten;

    // Do the Write
    BOOL success = ::WriteFile(hPipe, sendArray, arrayLength + 1,
                               &cbWritten, NULL);

    if (debug)
    {
        cout << cbWritten << " characters written" << endl;
    }
    
    if (!success)
    {
        if (debug)
        {
            cout << "ERROR: WriteFile failed: " << ::GetLastError()
                 << endl;
        }
    }
}

