package com.javageeks.client;

import java.io.*;
import java.net.*;

/**
 * This thread reads from the socket and writes the bytes received to the
 * console window.
 */
class SocketToConsoleThread extends Thread
{
    public SocketToConsoleThread(Socket s)
    {
        m_socket = s;

        setDaemon(true);

        // Necessary to work around some platforms        
        setPriority(Thread.currentThread().getPriority()+1);
    }

    public void run()
    {
        try
        {
            // Set up the necessary Reader from the Socket
            Reader fromSocket = new InputStreamReader(m_socket.getInputStream());
            // Set up the necessary Writer to the Console window
            Writer toUser = new PrintWriter(new OutputStreamWriter(System.out));
            
            int c;
            char[] buffer = new char[1024];
            while ((c = fromSocket.read(buffer)) != -1)
            {
                toUser.write(buffer, 0, c);
                toUser.flush();
            }
        }
        catch (java.io.IOException ioEx)
        {
            ioEx.printStackTrace();
            return;
        }
    }
    
    Socket m_socket;
}    

/**
 * This thread reads from the console window and writes the received keystrokes
 * to the socket.
 */
class ConsoleToSocketThread extends Thread
{
    public ConsoleToSocketThread(Socket s)
    {
        m_socket = s;
        
        setDaemon(true);
    }

    public void run()
    {
        try
        {
            BufferedReader fromUser = 
                new BufferedReader(new InputStreamReader(System.in));
            PrintWriter toSocket = 
                new PrintWriter(new OutputStreamWriter(m_socket.getOutputStream()));
            
            String line;
            while ((line = fromUser.readLine()) != null)
            {
                toSocket.println(line);
                toSocket.flush();
            }
        }
        catch (java.io.IOException ioEx)
        {
            ioEx.printStackTrace();
            return;
        }
    }
    
    Socket m_socket;
}    

/**
 * Client: connect to <host> on port <port>
 */
public class Client
{
    public static void main(String[] args)
        throws Exception
    {
        if (args.length < 1)
        {
            System.out.println("Usage: java Client <hostname:port>");
            return;
        }
        
        // Parse out hostname and port
        String host;
        Integer port;
        host = args[0].substring(0, args[0].indexOf(":"));
        port = new Integer(args[0].substring(args[0].indexOf(":")+1, args[0].length()));
        
        System.out.println("Connecting to " + host + ":" + port);
        
        Socket socket = new Socket(host, port.intValue());
        
        SocketToConsoleThread s2c = new SocketToConsoleThread(socket);
        s2c.start();
        ConsoleToSocketThread c2s = new ConsoleToSocketThread(socket);
        c2s.start();
        
        s2c.join();
    }
}    