package com.javageeks.client;


import java.io.*;
import java.util.Date;


/**
 *
 */
public class OutputStreamListener
    implements HeartbeatListener
{
    // Internal members
    //
    private PrintWriter m_writer;
    private boolean m_verbose = false;


    /**
     * Constructor
     */
    public OutputStreamListener(OutputStream out, boolean verbose)
        throws java.io.IOException
    {
        m_writer = new PrintWriter(out);
        m_verbose = verbose;
    }
    /**
     * Constructor
     */
    public OutputStreamListener(PrintStream outStream, boolean verbose)
        throws java.io.IOException
    {
        m_writer = new PrintWriter(outStream);
        m_verbose = verbose;
    }
    /**
     * Constructor
     */
    public OutputStreamListener(PrintWriter writer, boolean verbose)
        throws java.io.IOException
    {
        m_writer = writer;
        m_verbose = verbose;
    }


    /**
     * Return the verbosity of the Listener; if set to true, the
     * LogListener will write out onHeartbeatPing messages to the log
     */
    public boolean getVerbose()
    {
        return m_verbose;
    }
    /**
     * Set the verbosity of the Listener; if set to true, the
     * LogListener will write out onHeartbeatPing messages to the log
     */
    public void setVerbose(boolean verbose)
    {
        m_verbose = verbose;
    }


    /**
     * HeartbeatListener method. Received each time a 'ping' is
     * received from the HeartbeatService
     */
    public void onHeartbeatPing(String msg)
    {
        if (m_verbose)
        {
            m_writer.println(new Date() + ": " + msg + " received.");
            m_writer.flush();
        }
    }
    /**
     * HeartbeatListener method. Called when the 
     * HeartbeatListenerClient determines the heartbeat has failed
     */
    public void onHeartbeatFail(String msg)
    {
        m_writer.println(new Date() + ": ***Heartbeat failure: " + 
                         msg);
        m_writer.flush();
    }
}
