package com.javageeks.gjas;

import java.rmi.RemoteException;

/**
 * RMIServer wraps around another IServer (usually a LocalServer instance)
 * to provide RMI-access.
 */
public class RMIServer
    implements IServer
{
    public RMIServer(IRemoteServerManager svrMgr, long ID)
    {
        m_ID = ID;
        m_svrMgr = svrMgr;
    }

    /**
     * Start the wrapped Service instance. Services have 15 seconds in which to
     * either initialize, or else start a thread to perform the necessary
     * initialization and return. If a Service fails to respond within 15
     * seconds of the start of its <code>start</code> call, the Server and/or
     * ServerManager are free to destroy it.
     */
    public boolean start()
    {
        try
        {
            return m_svrMgr.start(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
    /**
     * Stop the wrapped Service instance; as with <code>start</code>, the Service
     * gets 15 seconds to stop itself before the ServerManager is free to take
     * more drastic steps.
     */
    public boolean stop()
    {
        try
        {
            return m_svrMgr.stop(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
    /**
     * Pauses the wrapped Service. The Service should respond within 15 seconds of
     * the start of this call; however, failure to do so is not sufficient grounds
     * for the ServerManager or Server to destroy it.
     */
    public boolean pause()
    {
        try
        {
            return m_svrMgr.pause(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
    /**
     * Resumes the wrapped Service. The Service should respond within 15 seconds of
     * the start of this call; however, failure to do so is not sufficient grounds
     * for the ServerManager or Server to destroy it.
     */
    public boolean resume()
    {
        try
        {
            return m_svrMgr.resume(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
    /**
     * Kills the wrapped Service.
     */
    public void kill()
    {
        try
        {
            m_svrMgr.kill(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
        

    /**
     * Returns the state of the wrapped Service.
     */
    public String getState()
    {
        try
        {
            return m_svrMgr.getState(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
        

    /**
     * Returns the instance ID of the wrapped Service.
     */
    public String getInstanceID()
    {
        try
        {
            return m_svrMgr.getInstanceID(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
        
        
    /**
     * Returns the last Exception thrown, if any, by the wrapped Service.
     */
    public Exception getLastError()
    {
        try
        {
            return m_svrMgr.getLastError(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }


    /**
     * Returns the ConfigurationInfo instance to use for configuration
     */
    public ConfigProperties getConfigInfo()
    {
        try
        {
            return m_svrMgr.getConfigInfo(m_ID);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }
    /**
     * Set the ConfigurationInfo instance for this Service
     */
    public void setConfigInfo(ConfigProperties info)
    {
        try
        {
            m_svrMgr.setConfigInfo(m_ID, info);
        }
        catch (RemoteException remoteEx)
        {
            throw new RuntimeException(remoteEx.toString());
        }
    }


    // Internal data
    //
    private IRemoteServerManager m_svrMgr;
    private long m_ID;
}
