package com.javageeks.gjas;

import java.util.Properties;

/**
 * Note: Service's Serializable interface should be honored,
 * because Serialization is the basic means of exchange between
 * JVMs in RMI calls, and if a Service is not Serializable then it
 * cannot be transferred across JVMs.
 *
 * <P> If a Service needs to maintain "interim" data that should not
 * be Serialized, then remember to mark the data members as
 * <code>transient</code>. Also, remember that a given Serializable
 * class can control what happens when it is serialized and
 * deserialized by means of thewriteObject and readObject methods.
 * This would allow, for example, those Services that make use of
 * JDBC Connections (as an example) to close down and reopen the
 * Connection upon serialization and subsequent deserialization.
 */
public interface Service extends java.io.Serializable
{
    /**
     * Start the Service.
     */
    public void start()
        throws Exception;
    /**
     * Stop the Service.
     */
    public void stop()
        throws Exception;
    /**
     * Pause the Service.
     */
    public void pause()
        throws Exception;
    /**
     * Resume the Service.
     */
    public void resume()
        throws Exception;
        

    /**
     * Get the current state of the Service; must be one of the
     * following types: STOPPED, STARTING, RUNNING, STOPPING,
     * PAUSING, PAUSED, or RESUMING.
     */
    public String getState();
    public static final String STOPPED = "STOPPED";
    public static final String STARTING = "STARTING";
    public static final String RUNNING = "RUNNING";
    public static final String STOPPING = "STOPPING";
    public static final String PAUSING = "PAUSING";
    public static final String PAUSED = "PAUSED";
    public static final String RESUMING = "RESUMING";


    /**
     * Return a String uniquely identifying this instance of the
     * Service; this String must be unique not just to the Service
     * class, but to the Service instance itself. Suggested return
     * format is something like:
     *
     * <code>String instanceID = this.getClass().getName() + ":" + 
     * getClassVersion() + ":" + System.currentTimeMillis();</code>
     *
     * Note that maintaining an "instance count" of the number of
     * instances of this class will fail, since all instances will
     * be maintained within their own ClassLoader, and static
     * members are stored on a per-ClassLoader basis.
     */
    public String getInstanceID()
        throws Exception;

    /**
     * Return the Properties configuration information
     */
    public ConfigProperties getConfigInfo();
    /**
     * Set the Properties configuration information
     */
    public void setConfigInfo(ConfigProperties info);
}
