package com.javageeks.gjas.connection;

import java.net.Socket;
import com.javageeks.gjas.ConfigProperties;
import com.javageeks.gjas.ConfigProperty;
import com.javageeks.gjas.ServerManager;
import com.javageeks.io.ThreadedPipeStream;


/**
 *
 */
public class RedirectConnection
    implements Connection
{
    /**
     *
     */
    public void serve(Socket socket, ConfigProperties configInfo)
        throws Exception
    {
        // Pull the host & port to redirect to
        String toHost =
            (String)configInfo.get("redirHost").getValue();
        Integer toPort =
            (Integer)configInfo.get("redirPort").getValue();

        if (toHost == null)
        {
            throw new IllegalArgumentException("redirHost " +
                "cannot be left empty; terminating connection");
        }
        if (toPort == null)
        {
            throw new IllegalArgumentException("redirPort " +
                "cannot be left empty; terminating connection");
        }

        // Open a Socket to the recipient system
        ServerManager.log(getClass().getName() + ": Attempting " +
            "to connect to server " + toHost + ":" + toPort);
        Socket redirSocket = new Socket(toHost, toPort.intValue());
        ServerManager.log(getClass().getName() + ": Success");

        // Pipe the Streams
        ThreadedPipeStream clientToRedir = 
            new ThreadedPipeStream(socket.getInputStream(), 
                                   redirSocket.getOutputStream());
        clientToRedir.getThread().start();
        ThreadedPipeStream redirToClient =
            new ThreadedPipeStream(redirSocket.getInputStream(),
                                   socket.getOutputStream());
        redirToClient.getThread().start();

        // Wait for client input to stop, then return
        clientToRedir.getThread().join();
    
        ServerManager.log(getClass().getName() + ": " +
            "Redirection complete");
    }

    public static ConfigProperties getConfigInfo()
    {
        return new ConfigProperties(new ConfigProperty[]
        {
            new ConfigProperty("redirHost", new String(""),
                               "Host to redirect requests to"),
            new ConfigProperty("redirPort", new Integer(0),
                               "Port on host to redirect to")
        });
    }
}
