package com.javageeks.gjas.connection;

import java.net.Socket;
import java.io.*;
import com.javageeks.gjas.ConfigProperties;
import com.javageeks.gjas.ServerManager;

/**
 *
 */
public class SocketClassLoaderConnection
    implements Connection
{
    /**
     *
     */
    public void serve(Socket socket, ConfigProperties configInfo)
        throws Exception
    {
        ServerManager.log(
            getClass().getName() + ".serve(): Entered");

        // ConfigProperties tell us the file root to use; otherwise,
        // we presume we use "user.dir" as the root
        String root = System.getProperty("user.dir");
        if (root == null)
        {
            root = ".";
        }   
        if (configInfo.get("root") != null)
        {
            root = (String)configInfo.get("root").getValue();
        }

        // Get our in/out streams
        InputStream in = socket.getInputStream();
        OutputStream out = socket.getOutputStream();

        BufferedReader br = 
            new BufferedReader(new InputStreamReader(in));
        PrintWriter pw = new PrintWriter(out);

        // Get the classname
        String classname = br.readLine();
        classname =
            classname.substring(classname.indexOf(":")+1,
                                classname.length());

        ServerManager.log(
            getClass().getName() + ".serve(): " + 
            "Request: " + classname);

        // Convert the classname to a file path
        String classfilename =
            classname.replace('.', '/');
        ServerManager.log(
            getClass().getName() + ".serve(): " + 
            "File name should be " + classfilename);

        
        // Find the file on the disk
        try
        {
            FileInputStream inFile =
                new FileInputStream(root + "/" + 
                                    classfilename + ".class");

            // Tell SocketClassLoader how much data to expect
            pw.println("Content-Length:" + inFile.available());
            pw.flush();

            // We have to use OutputStream directly here, because Writer
            // and PrintWriter have no methods to write out bytes
            byte[] buffer = new byte[1024];
            int nRead = 0;
            while ((nRead = inFile.read(buffer)) != -1)
                out.write(buffer, 0, nRead);
            out.flush();

            // Write one more blank line, just for luck
            pw.write("\r\n");
        }
        catch (FileNotFoundException ex)
        {
            ServerManager.log(
                getClass().getName() + ".serve(): " +
                "Error finding class file");
            ServerManager.log(ex);
            
            pw.println("Error");
        }
        finally
        {
            pw.flush();
        }
    }
}    