package com.javageeks.io;

/**
 * Class designed to "tee" output to multiple OutputStream objects
 *
 * @version 1.0
 * @author Ted Neward
 */
public class TeeOutputStream 
    extends java.io.OutputStream
{
	/**
	 * Builds a TeeOutputStream with no "captued" streams
	 */
	public TeeOutputStream()
	{ }
	
	/**
	 * Builds a TeeOutputStream with the streams passed already
	 * "captured" by the TeeOutputStream
	 */
	public TeeOutputStream(java.io.OutputStream streams[])
	{
		for (int i=0; i<streams.length; i++)
			addOutputStream(streams[i]);
	}
	
	/**
	 * Adds the OutputStream passed into the list of tee'ed streams
	 */
	public void addOutputStream(java.io.OutputStream os)
	{
		m_outputStreamList.addElement(os);
	}
	
	/**
	 * Writes a character to each of the OutputStreams "captured"
	 */
	public void write(int b) 
		throws java.io.IOException
	{
		for (int i=0; i<m_outputStreamList.size(); i++)
		{
			java.io.OutputStream os =
				(java.io.OutputStream)m_outputStreamList.elementAt(i);
			os.write(b);
		}
	}
	
	/**
	 * Calls flush() on each of the OutputStreams "captured"
	 */
	public void flush()
		throws java.io.IOException
	{
		for (int i=0; i<m_outputStreamList.size(); i++)
		{
			java.io.OutputStream os =
				(java.io.OutputStream)m_outputStreamList.elementAt(i);
			os.flush();
		}
	}

	/**
	 * Closes all of the OutputStreams "captured"
	 */
	public void close()
		throws java.io.IOException
	{
		for (int i=0; i<m_outputStreamList.size(); i++)
		{
			java.io.OutputStream os =
				(java.io.OutputStream)m_outputStreamList.elementAt(i);
			os.close();
		}
	}
	
	private java.util.Vector m_outputStreamList = new java.util.Vector(5,1);
}
