package com.javageeks.io;

import java.io.*;
import java.net.Socket;

/**
 * ThreadedPipeStream: reads from an InputStream, sends the received
 * data down the given OutputStream.
 */
public class ThreadedPipeStream
    implements Runnable
{
    // Internal data
    private Thread m_thread;
    private InputStream m_from;
    private OutputStream m_to;
    private Exception m_lastException;


    /**
     *
     */
    public ThreadedPipeStream(InputStream in, 
                              OutputStream out,
                              Thread thread)
    {
        m_thread = thread;
        m_from = in;
        m_to = out;
    }
    /**
     *
     */
    public ThreadedPipeStream(InputStream in, 
                              OutputStream out)
    {
        m_from = in;
        m_to = out;
    
        // Create our Thread
        m_thread = new Thread(this);
        m_thread.setDaemon(true);
        m_thread.setName(toString());
        m_thread.setPriority(Thread.currentThread().getPriority()+1);
    }


    /**
     * Convenience method to save from having to reimplement all the
     * Thread methods: start(), stop(), resume(), interrupt(), and
     * so forth. Should probably put those methods in here at some
     * point.
     */
    public Thread getThread()
    {
        return m_thread;
    }


    /**
     *
     */
    public String toString()
    {
        return new String("ThreadedPipeStream:" +
                          m_from.toString() + ":" + 
                          m_to.toString());
    }


    /**
     *
     */
    public void run()
    {
        try
        {
            // Set up the necessary Reader from the Socket
            Reader from = 
                new InputStreamReader(m_from);
            // Set up the necessary Writer to the Console window
            Writer to = 
                new PrintWriter(m_to);
            
            int c;
            char[] buffer = new char[1024];
            while ((c = from.read(buffer)) != -1)
            {
                to.write(buffer, 0, c);
                to.flush();
            }
        }
        catch (IOException ioEx)
        {
            m_lastException = ioEx;
            ioEx.printStackTrace();
        }
    }


    /**
     *
     */
    public static void main (String args[])
        throws Exception
    {
        if (args.length < 1)
        {
            System.out.println("Usage: java " + 
                "com.javageeks.io.ThreadedPipeStream <hostname:port>");
            return;
        }
        
        // Parse out hostname and port
        String host;
        Integer port;
        host = args[0].substring(0, args[0].indexOf(":"));
        port = new Integer(args[0].substring(args[0].indexOf(":")+1, args[0].length()));
        
        System.out.println("Connecting to " + host + ":" + port);
        
        Socket socket = new Socket(host, port.intValue());
        
        ThreadedPipeStream s2c = 
            new ThreadedPipeStream(socket.getInputStream(), System.out);
        s2c.getThread().start();
        ThreadedPipeStream c2s =
            new ThreadedPipeStream(System.in, socket.getOutputStream());
        c2s.getThread().start();

        s2c.getThread().join();
    }
}
