package com.javageeks.net;

import java.io.*;

/**
 * Receive input from an NT named pipe (presumably with INBOUND
 * mode set on it). Can only be obtained from a NamedPipe instance;
 * cannot be instantiated on its own.
 */
public class NamedPipeInputStream extends InputStream
{
    // Internal members
    //
    private NamedPipe m_pipe;


    /**
     * Package-friendly constructor. Used solely by NamedPipe.
     */
    /*package-friendly*/ NamedPipeInputStream(NamedPipe pipe)
    {
        m_pipe = pipe;
    }


    /**
     *
     */
    public int available()
    {
        return 0;
    }
    /**
     *
     */
    public void close()
    {
        // Do nothing--the named pipe may still be open in outbound
        // mode, so we don't want to close it
    }
    /**
     *
     */
    public boolean markSupported()
    {
        return false;
    }
    /**
     *
     */
    public int read(byte[] b)
    {
        return nativeRead(m_pipe.m_hPipe, b);
    }
    /**
     *
     */
    public int read()
    {
        return nativeRead(m_pipe.m_hPipe);
    }


    // Native methods
    //
    private static native void nativeAvailable(int hPipe);
    private static native int nativeRead(int hPipe, byte[] b);
    private static native int nativeRead(int hPipe);
}
