import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Properties;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;


/**
 * Class which exports as both a UnicastRemoteObject
 * and a PortableRemoteObject
 */
public class CombinedNameServer
    implements INameServer
{
    /**
     *
     */
    public CombinedNameServer()
        throws RemoteException
    {
        // Export as an IIOP object
        //
        PortableRemoteObject.exportObject(this);
        
        // Export as a JRMP object
        //
        UnicastRemoteObject.exportObject(this);
    }
    
    
    /**
     *
     */
    public boolean bindIIOP()
        throws Exception
    {
        // Set up the InitialContext factory for JNDI
        //
        Properties iiopProps = new Properties();
        iiopProps.setProperty("java.naming.factory.initial",
                              "com.sun.jndi.cosnaming.CNCtxFactory");
        
        // Set up the JNDI Naming provider URL
        //
        iiopProps.setProperty("java.naming.provider.url",
                              "iiop://localhost:900");
        
        // 
        // 
        Context ctx = new InitialContext(iiopProps);
        INameServer ns = this;
        ctx.rebind(INameServer.RMI_BINDING_NAME, ns);

        System.out.println("CombinedNameServer bound (IIOP) as '" +
                           INameServer.RMI_BINDING_NAME + "'");
                           
        return true;
    }
    /**
     *
     */
    public boolean bindJRMP()
        throws Exception
    {
        // 
        // 
        INameServer ns = this;
        
        // Use standard RMI-Registry/Naming code to bind the
        // server object
        //
        Naming.bind(INameServer.RMI_BINDING_NAME, ns);
        System.out.println("CombinedNameServer bound (JRMP) as '" +
                           INameServer.RMI_BINDING_NAME + "'");
        
        // Could also use JNDI-JRMP using this code:
        //
        /*
        Properties jrmpProps = new Properties();
        jrmpProps.setProperty("java.naming.factory.initial",
            "com.sun.jndi.rmi.registry.RegistryContextFactory");
        
        Context ctx = new InitialContext(jrmpProps);
        ctx.rebind(INameServer.RMI_BINDING_NAME, ns);
        System.out.println("CombinedNameServer bound (JRMP) as '" +
                           INameServer.RMI_BINDING_NAME + "'");
        */

        return true;
    }
	
	
    /**
     *
     */
    public String generateName()
        throws RemoteException
    {
        return "Fred";
    }


    /**
     *
     */
    public static void main (String args[])
        throws Exception
    {
        CombinedNameServer cns = new CombinedNameServer();
        
        if (cns.bindIIOP() && cns.bindJRMP())
        {
            System.out.println("Ready for both IIOP and JRMP");
        }
    }
}