import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

/**
 * RemoteObjectFactoryClient
 */
public class RemoteObjectFactoryClient
{
    /**
     * RemoteObjectFactoryClient constructor
     */
    public RemoteObjectFactoryClient(String host, int port)
    {
        m_host = host;
        m_port = port;
    }
    
    /**
     *
     */
    public Object construct(Constructor ctor, Object[] ctorArgs)
        throws Exception
    {
        // Transform ctorArgs into a Vector for easier Serialization
        Vector args = new Vector();
        if (ctorArgs != null)
        {
            for (int i=0; i<ctorArgs.length; i++)
                args.addElement(ctorArgs[i]);
        }

        Socket socket = new Socket(m_host, m_port);            
        ObjectInputStream objIn = 
            new ObjectInputStream(socket.getInputStream());
        ObjectOutputStream objOut = 
            new ObjectOutputStream(socket.getOutputStream());
            // Note that these streams must be constructed in
            // reverse order from how they are constructed in
            // the server class; see the code in RemoteObjectService
            // for more details
            
        objOut.writeUTF(ctor.getDeclaringClass().getName());
        objOut.writeUTF(ctor.toString());
        objOut.writeObject(args);
        objOut.flush();
        
        // Read back the response string--SUCCESS or ERROR
        String response = objIn.readUTF();
        if (response.equals("SUCCESS"))
        {
            // Next object is our serialized object; deserialize
            // and return it
            Object obj = objIn.readObject();
            return obj;
        }
        else
        {
            // Next object is an Exception; deserialize and throw it
            Exception ex = (Exception)objIn.readObject();
            throw ex;
        }
    }
    
    
    // Internal members
    String m_host;
    int m_port;
    
    
    /**
     * Test driver
     */
    public static void main(String[] args)
        throws Exception
    {
        if (args.length < 1)
        {
            System.out.println(
                "Usage: java RemoteObjectFactoryClient <hostname:port>");
            return;
        }
        
        // Parse out hostname and port
        String host;
        Integer port;
        host = args[0].substring(0, args[0].indexOf(":"));
        port = new Integer(args[0].substring(
            args[0].indexOf(":")+1, args[0].length()));
        
        System.out.println("Connecting to " + host + ":" + port);

        RemoteObjectFactoryClient client =
            new RemoteObjectFactoryClient(host, port.intValue());
        
        // Try constructing a few well-known Java objects
        
        // new Date()
        Class dateClass = Class.forName("java.util.Date");
        Constructor dateDefaultCtor =
            dateClass.getConstructor(null);
        Date date = (Date)client.construct(dateDefaultCtor, null);
        System.out.println("new Date(): " + date);
        
        // new String(String)
        Class stringClass = Class.forName("java.lang.String");
        Constructor stringCtor =
            stringClass.getConstructor(new Class[] 
            {
                Class.forName("java.lang.String")
            });
        String str =
            (String)client.construct(stringCtor, new Object[] 
            {
                new String("Test value")
            });
        System.out.println("new String(String): " + str);
    }
}    
