import java.io.*;
import java.util.*;

public class CustomSerialization
    implements Serializable
{
    public CustomSerialization()
    {
        m_int = 5;
        m_string = "This is a test";
        m_object = new Date();
    }
    
    private void writeObject(ObjectOutputStream oos)
        throws IOException
    {
        // One possible implementation
        /*
        oos.writeUTF("{BEGIN}");
        oos.writeInt(m_int);
        oos.writeUTF(m_string);
        oos.writeObject(m_object);
        oos.writeUTF("{END}");
        */
        
        // A second implementation
        Hashtable hash = new Hashtable();
        hash.put("m_int", new Integer(m_int));
        hash.put("m_string", m_string);
        hash.put("m_object", m_object);
        oos.writeObject(hash);
    }
    private void readObject(ObjectInputStream ois)
        throws IOException, ClassNotFoundException
    {
        // The deserialization to the implementation given first
        /*
        System.out.println((String)ois.readUTF());
        m_int = ois.readInt();
        m_string = ois.readUTF();
        m_object = ois.readObject();
        System.out.println((String)ois.readUTF());
        */
        
        // The deserialization to the second implementation
        Hashtable hash = (Hashtable)ois.readObject();
        m_int = ((Integer)hash.get("m_int")).intValue();
        m_string = (String)hash.get("m_string");
        m_object = hash.get("m_object");
    }
    
    public int m_int;
    public String m_string;
    public Object m_object;
    
    public static void main(String[] args)
        throws Exception
    {
        CustomSerialization custom = new CustomSerialization();
        custom.m_int = 12;
        custom.m_string = "Test data";
        custom.m_object = new Vector();
        ((Vector)custom.m_object).addElement("One");
        ((Vector)custom.m_object).addElement("Two");
        ((Vector)custom.m_object).addElement("Three");
        
        // Serialize it off to disk
        FileOutputStream os = 
            new FileOutputStream("data");
        ObjectOutputStream oos =
            new ObjectOutputStream(os);
        oos.writeObject(custom);
        oos.close();
        
        // Retrieve it from disk
        FileInputStream is =
            new FileInputStream("data");
        ObjectInputStream ois =
            new ObjectInputStream(is);
        CustomSerialization test =
            (CustomSerialization)ois.readObject();
            
        if (custom.m_int == test.m_int &&
            custom.m_string.equals(test.m_string))
        {
            System.out.println("It worked!");
        }
    }
}    