import java.io.*;

/**
 * OldClass is the "old" serialized format we wish to maintain
 */
class OldClass
    implements Serializable
{
    public OldClass(String data)
    {
        m_data = data;
    }
    
    public String getData()
    {
        return m_data;
    }
    
    public String toString()
    {
        return ("OldClass.m_data = " + m_data);
    }
    
    private String m_data;
    
    // Serialization Replacement method
    private Object readResolve()
        throws ObjectStreamException
    {
        return new NewClass(this);
    }
}

/**
 * NewClass is the "new" serialized format we wish to replace
 * OldClass with.
 */
class NewClass
    implements Serializable
{
    public NewClass(OldClass source)
    {
        System.out.println("Copy-constructing from " + source);
        m_ref = source;
        m_additionalData = null;
    }
    
    private Object m_additionalData;
    private OldClass m_ref;
        // This is counted as an incompatible change as far as
        // Serialization is concerned--we "removed" a serialized
        // field (m_data) from this new class type, we changed the
        // class name, and so forth.
    
    private Object writeReplace()
        throws ObjectStreamException
    {
        System.out.println("Nominating " + m_ref + " for " +
            "serialization instead of ourselves");
        return m_ref;
    }
}

/**
 *
 */
public class Replacement
{
    public static void main(String[] args)
        throws Exception
    {
        // Serialize out a version of OldClass to work with
        ByteArrayOutputStream out1 =
            new ByteArrayOutputStream();
        ObjectOutputStream objOut =
            new ObjectOutputStream(out1);
        OldClass data = new OldClass("Data value");
        objOut.writeObject(data);
        objOut.flush();
        
        // Deserialize written object into a NewClass instance
        ByteArrayInputStream in1 =
            new ByteArrayInputStream(out1.toByteArray());
        ObjectInputStream objIn =
            new ObjectInputStream(in1);
        NewClass newData = (NewClass)objIn.readObject();
        
        // Serialize NewClass instance back out to file
        ByteArrayOutputStream out2 =
            new ByteArrayOutputStream();
        ObjectOutputStream objOut2 =
            new ObjectOutputStream(out2);
        objOut2.writeObject(newData);
        objOut2.flush();
        
        // Ensure the two byte arras (out1 and out2) are identical;
        // for this example, we'll just compare sizes
        if (out1.toByteArray().length == out2.toByteArray().length)
            System.out.println("Lengths are identical!");
    }
}    