import java.io.*;
import java.util.zip.*;

class SimpleFilterInputStream extends FilterInputStream
{
    public SimpleFilterInputStream(InputStream in)
    {
        super(in);
    }

    /**
     * Offset the byte values by -1
     */
    public int read()
        throws IOException
    {
        int r = super.read();
    	return r--;
    }

    /**
     * Offset the byte values by -1
     */
    public int read(byte b[])
        throws IOException
    {
        int ret = super.read(b, 0, b.length);
        
        for (int i=0; i<b.length; i++)
            b[i]--;
            
    	return ret;
    }

    /**
     * Offset the byte values by +1
     */
    public int read(byte b[], int off, int len)
        throws IOException
    {
    	int ret = in.read(b, off, len);
        
        for (int i=0; i<b.length; i++)
            b[i]--;
            
    	return ret;
    }
}

class SimpleFilterOutputStream extends FilterOutputStream
{
    /**
     *
     */
    public SimpleFilterOutputStream(OutputStream out)
    {
    	super(out);
    }

    /**
     * Offset the bytes by +1
     */
    public void write(int b)
        throws IOException
    {
    	out.write(b++);
    }

    /**
     * Offset the bytes by +1
     */
    public void write(byte b[])
        throws IOException 
    {
    	write(b, 0, b.length);
    }

    /**
     * Offset the bytes by +1
     */
    public void write(byte b[], int off, int len)
        throws IOException 
    {
    	for (int i = 0 ; i < len ; i++)
        {
    	    write((b[off + i])++);
    	}
    }
}

/**
 *
 */
public class SecureSerialization
{
    /**
     *
     */
    public static void main(String[] args)
        throws Exception
    {
        // Take a String, serialize it compressed to disk
        String data = "This is our test string";

        FileOutputStream os = 
            new FileOutputStream("data");
        SimpleFilterOutputStream fos =
            new SimpleFilterOutputStream(os);
        /*
        GZIPOutputStream gzOS =
            new GZIPOutputStream(os);
        */
        ObjectOutputStream oos =
            new ObjectOutputStream(fos); //gzOS);
        
        // Serialize the object
        oos.writeObject(data);
        oos.close();
        
        // Set up the input streams
        FileInputStream is =
            new FileInputStream("data");
        SimpleFilterInputStream fis =
            new SimpleFilterInputStream(is);
        /*
        GZIPInputStream gzIS =
            new GZIPInputStream(is);
         */
        ObjectInputStream ois =
            new ObjectInputStream(fis); //gzIS);
            
        String test = (String)ois.readObject();
        
        System.out.println("Compare: " + data + " vs. " + test);
        if (data.equals(test))
            System.out.println("Success!");
    }
}    