import java.io.*;
import java.net.*;

/**
 *
 */
public class SocketClassLoader extends ClassLoader
{
    /**
     *
     */
    public SocketClassLoader(String host, int port)
    {
        m_host = host;
        m_port = port;
    }
    
    /**
     *
     */
    public Class findClass(String className)
        throws ClassNotFoundException
    {
        try
        {
            // Connect to the host on port
            Socket socket = new Socket(m_host, m_port);

            BufferedReader reader =
                new BufferedReader(new InputStreamReader(socket.getInputStream()));
            PrintWriter writer =
                new PrintWriter(socket.getOutputStream());
            
            // Send the classname
            writer.println("Classname:" + className);
            writer.flush();
            
            // Get back the resulting bytecode, or get nothing back (an error)
            String line = reader.readLine();
            if (line.equals("Error"))
                throw new ClassNotFoundException("Host returned error");
            else if (line.startsWith("Content-Length"))
            {
                // Find out how much we're expecting back
                int colonLoc = line.indexOf(":");
                Integer l = 
                    new Integer(line.substring(colonLoc + 1, line.length()));
                byte[] classBytes = new byte[l.intValue()];
                
                // Throw away any data between our current point in the stream
                // and the first magic number of the Java .class file ('CA')
                while (reader.read() != (int)0xCA)
                    ;

                // We already pulled back the first magic number of the class,
                // so manually insert it into the byte array. Read the rest
                // from the socket
                classBytes[0] = (byte)0xCA;
                for (int i=1; i<classBytes.length; i++)
                    classBytes[i] = (byte)reader.read();

                return defineClass(className, classBytes, 0, classBytes.length);
            }
            else
                throw new ClassNotFoundException("Host returned unknown response");
        }
        catch (UnknownHostException uhEx)
        {
            throw new ClassNotFoundException("Unknown host: " + uhEx.toString());
        }
        catch (IOException ioEx)
        {
            throw new ClassNotFoundException("Socket error: " + ioEx.toString());
        }
    }

    // Internal members    
    //
    String m_host;
    int m_port;
    

    // To test this effectively from the CD, copy SocketClassLoader.class to a
    // directory elsewhere on your hard drive. Start GJAS in the chap4 directory,
    // then run SocketClassLoader from the command-line.    
    public static void main(String[] args)
        throws Exception
    {
        // If Hello.class exists in the current directory, the bootstrap
        // ClassLoader, which is always given first crack, will pick it up and
        // load the class, instead of the SocketClassLoader.
        File file = new File("Hello.class");
        if (file.exists())
            System.out.println("Warning--Hello.class exists " +
                "in the current directory. SocketClassLoader will NOT be used " +
                "to retrieve the file; the primordial ClassLoader will.");

        // Connect to the local host on port 8085 to see if Hello can be
        // loaded.    
        SocketClassLoader scl = new SocketClassLoader("localhost", 8085);
        Class cls = scl.loadClass("Hello");
        Object h = cls.newInstance();
    }
}    